﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class BMAP
    {
        private unsafe NativeBMAP* BMAPPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public BMAP(IntPtr param0)
        {
            unsafe
            {
                BMAPPointer = (NativeBMAP*)param0;
            }
        }

        /// <summary>
        /// flags
        /// </summary>
        public enum FLAGS : int
        {
            BMPF_ARGB = 1 << 8,
            BMPF_SAVE = 1 << 9,
            BMAP_MIPMAPS = 1 << 10
        }

        /// <summary>
        /// flags - see Gamestudio manual
        /// </summary>
        public int flags
        {
            get
            {
                unsafe
                {
                    return (*BMAPPointer).flags;
                }
            }
            set
            {
                unsafe
                {
                    (*BMAPPointer).flags = value;
                }
            }
        }

        /// <summary>
        /// enable_render
        /// </summary>
        public Boolean BMPF_ARGB
        {
            get
            {
                unsafe
                {
                    if (((*BMAPPointer).flags & (int)FLAGS.BMPF_ARGB) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*BMAPPointer).flags = (*BMAPPointer).flags | (int)FLAGS.BMPF_ARGB;
                    else
                        (*BMAPPointer).flags = (*BMAPPointer).flags & ~(int)FLAGS.BMPF_ARGB;

                }
            }
        }

        /// <summary>
        /// enable_render
        /// </summary>
        public Boolean BMPF_SAVE
        {
            get
            {
                unsafe
                {
                    if (((*BMAPPointer).flags & (int)FLAGS.BMPF_SAVE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*BMAPPointer).flags = (*BMAPPointer).flags | (int)FLAGS.BMPF_SAVE;
                    else
                        (*BMAPPointer).flags = (*BMAPPointer).flags & ~(int)FLAGS.BMPF_SAVE;

                }
            }
        }

        /// <summary>
        /// enable_render
        /// </summary>
        public Boolean BMAP_MIPMAPS
        {
            get
            {
                unsafe
                {
                    if (((*BMAPPointer).flags & (int)FLAGS.BMAP_MIPMAPS) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*BMAPPointer).flags = (*BMAPPointer).flags | (int)FLAGS.BMAP_MIPMAPS;
                    else
                        (*BMAPPointer).flags = (*BMAPPointer).flags & ~(int)FLAGS.BMAP_MIPMAPS;

                }
            }
        }

        /* -------- a7.BMAP -------- */
        /* a7.BMAP to a7.NativeBMAP* (address in void*) */
        public static explicit operator IntPtr(BMAP d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativeBMAP*)d.BMAPPointer;
                }
            }
        }
        /* a7.NativeBMAP* (address in void*) to a7.BMAP */
        public static explicit operator BMAP(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new BMAP(d);
            }
        }
    }
}