﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class D3DVERTEX
    {
        private unsafe NativeD3DVERTEX* D3DVERTEXPointer;
        
        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public D3DVERTEX(IntPtr param0)
        {
            unsafe
            {
                D3DVERTEXPointer = (NativeD3DVERTEX*)param0;
            }
        }

        /// <summary>
        /// x - see Gamestudio manual
        /// </summary>
        public float x
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).x;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).x = value;
                }
            }
        }


        /// <summary>
        /// y - see Gamestudio manual
        /// </summary>
        public float y
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).y;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).y = value;
                }
            }
        }


        /// <summary>
        /// z - see Gamestudio manual
        /// </summary>
        public float z
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).z;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).z = value;
                }
            }
        }

        /// <summary>
        /// xyz - not listed in the Gamestudio manual - allows to access x, y and z as a Vector
        /// </summary>
        private Vector xyz_ = null;
        public Vector xyz
        {
            get
            {
                unsafe
                {
                    if (xyz_ == null)
                    {
                        xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        xyz_.x = (Var)(IntPtr)(&((*D3DVERTEXPointer).x));
                        xyz_.y = (Var)(IntPtr)(&((*D3DVERTEXPointer).y));
                        xyz_.z = (Var)(IntPtr)(&((*D3DVERTEXPointer).z));
                    }
                }
                return xyz_;
            }
        }


        /// <summary>
        /// nx - see Gamestudio manual
        /// </summary>
        public float nx
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).nx;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).nx = value;
                }
            }
        }


        /// <summary>
        /// ny - see Gamestudio manual
        /// </summary>
        public float ny
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).ny;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).ny = value;
                }
            }
        }


        /// <summary>
        /// nz - see Gamestudio manual
        /// </summary>
        public float nz
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).nz;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).nz = value;
                }
            }
        }

        /// <summary>
        /// nxnynz - not listed in the Gamestudio manual - allows to access nx, ny and nz as a Vector
        /// </summary>
        private Vector nxnynz_ = null;
        public Vector nxnynz
        {
            get
            {
                unsafe
                {
                    if (nxnynz_ == null)
                    {
                        nxnynz_ = new Vector((Var)0, (Var)0, (Var)0);
                        nxnynz_.x = (Var)(IntPtr)(&((*D3DVERTEXPointer).nx));
                        nxnynz_.y = (Var)(IntPtr)(&((*D3DVERTEXPointer).ny));
                        nxnynz_.z = (Var)(IntPtr)(&((*D3DVERTEXPointer).nz));
                    }
                }
                return nxnynz_;
            }
        }


        /// <summary>
        /// u1 - see Gamestudio manual
        /// </summary>
        public float u1
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).u1;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).u1 = value;
                }
            }
        }


        /// <summary>
        /// v1 - see Gamestudio manual
        /// </summary>
        public float v1
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).v1;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).v1 = value;
                }
            }
        }


        /// <summary>
        /// u2 - see Gamestudio manual
        /// </summary>
        public float u2
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).u2;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).u2 = value;
                }
            }
        }


        /// <summary>
        /// v2 - see Gamestudio manual
        /// </summary>
        public float v2
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).v2;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).v2 = value;
                }
            }
        }


        /// <summary>
        /// x3 - see Gamestudio manual
        /// </summary>
        public float x3
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).x3;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).x3 = value;
                }
            }
        }


        /// <summary>
        /// y3 - see Gamestudio manual
        /// </summary>
        public float y3
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).y3;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).y3 = value;
                }
            }
        }


        /// <summary>
        /// z3 - see Gamestudio manual
        /// </summary>
        public float z3
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).z3;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).z3 = value;
                }
            }
        }

        /// <summary>
        /// x3y3z3 - not listed in the Gamestudio manual - allows to access x3, y3 and z3 as a Vector
        /// </summary>
        private Vector x3y3z3_ = null;
        public Vector x3y3z3
        {
            get
            {
                unsafe
                {
                    if (x3y3z3_ == null)
                    {
                        x3y3z3_ = new Vector((Var)0, (Var)0, (Var)0);
                        x3y3z3_.x = (Var)(IntPtr)(&((*D3DVERTEXPointer).x3));
                        x3y3z3_.y = (Var)(IntPtr)(&((*D3DVERTEXPointer).y3));
                        x3y3z3_.z = (Var)(IntPtr)(&((*D3DVERTEXPointer).z3));
                    }
                }
                return x3y3z3_;
            }
        }


        /// <summary>
        /// w3 - see Gamestudio manual
        /// </summary>
        public float w3
        {
            get
            {
                unsafe
                {
                    return (*D3DVERTEXPointer).w3;
                }
            }
            set
            {
                unsafe
                {
                    (*D3DVERTEXPointer).w3 = value;
                }
            }
        }

        /* -------- a7.D3DVERTEX -------- */
        /* a7.D3DVERTEX to a7.NativeD3DVERTEX* (address in void*) */
        public static explicit operator IntPtr(D3DVERTEX d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativeD3DVERTEX*)d.D3DVERTEXPointer;
                }
            }
        }
        /* a7.NativeD3DVERTEX* (address in void*) to a7.D3DVERTEX */
        public static explicit operator D3DVERTEX(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new D3DVERTEX(d);
            }
        }
    }
}