﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class ENTITY
    {
        private unsafe NativeENTITY* ENTITYPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public ENTITY(IntPtr param0)
        {
            unsafe
            {
                ENTITYPointer = (NativeENTITY*)param0;
            }
        }

        /// <summary>
        /// x - see Gamestudio manual
        /// </summary>
        private Var x_ = null;
        public Var x
        {
            get
            {
                unsafe
                {
                    if (x_ == null)
                    {
                        x_ = (Var)(IntPtr)(&((*ENTITYPointer).x));
                    }
                }
                return x_;
            }
        }

        /// <summary>
        /// y - see Gamestudio manual
        /// </summary>
        private Var y_ = null;
        public Var y
        {
            get
            {
                unsafe
                {
                    if (y_ == null)
                    {
                        y_ = (Var)(IntPtr)(&((*ENTITYPointer).y));
                    }
                }
                return y_;
            }
        }

        /// <summary>
        /// z - see Gamestudio manual
        /// </summary>
        private Var z_ = null;
        public Var z
        {
            get
            {
                unsafe
                {
                    if (z_ == null)
                    {
                        z_ = (Var)(IntPtr)(&((*ENTITYPointer).z));
                    }
                }
                return z_;
            }
        }

        /// <summary>
        /// xyz - not listed in the Gamestudio manual - allows to access x, y and z as a Vector
        /// </summary>
        private Vector xyz_ = null;
        public Vector xyz
        {
            get
            {
                unsafe
                {
                    if (xyz_ == null)
                    {
                        xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        xyz_.x = (Var)(IntPtr)(&((*ENTITYPointer).x));
                        xyz_.y = (Var)(IntPtr)(&((*ENTITYPointer).y));
                        xyz_.z = (Var)(IntPtr)(&((*ENTITYPointer).z));
                    }
                }
                return xyz_;
            }
        }

        /// <summary>
        /// pan - see Gamestudio manual
        /// </summary>
        private Var pan_ = null;
        public Var pan
        {
            get
            {
                unsafe
                {
                    if (pan_ == null)
                    {
                        pan_ = (Var)(IntPtr)(&((*ENTITYPointer).pan));
                    }
                }
                return pan_;
            }
        }

        /// <summary>
        /// tilt - see Gamestudio manual
        /// </summary>
        private Var tilt_ = null;
        public Var tilt
        {
            get
            {
                unsafe
                {
                    if (tilt_ == null)
                    {
                        tilt_ = (Var)(IntPtr)(&((*ENTITYPointer).tilt));
                    }
                }
                return tilt_;
            }
        }

        /// <summary>
        /// roll - see Gamestudio manual
        /// </summary>
        private Var roll_ = null;
        public Var roll
        {
            get
            {
                unsafe
                {
                    if (roll_ == null)
                    {
                        roll_ = (Var)(IntPtr)(&((*ENTITYPointer).roll));
                    }
                }
                return roll_;
            }
        }

        /// <summary>
        /// panTiltRoll - not listed in the Gamestudio manual - allows to access pan, tilt and roll as a Vector
        /// </summary>
        private Angle panTiltRoll_ = null;
        public Angle panTiltRoll
        {
            get
            {
                unsafe
                {
                    if (panTiltRoll_ == null)
                    {
                        panTiltRoll_ = new Angle((Var)0, (Var)0, (Var)0);
                        panTiltRoll_.pan = (Var)(IntPtr)(&((*ENTITYPointer).pan));
                        panTiltRoll_.tilt = (Var)(IntPtr)(&((*ENTITYPointer).tilt));
                        panTiltRoll_.roll = (Var)(IntPtr)(&((*ENTITYPointer).roll));
                    }
                }
                return panTiltRoll_;
            }
        }

        /// <summary>
        /// scale_x - see Gamestudio manual
        /// </summary>
        private Var scale_x_ = null;
        public Var scale_x
        {
            get
            {
                unsafe
                {
                    if (scale_x_ == null)
                    {
                        scale_x_ = (Var)(IntPtr)(&((*ENTITYPointer).scale_x));
                    }
                }
                return scale_x_;
            }
        }

        /// <summary>
        /// scale_y - see Gamestudio manual
        /// </summary>
        private Var scale_y_ = null;
        public Var scale_y
        {
            get
            {
                unsafe
                {
                    if (scale_y_ == null)
                    {
                        scale_y_ = (Var)(IntPtr)(&((*ENTITYPointer).scale_y));
                    }
                }
                return scale_y_;
            }
        }

        /// <summary>
        /// scale_z - see Gamestudio manual
        /// </summary>
        private Var scale_z_ = null;
        public Var scale_z
        {
            get
            {
                unsafe
                {
                    if (scale_z_ == null)
                    {
                        scale_z_ = (Var)(IntPtr)(&((*ENTITYPointer).scale_z));
                    }
                }
                return scale_z_;
            }
        }

        /// <summary>
        /// scaleXYZ - not listed in the Gamestudio manual - allows to access scale_x, scale_y and scale_z as a Vector
        /// </summary>
        private Vector scaleXYZ_ = null;
        public Vector scaleXYZ
        {
            get
            {
                unsafe
                {
                    if (scaleXYZ_ == null)
                    {
                        scaleXYZ_ = new Vector((Var)0, (Var)0, (Var)0);
                        scaleXYZ_.x = (Var)(IntPtr)(&((*ENTITYPointer).scale_x));
                        scaleXYZ_.y = (Var)(IntPtr)(&((*ENTITYPointer).scale_y));
                        scaleXYZ_.z = (Var)(IntPtr)(&((*ENTITYPointer).scale_z));
                    }
                }
                return scaleXYZ_;
            }
        }

        /// <summary>
        /// layer - see Gamestudio manual
        /// </summary>
        private Var layer_ = null;
        public Var layer
        {
            get
            {
                unsafe
                {
                    if (layer_ == null)
                    {
                        layer_ = (Var)(IntPtr)(&((*ENTITYPointer).layer));
                    }
                }
                return layer_;
            }
        }

        /// <summary>
        /// vmask - see Gamestudio manual
        /// </summary>
        public int vmask
        {
            get
            {
                unsafe
                {
                    return (*ENTITYPointer).vmask;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).vmask = value;
                }
            }
        }

        /// <summary>
        /// type - see Gamestudio manual
        /// </summary>
        public String type
        {
            get
            {
                unsafe
                {
                    return Marshal.PtrToStringAnsi((IntPtr)(*ENTITYPointer).type);
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).type = (char*)(void*)Marshal.StringToHGlobalAnsi(value);
                }
            }
        }

        /// <summary>
        /// alpha - see Gamestudio manual
        /// </summary>
        private Var alpha_ = null;
        public Var alpha
        {
            get
            {
                unsafe
                {
                    if (alpha_ == null)
                    {
                        alpha_ = (Var)(IntPtr)(&((*ENTITYPointer).alpha));
                    }
                }
                return alpha_;
            }
        }

        /// <summary>
        /// ambient - see Gamestudio manual
        /// </summary>
        private Var ambient_ = null;
        public Var ambient
        {
            get
            {
                unsafe
                {
                    if (ambient_ == null)
                    {
                        ambient_ = (Var)(IntPtr)(&((*ENTITYPointer).ambient));
                    }
                }
                return ambient_;
            }
        }

        /// <summary>
        /// lightrange - see Gamestudio manual
        /// </summary>
        private Var lightrange_ = null;
        public Var lightrange
        {
            get
            {
                unsafe
                {
                    if (lightrange_ == null)
                    {
                        lightrange_ = (Var)(IntPtr)(&((*ENTITYPointer).lightrange));
                    }
                }
                return lightrange_;
            }
        }

        /// <summary>
        /// red - see Gamestudio manual
        /// </summary>
        private Var red_ = null;
        public Var red
        {
            get
            {
                unsafe
                {
                    if (red_ == null)
                    {
                        red_ = (Var)(IntPtr)(&((*ENTITYPointer).red));
                    }
                }
                return red_;
            }
        }

        /// <summary>
        /// green - see Gamestudio manual
        /// </summary>
        private Var green_ = null;
        public Var green
        {
            get
            {
                unsafe
                {
                    if (green_ == null)
                    {
                        green_ = (Var)(IntPtr)(&((*ENTITYPointer).green));
                    }
                }
                return green_;
            }
        }

        /// <summary>
        /// blue - see Gamestudio manual
        /// </summary>
        private Var blue_ = null;
        public Var blue
        {
            get
            {
                unsafe
                {
                    if (blue_ == null)
                    {
                        blue_ = (Var)(IntPtr)(&((*ENTITYPointer).blue));
                    }
                }
                return blue_;
            }
        }

        /// <summary>
        /// greenBlueRed - not listed in the Gamestudio manual - allows to access scale_x, scale_y and scale_z as a Vector
        /// </summary>
        private Color greenBlueRed_ = null;
        public Color greenBlueRed
        {
            get
            {
                unsafe
                {
                    if (greenBlueRed_ == null)
                    {
                        greenBlueRed_ = new Color((Var)0, (Var)0, (Var)0);
                        greenBlueRed_.red = (Var)(IntPtr)(&((*ENTITYPointer).red));
                        greenBlueRed_.green = (Var)(IntPtr)(&((*ENTITYPointer).green));
                        greenBlueRed_.blue = (Var)(IntPtr)(&((*ENTITYPointer).blue));
                    }
                }
                return greenBlueRed_;
            }
        }

        /// <summary>
        /// frame - see Gamestudio manual
        /// </summary>
        private Var frame_ = null;
        public Var frame
        {
            get
            {
                unsafe
                {
                    if (frame_ == null)
                    {
                        frame_ = (Var)(IntPtr)(&((*ENTITYPointer).frame));
                    }
                }
                return frame_;
            }
        }

        /// <summary>
        /// next_frame - see Gamestudio manual
        /// </summary>
        private Var next_frame_ = null;
        public Var next_frame
        {
            get
            {
                unsafe
                {
                    if (next_frame_ == null)
                    {
                        next_frame_ = (Var)(IntPtr)(&((*ENTITYPointer).next_frame));
                    }
                }
                return next_frame_;
            }
        }

        /// <summary>
        /// pose - see Gamestudio manual
        /// </summary>
        private Var pose_ = null;
        public Var pose
        {
            get
            {
                unsafe
                {
                    if (pose_ == null)
                    {
                        pose_ = (Var)(IntPtr)(&((*ENTITYPointer).pose));
                    }
                }
                return pose_;
            }
        }

        /// <summary>
        /// skin - see Gamestudio manual
        /// </summary>
        private Var skin_ = null;
        public Var skin
        {
            get
            {
                unsafe
                {
                    if (skin_ == null)
                    {
                        skin_ = (Var)(IntPtr)(&((*ENTITYPointer).skin));
                    }
                }
                return skin_;
            }
        }

        /// <summary>
        /// material - see Gamestudio manual
        /// </summary>
        public MATERIAL material
        {
            get
            {
                unsafe
                {
                    return (MATERIAL)(IntPtr)(*ENTITYPointer).material;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).material = (NativeMATERIAL*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// lightmap - see Gamestudio manual
        /// </summary>
        public BMAP lightmap
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*ENTITYPointer).lightmap;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).lightmap = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// shadow - see Gamestudio manual
        /// </summary>
        public ENTITY shadow
        {
            get
            {
                unsafe
                {
                    return (ENTITY)(IntPtr)(*ENTITYPointer).shadow;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).shadow = (NativeENTITY*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// min_x - see Gamestudio manual
        /// </summary>
        private Var min_x_ = null;
        public Var min_x
        {
            get
            {
                unsafe
                {
                    if (min_x_ == null)
                    {
                        min_x_ = (Var)(IntPtr)(&((*ENTITYPointer).min_x));
                    }
                }
                return min_x_;
            }
        }

        /// <summary>
        /// min_y - see Gamestudio manual
        /// </summary>
        private Var min_y_ = null;
        public Var min_y
        {
            get
            {
                unsafe
                {
                    if (min_y_ == null)
                    {
                        min_y_ = (Var)(IntPtr)(&((*ENTITYPointer).min_y));
                    }
                }
                return min_y_;
            }
        }

        /// <summary>
        /// min_z - see Gamestudio manual
        /// </summary>
        private Var min_z_ = null;
        public Var min_z
        {
            get
            {
                unsafe
                {
                    if (min_z_ == null)
                    {
                        min_z_ = (Var)(IntPtr)(&((*ENTITYPointer).min_z));
                    }
                }
                return min_z_;
            }
        }

        /// <summary>
        /// min_XYZ - not listed in the Gamestudio manual - allows to access min_x, min_y and min_z as a Vector
        /// </summary>
        private Vector min_XYZ_ = null;
        public Vector min_XYZ
        {
            get
            {
                unsafe
                {
                    if (min_XYZ_ == null)
                    {
                        min_XYZ_ = new Vector((Var)0, (Var)0, (Var)0);
                        min_XYZ_.x = (Var)(IntPtr)(&((*ENTITYPointer).min_x));
                        min_XYZ_.y = (Var)(IntPtr)(&((*ENTITYPointer).min_y));
                        min_XYZ_.z = (Var)(IntPtr)(&((*ENTITYPointer).min_z));
                    }
                }
                return min_XYZ_;
            }
        }

        /// <summary>
        /// max_x - see Gamestudio manual
        /// </summary>
        private Var max_x_ = null;
        public Var max_x
        {
            get
            {
                unsafe
                {
                    if (max_x_ == null)
                    {
                        max_x_ = (Var)(IntPtr)(&((*ENTITYPointer).max_x));
                    }
                }
                return max_x_;
            }
        }

        /// <summary>
        /// max_y - see Gamestudio manual
        /// </summary>
        private Var max_y_ = null;
        public Var max_y
        {
            get
            {
                unsafe
                {
                    if (max_y_ == null)
                    {
                        max_y_ = (Var)(IntPtr)(&((*ENTITYPointer).max_y));
                    }
                }
                return max_y_;
            }
        }

        /// <summary>
        /// max_z - see Gamestudio manual
        /// </summary>
        private Var max_z_ = null;
        public Var max_z
        {
            get
            {
                unsafe
                {
                    if (max_z_ == null)
                    {
                        max_z_ = (Var)(IntPtr)(&((*ENTITYPointer).max_z));
                    }
                }
                return max_z_;
            }
        }

        /// <summary>
        /// max_XYZ - not listed in the Gamestudio manual - allows to access max_x, max_y and max_z as a Vector
        /// </summary>
        private Vector max_XYZ_ = null;
        public Vector max_XYZ
        {
            get
            {
                unsafe
                {
                    if (max_XYZ_ == null)
                    {
                        max_XYZ_ = new Vector((Var)0, (Var)0, (Var)0);
                        max_XYZ_.x = (Var)(IntPtr)(&((*ENTITYPointer).max_x));
                        max_XYZ_.y = (Var)(IntPtr)(&((*ENTITYPointer).max_y));
                        max_XYZ_.z = (Var)(IntPtr)(&((*ENTITYPointer).max_z));
                    }
                }
                return max_XYZ_;
            }
        }

        /// <summary>
        /// push - see Gamestudio manual
        /// </summary>
        private Var push_ = null;
        public Var push
        {
            get
            {
                unsafe
                {
                    if (push_ == null)
                    {
                        push_ = (Var)(IntPtr)(&((*ENTITYPointer).push));
                    }
                }
                return push_;
            }
        }

        /// <summary>
        /// trigger_range - see Gamestudio manual
        /// </summary>
        private Var trigger_range_ = null;
        public Var trigger_range
        {
            get
            {
                unsafe
                {
                    if (trigger_range_ == null)
                    {
                        trigger_range_ = (Var)(IntPtr)(&((*ENTITYPointer).trigger_range));
                    }
                }
                return trigger_range_;
            }
        }

        /// <summary>
        /// parent - see Gamestudio manual
        /// </summary>
        public ENTITY parent
        {
            get
            {
                unsafe
                {
                    return (ENTITY)(IntPtr)(*ENTITYPointer).parent;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).parent = (NativeENTITY*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// client_id - see Gamestudio manual
        /// </summary>
        private Var client_id_ = null;
        public Var client_id
        {
            get
            {
                unsafe
                {
                    if (client_id_ == null)
                    {
                        client_id_ = (Var)(IntPtr)(&((*ENTITYPointer).client_id));
                    }
                }
                return client_id_;
            }
        }

        /// <summary>
        /// floor_dist - see Gamestudio manual
        /// </summary>
        private Var floor_dist_ = null;
        public Var floor_dist
        {
            get
            {
                unsafe
                {
                    if (floor_dist_ == null)
                    {
                        floor_dist_ = (Var)(IntPtr)(&((*ENTITYPointer).floor_dist));
                    }
                }
                return floor_dist_;
            }
        }


        /// <summary>
        /// event - see Gamestudio manual
        /// </summary>
        public EventVoid event_
        {
            set
            {
                unsafe
                {
                    Scheduler.EventVoidReference TempEventVoidReference = new Scheduler.EventVoidReference(value, (IntPtr)ENTITYPointer);
                    (*ENTITYPointer).event_.mPtr = (byte*)(void*)Marshal.GetFunctionPointerForDelegate(TempEventVoidReference.NativeEventVoidFunc);
                }
            }
        }

        /// <summary>
        /// skill1 - see Gamestudio manual
        /// </summary>
        private Var skill1_ = null;
        public Var skill1
        {
            get
            {
                unsafe
                {
                    if (skill1_ == null)
                    {
                        skill1_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_001));
                    }
                }
                return skill1_;
            }
        }

        /// <summary>
        /// skill2 - see Gamestudio manual
        /// </summary>
        private Var skill2_ = null;
        public Var skill2
        {
            get
            {
                unsafe
                {
                    if (skill2_ == null)
                    {
                        skill2_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_002));
                    }
                }
                return skill2_;
            }
        }

        /// <summary>
        /// skill3 - see Gamestudio manual
        /// </summary>
        private Var skill3_ = null;
        public Var skill3
        {
            get
            {
                unsafe
                {
                    if (skill3_ == null)
                    {
                        skill3_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_003));
                    }
                }
                return skill3_;
            }
        }

        /// <summary>
        /// skill4 - see Gamestudio manual
        /// </summary>
        private Var skill4_ = null;
        public Var skill4
        {
            get
            {
                unsafe
                {
                    if (skill4_ == null)
                    {
                        skill4_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_004));
                    }
                }
                return skill4_;
            }
        }

        /// <summary>
        /// skill5 - see Gamestudio manual
        /// </summary>
        private Var skill5_ = null;
        public Var skill5
        {
            get
            {
                unsafe
                {
                    if (skill5_ == null)
                    {
                        skill5_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_005));
                    }
                }
                return skill5_;
            }
        }

        /// <summary>
        /// skill6 - see Gamestudio manual
        /// </summary>
        private Var skill6_ = null;
        public Var skill6
        {
            get
            {
                unsafe
                {
                    if (skill6_ == null)
                    {
                        skill6_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_006));
                    }
                }
                return skill6_;
            }
        }

        /// <summary>
        /// skill7 - see Gamestudio manual
        /// </summary>
        private Var skill7_ = null;
        public Var skill7
        {
            get
            {
                unsafe
                {
                    if (skill7_ == null)
                    {
                        skill7_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_007));
                    }
                }
                return skill7_;
            }
        }

        /// <summary>
        /// skill8 - see Gamestudio manual
        /// </summary>
        private Var skill8_ = null;
        public Var skill8
        {
            get
            {
                unsafe
                {
                    if (skill8_ == null)
                    {
                        skill8_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_008));
                    }
                }
                return skill8_;
            }
        }

        /// <summary>
        /// skill9 - see Gamestudio manual
        /// </summary>
        private Var skill9_ = null;
        public Var skill9
        {
            get
            {
                unsafe
                {
                    if (skill9_ == null)
                    {
                        skill9_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_009));
                    }
                }
                return skill9_;
            }
        }

        /// <summary>
        /// skill10 - see Gamestudio manual
        /// </summary>
        private Var skill10_ = null;
        public Var skill10
        {
            get
            {
                unsafe
                {
                    if (skill10_ == null)
                    {
                        skill10_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_010));
                    }
                }
                return skill10_;
            }
        }

        /// <summary>
        /// skill11 - see Gamestudio manual
        /// </summary>
        private Var skill11_ = null;
        public Var skill11
        {
            get
            {
                unsafe
                {
                    if (skill11_ == null)
                    {
                        skill11_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_011));
                    }
                }
                return skill11_;
            }
        }

        /// <summary>
        /// skill12 - see Gamestudio manual
        /// </summary>
        private Var skill12_ = null;
        public Var skill12
        {
            get
            {
                unsafe
                {
                    if (skill12_ == null)
                    {
                        skill12_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_012));
                    }
                }
                return skill12_;
            }
        }

        /// <summary>
        /// skill13 - see Gamestudio manual
        /// </summary>
        private Var skill13_ = null;
        public Var skill13
        {
            get
            {
                unsafe
                {
                    if (skill13_ == null)
                    {
                        skill13_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_013));
                    }
                }
                return skill13_;
            }
        }

        /// <summary>
        /// skill14 - see Gamestudio manual
        /// </summary>
        private Var skill14_ = null;
        public Var skill14
        {
            get
            {
                unsafe
                {
                    if (skill14_ == null)
                    {
                        skill14_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_014));
                    }
                }
                return skill14_;
            }
        }

        /// <summary>
        /// skill15 - see Gamestudio manual
        /// </summary>
        private Var skill15_ = null;
        public Var skill15
        {
            get
            {
                unsafe
                {
                    if (skill15_ == null)
                    {
                        skill15_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_015));
                    }
                }
                return skill15_;
            }
        }

        /// <summary>
        /// skill16 - see Gamestudio manual
        /// </summary>
        private Var skill16_ = null;
        public Var skill16
        {
            get
            {
                unsafe
                {
                    if (skill16_ == null)
                    {
                        skill16_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_016));
                    }
                }
                return skill16_;
            }
        }

        /// <summary>
        /// skill17 - see Gamestudio manual
        /// </summary>
        private Var skill17_ = null;
        public Var skill17
        {
            get
            {
                unsafe
                {
                    if (skill17_ == null)
                    {
                        skill17_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_017));
                    }
                }
                return skill17_;
            }
        }

        /// <summary>
        /// skill18 - see Gamestudio manual
        /// </summary>
        private Var skill18_ = null;
        public Var skill18
        {
            get
            {
                unsafe
                {
                    if (skill18_ == null)
                    {
                        skill18_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_018));
                    }
                }
                return skill18_;
            }
        }

        /// <summary>
        /// skill19 - see Gamestudio manual
        /// </summary>
        private Var skill19_ = null;
        public Var skill19
        {
            get
            {
                unsafe
                {
                    if (skill19_ == null)
                    {
                        skill19_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_019));
                    }
                }
                return skill19_;
            }
        }

        /// <summary>
        /// skill20 - see Gamestudio manual
        /// </summary>
        private Var skill20_ = null;
        public Var skill20
        {
            get
            {
                unsafe
                {
                    if (skill20_ == null)
                    {
                        skill20_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_020));
                    }
                }
                return skill20_;
            }
        }

        /// <summary>
        /// skill21 - see Gamestudio manual
        /// </summary>
        private Var skill21_ = null;
        public Var skill21
        {
            get
            {
                unsafe
                {
                    if (skill21_ == null)
                    {
                        skill21_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_021));
                    }
                }
                return skill21_;
            }
        }

        /// <summary>
        /// skill22 - see Gamestudio manual
        /// </summary>
        private Var skill22_ = null;
        public Var skill22
        {
            get
            {
                unsafe
                {
                    if (skill22_ == null)
                    {
                        skill22_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_022));
                    }
                }
                return skill22_;
            }
        }

        /// <summary>
        /// skill23 - see Gamestudio manual
        /// </summary>
        private Var skill23_ = null;
        public Var skill23
        {
            get
            {
                unsafe
                {
                    if (skill23_ == null)
                    {
                        skill23_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_023));
                    }
                }
                return skill23_;
            }
        }

        /// <summary>
        /// skill24 - see Gamestudio manual
        /// </summary>
        private Var skill24_ = null;
        public Var skill24
        {
            get
            {
                unsafe
                {
                    if (skill24_ == null)
                    {
                        skill24_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_024));
                    }
                }
                return skill24_;
            }
        }

        /// <summary>
        /// skill25 - see Gamestudio manual
        /// </summary>
        private Var skill25_ = null;
        public Var skill25
        {
            get
            {
                unsafe
                {
                    if (skill25_ == null)
                    {
                        skill25_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_025));
                    }
                }
                return skill25_;
            }
        }

        /// <summary>
        /// skill26 - see Gamestudio manual
        /// </summary>
        private Var skill26_ = null;
        public Var skill26
        {
            get
            {
                unsafe
                {
                    if (skill26_ == null)
                    {
                        skill26_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_026));
                    }
                }
                return skill26_;
            }
        }

        /// <summary>
        /// skill27 - see Gamestudio manual
        /// </summary>
        private Var skill27_ = null;
        public Var skill27
        {
            get
            {
                unsafe
                {
                    if (skill27_ == null)
                    {
                        skill27_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_027));
                    }
                }
                return skill27_;
            }
        }

        /// <summary>
        /// skill28 - see Gamestudio manual
        /// </summary>
        private Var skill28_ = null;
        public Var skill28
        {
            get
            {
                unsafe
                {
                    if (skill28_ == null)
                    {
                        skill28_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_028));
                    }
                }
                return skill28_;
            }
        }

        /// <summary>
        /// skill29 - see Gamestudio manual
        /// </summary>
        private Var skill29_ = null;
        public Var skill29
        {
            get
            {
                unsafe
                {
                    if (skill29_ == null)
                    {
                        skill29_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_029));
                    }
                }
                return skill29_;
            }
        }

        /// <summary>
        /// skill30 - see Gamestudio manual
        /// </summary>
        private Var skill30_ = null;
        public Var skill30
        {
            get
            {
                unsafe
                {
                    if (skill30_ == null)
                    {
                        skill30_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_030));
                    }
                }
                return skill30_;
            }
        }

        /// <summary>
        /// skill31 - see Gamestudio manual
        /// </summary>
        private Var skill31_ = null;
        public Var skill31
        {
            get
            {
                unsafe
                {
                    if (skill31_ == null)
                    {
                        skill31_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_031));
                    }
                }
                return skill31_;
            }
        }

        /// <summary>
        /// skill32 - see Gamestudio manual
        /// </summary>
        private Var skill32_ = null;
        public Var skill32
        {
            get
            {
                unsafe
                {
                    if (skill32_ == null)
                    {
                        skill32_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_032));
                    }
                }
                return skill32_;
            }
        }

        /// <summary>
        /// skill33 - see Gamestudio manual
        /// </summary>
        private Var skill33_ = null;
        public Var skill33
        {
            get
            {
                unsafe
                {
                    if (skill33_ == null)
                    {
                        skill33_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_033));
                    }
                }
                return skill33_;
            }
        }

        /// <summary>
        /// skill34 - see Gamestudio manual
        /// </summary>
        private Var skill34_ = null;
        public Var skill34
        {
            get
            {
                unsafe
                {
                    if (skill34_ == null)
                    {
                        skill34_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_034));
                    }
                }
                return skill34_;
            }
        }

        /// <summary>
        /// skill35 - see Gamestudio manual
        /// </summary>
        private Var skill35_ = null;
        public Var skill35
        {
            get
            {
                unsafe
                {
                    if (skill35_ == null)
                    {
                        skill35_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_035));
                    }
                }
                return skill35_;
            }
        }

        /// <summary>
        /// skill36 - see Gamestudio manual
        /// </summary>
        private Var skill36_ = null;
        public Var skill36
        {
            get
            {
                unsafe
                {
                    if (skill36_ == null)
                    {
                        skill36_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_036));
                    }
                }
                return skill36_;
            }
        }

        /// <summary>
        /// skill37 - see Gamestudio manual
        /// </summary>
        private Var skill37_ = null;
        public Var skill37
        {
            get
            {
                unsafe
                {
                    if (skill37_ == null)
                    {
                        skill37_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_037));
                    }
                }
                return skill37_;
            }
        }

        /// <summary>
        /// skill38 - see Gamestudio manual
        /// </summary>
        private Var skill38_ = null;
        public Var skill38
        {
            get
            {
                unsafe
                {
                    if (skill38_ == null)
                    {
                        skill38_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_038));
                    }
                }
                return skill38_;
            }
        }

        /// <summary>
        /// skill39 - see Gamestudio manual
        /// </summary>
        private Var skill39_ = null;
        public Var skill39
        {
            get
            {
                unsafe
                {
                    if (skill39_ == null)
                    {
                        skill39_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_039));
                    }
                }
                return skill39_;
            }
        }

        /// <summary>
        /// skill40 - see Gamestudio manual
        /// </summary>
        private Var skill40_ = null;
        public Var skill40
        {
            get
            {
                unsafe
                {
                    if (skill40_ == null)
                    {
                        skill40_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_040));
                    }
                }
                return skill40_;
            }
        }

        /// <summary>
        /// skill41 - see Gamestudio manual
        /// </summary>
        private Var skill41_ = null;
        public Var skill41
        {
            get
            {
                unsafe
                {
                    if (skill41_ == null)
                    {
                        skill41_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_041));
                    }
                }
                return skill41_;
            }
        }

        /// <summary>
        /// skill42 - see Gamestudio manual
        /// </summary>
        private Var skill42_ = null;
        public Var skill42
        {
            get
            {
                unsafe
                {
                    if (skill42_ == null)
                    {
                        skill42_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_042));
                    }
                }
                return skill42_;
            }
        }

        /// <summary>
        /// skill43 - see Gamestudio manual
        /// </summary>
        private Var skill43_ = null;
        public Var skill43
        {
            get
            {
                unsafe
                {
                    if (skill43_ == null)
                    {
                        skill43_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_043));
                    }
                }
                return skill43_;
            }
        }

        /// <summary>
        /// skill44 - see Gamestudio manual
        /// </summary>
        private Var skill44_ = null;
        public Var skill44
        {
            get
            {
                unsafe
                {
                    if (skill44_ == null)
                    {
                        skill44_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_044));
                    }
                }
                return skill44_;
            }
        }

        /// <summary>
        /// skill45 - see Gamestudio manual
        /// </summary>
        private Var skill45_ = null;
        public Var skill45
        {
            get
            {
                unsafe
                {
                    if (skill45_ == null)
                    {
                        skill45_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_045));
                    }
                }
                return skill45_;
            }
        }

        /// <summary>
        /// skill46 - see Gamestudio manual
        /// </summary>
        private Var skill46_ = null;
        public Var skill46
        {
            get
            {
                unsafe
                {
                    if (skill46_ == null)
                    {
                        skill46_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_046));
                    }
                }
                return skill46_;
            }
        }

        /// <summary>
        /// skill47 - see Gamestudio manual
        /// </summary>
        private Var skill47_ = null;
        public Var skill47
        {
            get
            {
                unsafe
                {
                    if (skill47_ == null)
                    {
                        skill47_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_047));
                    }
                }
                return skill47_;
            }
        }

        /// <summary>
        /// skill48 - see Gamestudio manual
        /// </summary>
        private Var skill48_ = null;
        public Var skill48
        {
            get
            {
                unsafe
                {
                    if (skill48_ == null)
                    {
                        skill48_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_048));
                    }
                }
                return skill48_;
            }
        }

        /// <summary>
        /// skill49 - see Gamestudio manual
        /// </summary>
        private Var skill49_ = null;
        public Var skill49
        {
            get
            {
                unsafe
                {
                    if (skill49_ == null)
                    {
                        skill49_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_049));
                    }
                }
                return skill49_;
            }
        }

        /// <summary>
        /// skill50 - see Gamestudio manual
        /// </summary>
        private Var skill50_ = null;
        public Var skill50
        {
            get
            {
                unsafe
                {
                    if (skill50_ == null)
                    {
                        skill50_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_050));
                    }
                }
                return skill50_;
            }
        }

        /// <summary>
        /// skill51 - see Gamestudio manual
        /// </summary>
        private Var skill51_ = null;
        public Var skill51
        {
            get
            {
                unsafe
                {
                    if (skill51_ == null)
                    {
                        skill51_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_051));
                    }
                }
                return skill51_;
            }
        }

        /// <summary>
        /// skill52 - see Gamestudio manual
        /// </summary>
        private Var skill52_ = null;
        public Var skill52
        {
            get
            {
                unsafe
                {
                    if (skill52_ == null)
                    {
                        skill52_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_052));
                    }
                }
                return skill52_;
            }
        }

        /// <summary>
        /// skill53 - see Gamestudio manual
        /// </summary>
        private Var skill53_ = null;
        public Var skill53
        {
            get
            {
                unsafe
                {
                    if (skill53_ == null)
                    {
                        skill53_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_053));
                    }
                }
                return skill53_;
            }
        }

        /// <summary>
        /// skill54 - see Gamestudio manual
        /// </summary>
        private Var skill54_ = null;
        public Var skill54
        {
            get
            {
                unsafe
                {
                    if (skill54_ == null)
                    {
                        skill54_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_054));
                    }
                }
                return skill54_;
            }
        }

        /// <summary>
        /// skill55 - see Gamestudio manual
        /// </summary>
        private Var skill55_ = null;
        public Var skill55
        {
            get
            {
                unsafe
                {
                    if (skill55_ == null)
                    {
                        skill55_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_055));
                    }
                }
                return skill55_;
            }
        }

        /// <summary>
        /// skill56 - see Gamestudio manual
        /// </summary>
        private Var skill56_ = null;
        public Var skill56
        {
            get
            {
                unsafe
                {
                    if (skill56_ == null)
                    {
                        skill56_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_056));
                    }
                }
                return skill56_;
            }
        }

        /// <summary>
        /// skill57 - see Gamestudio manual
        /// </summary>
        private Var skill57_ = null;
        public Var skill57
        {
            get
            {
                unsafe
                {
                    if (skill57_ == null)
                    {
                        skill57_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_057));
                    }
                }
                return skill57_;
            }
        }

        /// <summary>
        /// skill58 - see Gamestudio manual
        /// </summary>
        private Var skill58_ = null;
        public Var skill58
        {
            get
            {
                unsafe
                {
                    if (skill58_ == null)
                    {
                        skill58_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_058));
                    }
                }
                return skill58_;
            }
        }

        /// <summary>
        /// skill59 - see Gamestudio manual
        /// </summary>
        private Var skill59_ = null;
        public Var skill59
        {
            get
            {
                unsafe
                {
                    if (skill59_ == null)
                    {
                        skill59_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_059));
                    }
                }
                return skill59_;
            }
        }

        /// <summary>
        /// skill60 - see Gamestudio manual
        /// </summary>
        private Var skill60_ = null;
        public Var skill60
        {
            get
            {
                unsafe
                {
                    if (skill60_ == null)
                    {
                        skill60_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_060));
                    }
                }
                return skill60_;
            }
        }

        /// <summary>
        /// skill61 - see Gamestudio manual
        /// </summary>
        private Var skill61_ = null;
        public Var skill61
        {
            get
            {
                unsafe
                {
                    if (skill61_ == null)
                    {
                        skill61_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_061));
                    }
                }
                return skill61_;
            }
        }

        /// <summary>
        /// skill62 - see Gamestudio manual
        /// </summary>
        private Var skill62_ = null;
        public Var skill62
        {
            get
            {
                unsafe
                {
                    if (skill62_ == null)
                    {
                        skill62_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_062));
                    }
                }
                return skill62_;
            }
        }

        /// <summary>
        /// skill63 - see Gamestudio manual
        /// </summary>
        private Var skill63_ = null;
        public Var skill63
        {
            get
            {
                unsafe
                {
                    if (skill63_ == null)
                    {
                        skill63_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_063));
                    }
                }
                return skill63_;
            }
        }

        /// <summary>
        /// skill64 - see Gamestudio manual
        /// </summary>
        private Var skill64_ = null;
        public Var skill64
        {
            get
            {
                unsafe
                {
                    if (skill64_ == null)
                    {
                        skill64_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_064));
                    }
                }
                return skill64_;
            }
        }

        /// <summary>
        /// skill65 - see Gamestudio manual
        /// </summary>
        private Var skill65_ = null;
        public Var skill65
        {
            get
            {
                unsafe
                {
                    if (skill65_ == null)
                    {
                        skill65_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_065));
                    }
                }
                return skill65_;
            }
        }

        /// <summary>
        /// skill66 - see Gamestudio manual
        /// </summary>
        private Var skill66_ = null;
        public Var skill66
        {
            get
            {
                unsafe
                {
                    if (skill66_ == null)
                    {
                        skill66_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_066));
                    }
                }
                return skill66_;
            }
        }

        /// <summary>
        /// skill67 - see Gamestudio manual
        /// </summary>
        private Var skill67_ = null;
        public Var skill67
        {
            get
            {
                unsafe
                {
                    if (skill67_ == null)
                    {
                        skill67_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_067));
                    }
                }
                return skill67_;
            }
        }

        /// <summary>
        /// skill68 - see Gamestudio manual
        /// </summary>
        private Var skill68_ = null;
        public Var skill68
        {
            get
            {
                unsafe
                {
                    if (skill68_ == null)
                    {
                        skill68_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_068));
                    }
                }
                return skill68_;
            }
        }

        /// <summary>
        /// skill69 - see Gamestudio manual
        /// </summary>
        private Var skill69_ = null;
        public Var skill69
        {
            get
            {
                unsafe
                {
                    if (skill69_ == null)
                    {
                        skill69_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_069));
                    }
                }
                return skill69_;
            }
        }

        /// <summary>
        /// skill70 - see Gamestudio manual
        /// </summary>
        private Var skill70_ = null;
        public Var skill70
        {
            get
            {
                unsafe
                {
                    if (skill70_ == null)
                    {
                        skill70_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_070));
                    }
                }
                return skill70_;
            }
        }

        /// <summary>
        /// skill71 - see Gamestudio manual
        /// </summary>
        private Var skill71_ = null;
        public Var skill71
        {
            get
            {
                unsafe
                {
                    if (skill71_ == null)
                    {
                        skill71_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_071));
                    }
                }
                return skill71_;
            }
        }

        /// <summary>
        /// skill72 - see Gamestudio manual
        /// </summary>
        private Var skill72_ = null;
        public Var skill72
        {
            get
            {
                unsafe
                {
                    if (skill72_ == null)
                    {
                        skill72_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_072));
                    }
                }
                return skill72_;
            }
        }

        /// <summary>
        /// skill73 - see Gamestudio manual
        /// </summary>
        private Var skill73_ = null;
        public Var skill73
        {
            get
            {
                unsafe
                {
                    if (skill73_ == null)
                    {
                        skill73_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_073));
                    }
                }
                return skill73_;
            }
        }

        /// <summary>
        /// skill74 - see Gamestudio manual
        /// </summary>
        private Var skill74_ = null;
        public Var skill74
        {
            get
            {
                unsafe
                {
                    if (skill74_ == null)
                    {
                        skill74_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_074));
                    }
                }
                return skill74_;
            }
        }

        /// <summary>
        /// skill75 - see Gamestudio manual
        /// </summary>
        private Var skill75_ = null;
        public Var skill75
        {
            get
            {
                unsafe
                {
                    if (skill75_ == null)
                    {
                        skill75_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_075));
                    }
                }
                return skill75_;
            }
        }

        /// <summary>
        /// skill76 - see Gamestudio manual
        /// </summary>
        private Var skill76_ = null;
        public Var skill76
        {
            get
            {
                unsafe
                {
                    if (skill76_ == null)
                    {
                        skill76_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_076));
                    }
                }
                return skill76_;
            }
        }

        /// <summary>
        /// skill77 - see Gamestudio manual
        /// </summary>
        private Var skill77_ = null;
        public Var skill77
        {
            get
            {
                unsafe
                {
                    if (skill77_ == null)
                    {
                        skill77_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_077));
                    }
                }
                return skill77_;
            }
        }

        /// <summary>
        /// skill78 - see Gamestudio manual
        /// </summary>
        private Var skill78_ = null;
        public Var skill78
        {
            get
            {
                unsafe
                {
                    if (skill78_ == null)
                    {
                        skill78_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_078));
                    }
                }
                return skill78_;
            }
        }

        /// <summary>
        /// skill79 - see Gamestudio manual
        /// </summary>
        private Var skill79_ = null;
        public Var skill79
        {
            get
            {
                unsafe
                {
                    if (skill79_ == null)
                    {
                        skill79_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_079));
                    }
                }
                return skill79_;
            }
        }

        /// <summary>
        /// skill80 - see Gamestudio manual
        /// </summary>
        private Var skill80_ = null;
        public Var skill80
        {
            get
            {
                unsafe
                {
                    if (skill80_ == null)
                    {
                        skill80_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_080));
                    }
                }
                return skill80_;
            }
        }

        /// <summary>
        /// skill81 - see Gamestudio manual
        /// </summary>
        private Var skill81_ = null;
        public Var skill81
        {
            get
            {
                unsafe
                {
                    if (skill81_ == null)
                    {
                        skill81_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_081));
                    }
                }
                return skill81_;
            }
        }

        /// <summary>
        /// skill82 - see Gamestudio manual
        /// </summary>
        private Var skill82_ = null;
        public Var skill82
        {
            get
            {
                unsafe
                {
                    if (skill82_ == null)
                    {
                        skill82_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_082));
                    }
                }
                return skill82_;
            }
        }

        /// <summary>
        /// skill83 - see Gamestudio manual
        /// </summary>
        private Var skill83_ = null;
        public Var skill83
        {
            get
            {
                unsafe
                {
                    if (skill83_ == null)
                    {
                        skill83_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_083));
                    }
                }
                return skill83_;
            }
        }

        /// <summary>
        /// skill84 - see Gamestudio manual
        /// </summary>
        private Var skill84_ = null;
        public Var skill84
        {
            get
            {
                unsafe
                {
                    if (skill84_ == null)
                    {
                        skill84_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_084));
                    }
                }
                return skill84_;
            }
        }

        /// <summary>
        /// skill85 - see Gamestudio manual
        /// </summary>
        private Var skill85_ = null;
        public Var skill85
        {
            get
            {
                unsafe
                {
                    if (skill85_ == null)
                    {
                        skill85_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_085));
                    }
                }
                return skill85_;
            }
        }

        /// <summary>
        /// skill86 - see Gamestudio manual
        /// </summary>
        private Var skill86_ = null;
        public Var skill86
        {
            get
            {
                unsafe
                {
                    if (skill86_ == null)
                    {
                        skill86_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_086));
                    }
                }
                return skill86_;
            }
        }

        /// <summary>
        /// skill87 - see Gamestudio manual
        /// </summary>
        private Var skill87_ = null;
        public Var skill87
        {
            get
            {
                unsafe
                {
                    if (skill87_ == null)
                    {
                        skill87_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_087));
                    }
                }
                return skill87_;
            }
        }

        /// <summary>
        /// skill88 - see Gamestudio manual
        /// </summary>
        private Var skill88_ = null;
        public Var skill88
        {
            get
            {
                unsafe
                {
                    if (skill88_ == null)
                    {
                        skill88_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_088));
                    }
                }
                return skill88_;
            }
        }

        /// <summary>
        /// skill89 - see Gamestudio manual
        /// </summary>
        private Var skill89_ = null;
        public Var skill89
        {
            get
            {
                unsafe
                {
                    if (skill89_ == null)
                    {
                        skill89_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_089));
                    }
                }
                return skill89_;
            }
        }

        /// <summary>
        /// skill90 - see Gamestudio manual
        /// </summary>
        private Var skill90_ = null;
        public Var skill90
        {
            get
            {
                unsafe
                {
                    if (skill90_ == null)
                    {
                        skill90_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_090));
                    }
                }
                return skill90_;
            }
        }

        /// <summary>
        /// skill91 - see Gamestudio manual
        /// </summary>
        private Var skill91_ = null;
        public Var skill91
        {
            get
            {
                unsafe
                {
                    if (skill91_ == null)
                    {
                        skill91_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_091));
                    }
                }
                return skill91_;
            }
        }

        /// <summary>
        /// skill92 - see Gamestudio manual
        /// </summary>
        private Var skill92_ = null;
        public Var skill92
        {
            get
            {
                unsafe
                {
                    if (skill92_ == null)
                    {
                        skill92_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_092));
                    }
                }
                return skill92_;
            }
        }

        /// <summary>
        /// skill93 - see Gamestudio manual
        /// </summary>
        private Var skill93_ = null;
        public Var skill93
        {
            get
            {
                unsafe
                {
                    if (skill93_ == null)
                    {
                        skill93_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_093));
                    }
                }
                return skill93_;
            }
        }

        /// <summary>
        /// skill94 - see Gamestudio manual
        /// </summary>
        private Var skill94_ = null;
        public Var skill94
        {
            get
            {
                unsafe
                {
                    if (skill94_ == null)
                    {
                        skill94_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_094));
                    }
                }
                return skill94_;
            }
        }

        /// <summary>
        /// skill95 - see Gamestudio manual
        /// </summary>
        private Var skill95_ = null;
        public Var skill95
        {
            get
            {
                unsafe
                {
                    if (skill95_ == null)
                    {
                        skill95_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_095));
                    }
                }
                return skill95_;
            }
        }

        /// <summary>
        /// skill96 - see Gamestudio manual
        /// </summary>
        private Var skill96_ = null;
        public Var skill96
        {
            get
            {
                unsafe
                {
                    if (skill96_ == null)
                    {
                        skill96_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_096));
                    }
                }
                return skill96_;
            }
        }

        /// <summary>
        /// skill97 - see Gamestudio manual
        /// </summary>
        private Var skill97_ = null;
        public Var skill97
        {
            get
            {
                unsafe
                {
                    if (skill97_ == null)
                    {
                        skill97_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_097));
                    }
                }
                return skill97_;
            }
        }

        /// <summary>
        /// skill98 - see Gamestudio manual
        /// </summary>
        private Var skill98_ = null;
        public Var skill98
        {
            get
            {
                unsafe
                {
                    if (skill98_ == null)
                    {
                        skill98_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_098));
                    }
                }
                return skill98_;
            }
        }

        /// <summary>
        /// skill99 - see Gamestudio manual
        /// </summary>
        private Var skill99_ = null;
        public Var skill99
        {
            get
            {
                unsafe
                {
                    if (skill99_ == null)
                    {
                        skill99_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_099));
                    }
                }
                return skill99_;
            }
        }

        /// <summary>
        /// skill100 - see Gamestudio manual
        /// </summary>
        private Var skill100_ = null;
        public Var skill100
        {
            get
            {
                unsafe
                {
                    if (skill100_ == null)
                    {
                        skill100_ = (Var)(IntPtr)(&((*ENTITYPointer).skill_100));
                    }
                }
                return skill100_;
            }
        }

        /// <summary>
        /// string1 - see Gamestudio manual
        /// </summary>
        public String string1
        {
            get
            {
                unsafe
                {
                    return Marshal.PtrToStringAnsi((IntPtr)(*ENTITYPointer).string1);
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).string1 = (char*)(void*)Marshal.StringToHGlobalAnsi(value);
                }
            }
        }

        /// <summary>
        /// string2 - see Gamestudio manual
        /// </summary>
        public String string2
        {
            get
            {
                unsafe
                {
                    return Marshal.PtrToStringAnsi((IntPtr)(*ENTITYPointer).string2);
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).string2 = (char*)(void*)Marshal.StringToHGlobalAnsi(value);
                }
            }
        }


        /// <summary>
        /// ENTITY flags (flags)
        /// </summary>
        public enum FLAG : int
        {
            FLAG1 = 1 << 0,
            FLAG2 = 1 << 1,
            FLAG3 = 1 << 2,
            FLAG4 = 1 << 3,
            FLAG5 = 1 << 4,
            FLAG6 = 1 << 5,
            FLAG7 = 1 << 6,
            FLAG8 = 1 << 7,
            INVISIBLE = 1 << 8,
            PASSABLE = 1 << 9,
            TRANSLUCENT = 1 << 10,
            OVERLAY = 1 << 12,
            SPOTLIGHT = 1 << 13,
            ZNEAR = 1 << 14,
            NOFILTER = 1 << 16,
            UNLIT = 1 << 17,
            SHADOW = 1 << 18,
            LIGHT = 1 << 19,
            NOFOG = 1 << 20,
            BRIGHT = 1 << 21,
            DECAL = 1 << 22,
            METAL = 1 << 22,
            CAST = 1 << 23,
            POLYGON = 1 << 26
        }

        /// <summary>
        /// flags - see Gamestudio manual
        /// </summary>
        public int flags
        {
            get
            {
                unsafe
                {
                    return (*ENTITYPointer).flags;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).flags = value;
                }
            }
        }

        /// <summary>
        /// ENTITY eFLAG (eflags)
        /// </summary>
        public enum eFLAG : int
        {
            FAT = 1 << 0,
            NARROW = 1 << 1,
            PASS_TRANSP = 1 << 7,
            PASS_BOTH = 1 << 8,
            LOD0 = 1 << 16,
            LOD1 = 1 << 17,
            CLIP0 = 1 << 18,
            CLIP1 = 1 << 19,
            CLIPPED = 1 << 21,
            ANIMATE = 1 << 24,
            LOCAL = 1 << 25
        }

        /// <summary>
        /// eflags - see Gamestudio manual
        /// </summary>
        public int eflags
        {
            get
            {
                unsafe
                {
                    return (*ENTITYPointer).eflags;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).eflags = value;
                }
            }
        }

        /// <summary>
        /// ENTITY FLAGS2 used by for example ent_createlayer
        /// </summary>
        public enum FLAGS2 : int
        {
            UNTOUCHABLE = 1 << 0,
            SHOW = 1 << 14,
            SKY = 1 << 17,
            DOME = 1 << 18,
            SCENE = 1 << 19,
            CUBE = 1 << 20,
            CYLINDER = 1 << 21
        }

        /// <summary>
        /// emask - see Gamestudio manual
        /// </summary>
        public int flags2
        {
            get
            {
                unsafe
                {
                    return (*ENTITYPointer).flags2;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).flags2 = value;
                }
            }
        }

        /// <summary>
        /// Entity event flags (emask)
        /// </summary>
        public enum eMask : int
        {
            DYNAMIC = 1 << 0,
            ENABLE_BLOCK = 1 << EngVar.EVENT_DEFINITIONS.EVENT_BLOCK,
            ENABLE_ENTITY = 1 << EngVar.EVENT_DEFINITIONS.EVENT_ENTITY,
            ENABLE_STUCK = 1 << EngVar.EVENT_DEFINITIONS.EVENT_STUCK,
            ENABLE_PUSH = 1 << EngVar.EVENT_DEFINITIONS.EVENT_PUSH,
            ENABLE_IMPACT = 1 << EngVar.EVENT_DEFINITIONS.EVENT_IMPACT,
            ENABLE_FRICTION = 1 << EngVar.EVENT_DEFINITIONS.EVENT_FRICTION,
            ENABLE_DETECT = 1 << EngVar.EVENT_DEFINITIONS.EVENT_DETECT,
            ENABLE_SCAN = 1 << EngVar.EVENT_DEFINITIONS.EVENT_SCAN,
            ENABLE_SHOOT = 1 << EngVar.EVENT_DEFINITIONS.EVENT_SHOOT,
            ENABLE_TRIGGER = 1 << EngVar.EVENT_DEFINITIONS.EVENT_TRIGGER,
            ENABLE_SONAR = 1 << EngVar.EVENT_DEFINITIONS.EVENT_SONAR,
            ENABLE_FRAME = 1 << EngVar.EVENT_DEFINITIONS.EVENT_FRAME,
            ENABLE_RECEIVE = 1 << EngVar.EVENT_DEFINITIONS.EVENT_RECEIVE,
            ENABLE_DISCONNECT = 1 << EngVar.EVENT_DEFINITIONS.EVENT_DISCONNECT,
            ENABLE_BUTTONUP = 1 << EngVar.EVENT_DEFINITIONS.EVENT_BUTTONUP,
            ENABLE_TOUCH = 1 << EngVar.EVENT_DEFINITIONS.EVENT_TOUCH,
            ENABLE_RELEASE = 1 << EngVar.EVENT_DEFINITIONS.EVENT_RELEASE,
            ENABLE_CLICK = 1 << EngVar.EVENT_DEFINITIONS.EVENT_CLICK,
            ENABLE_RIGHTCLICK = 1 << EngVar.EVENT_DEFINITIONS.EVENT_RIGHTCLICK
        }

        /// <summary>
        /// emask - see Gamestudio manual
        /// </summary>
        public int emask
        {
            get
            {
                unsafe
                {
                    return (*ENTITYPointer).emask;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).emask = value;
                }
            }
        }

        /// <summary>
        /// ENABLE_BLOCK - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_BLOCK
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_BLOCK) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_BLOCK;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_BLOCK;

                }
            }
        }

        /// <summary>
        /// ENABLE_ENTITY - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_ENTITY
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_ENTITY) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_ENTITY;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_ENTITY;

                }
            }
        }

        /// <summary>
        /// ENABLE_STUCK - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_STUCK
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_STUCK) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_STUCK;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_STUCK;

                }
            }
        }

        /// <summary>
        /// ENABLE_PUSH - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_PUSH
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_PUSH) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_PUSH;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_STUCK;

                }
            }
        }

        /// <summary>
        /// ENABLE_IMPACT - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_IMPACT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_IMPACT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_IMPACT;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_IMPACT;

                }
            }
        }

        /// <summary>
        /// ENABLE_FRICTION - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_FRICTION
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_FRICTION) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_FRICTION;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_FRICTION;

                }
            }
        }

        /// <summary>
        /// ENABLE_DETECT - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_DETECT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_DETECT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_DETECT;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_DETECT;

                }
            }
        }

        /// <summary>
        /// ENABLE_SCAN - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_SCAN
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_SCAN) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_SCAN;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_SCAN;

                }
            }
        }

        /// <summary>
        /// ENABLE_SHOOT - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_SHOOT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_SHOOT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_SHOOT;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_SHOOT;

                }
            }
        }

        /// <summary>
        /// ENABLE_TRIGGER - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_TRIGGER
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_TRIGGER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_TRIGGER;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_TRIGGER;

                }
            }
        }

        /// <summary>
        /// ENABLE_SONAR - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_SONAR
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_SONAR) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_SONAR;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_SONAR;

                }
            }
        }

        /// <summary>
        /// ENABLE_FRAME - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_FRAME
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_FRAME) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_FRAME;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_FRAME;

                }
            }
        }

        /// <summary>
        /// ENABLE_RECEIVE - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_RECEIVE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_RECEIVE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_RECEIVE;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_RECEIVE;

                }
            }
        }

        /// <summary>
        /// ENABLE_DISCONNECT - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_DISCONNECT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_DISCONNECT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_DISCONNECT;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_DISCONNECT;

                }
            }
        }

        /// <summary>
        /// ENABLE_BUTTONUP - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_BUTTONUP
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_BUTTONUP) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_BUTTONUP;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_BUTTONUP;

                }
            }
        }

        /// <summary>
        /// ENABLE_TOUCH - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_TOUCH
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_TOUCH) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_TOUCH;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_TOUCH;

                }
            }
        }

        /// <summary>
        /// ENABLE_RELEASE - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_RELEASE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_RELEASE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_RELEASE;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_RELEASE;

                }
            }
        }

        /// <summary>
        /// ENABLE_CLICK - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_CLICK
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_CLICK) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_CLICK;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_CLICK;

                }
            }
        }

        /// <summary>
        /// ENABLE_RIGHTCLICK - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_RIGHTCLICK
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.ENABLE_RIGHTCLICK) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.ENABLE_RIGHTCLICK;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.ENABLE_RIGHTCLICK;

                }
            }
        }

        /// <summary>
        /// NOSEND flasg
        /// </summary>
        public enum NOSENDFLAGS : int
        {
            NOSEND = 1 << 15,
            NOSEND_ALPHA = 1 << 8,
            NOSEND_AMBIENT = 1 << 7,
            NOSEND_ANGLES = 1 << 3,
            NOSEND_ATTACH = 1 << 13,
            NOSEND_COLOR = 1 << 10,
            NOSEND_FLAGS = 1 << 6,
            NOSEND_FRAME = 1 << 4,
            NOSEND_LIGHT = 1 << 9,
            NOSEND_ORIGIN = 1 << 2,
            NOSEND_SCALE = 1 << 1,
            NOSEND_SKIN = 1 << 5,
            NOSEND_SOUND = 1 << 12,
            NOSEND_Z = 1 << 14
        }

        /// <summary>
        /// smask - see Gamestudio manual
        /// </summary>
        public int smask
        {
            get
            {
                unsafe
                {
                    return (*ENTITYPointer).smask;
                }
            }
            set
            {
                unsafe
                {
                    (*ENTITYPointer).smask = value;
                }
            }
        }


        /// <summary>
        /// passable - see Gamestudio manual
        /// </summary>
        public Boolean PASSABLE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.PASSABLE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.PASSABLE;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.PASSABLE;

                }
            }
        }


        /// <summary>
        /// polygon - see Gamestudio manual
        /// </summary>
        public Boolean POLYGON
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.POLYGON) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.POLYGON;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.POLYGON;

                }
            }
        }


        /// <summary>
        /// fat - see Gamestudio manual
        /// </summary>
        public Boolean FAT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.FAT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags | (int)eFLAG.FAT;
                    else
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags & ~(int)eFLAG.FAT;

                }
            }
        }


        /// <summary>
        /// narrow - see Gamestudio manual
        /// </summary>
        public Boolean NARROW
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.NARROW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags | (int)eFLAG.NARROW;
                    else
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags & ~(int)eFLAG.NARROW;

                }
            }
        }


        /// <summary>
        /// untouchable - see Gamestudio manual
        /// </summary>
        public Boolean UNTOUCHABLE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags2 & (int)FLAGS2.UNTOUCHABLE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 | (int)FLAGS2.UNTOUCHABLE;
                    else
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 & ~(int)FLAGS2.UNTOUCHABLE;
                }
            }
        }


        /// <summary>
        /// clipped - see Gamestudio manual
        /// </summary>
        public Boolean CLIPPED
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.CLIPPED) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
            }
        }


        /// <summary>
        /// flag1 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG1
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG1) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG1;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG1;
                }
            }
        }


        /// <summary>
        /// flag2 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG2
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG2) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG2;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG2;
                }
            }
        }


        /// <summary>
        /// flag3 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG3
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG3) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG3;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG3;
                }
            }
        }


        /// <summary>
        /// flag4 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG4
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG4) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG4;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG4;
                }
            }
        }


        /// <summary>
        /// flag5 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG5
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG5) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG5;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG5;
                }
            }
        }


        /// <summary>
        /// flag6 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG6
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG6) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG6;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG6;
                }
            }
        }


        /// <summary>
        /// flag7 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG7
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG7) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG7;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG7;
                }
            }
        }


        /// <summary>
        /// flag8 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG8
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.FLAG8) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.FLAG8;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.FLAG8;
                }
            }
        }


        /// <summary>
        /// lod0 - see Gamestudio manual
        /// </summary>
        public Boolean LOD0
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.LOD0) == 0)
                        return false;
                    else
                        return true;
                }
            }
        }


        /// <summary>
        /// lod1 - see Gamestudio manual
        /// </summary>
        public Boolean LOD1
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.LOD1) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
            }
        }


        /// <summary>
        /// CLIP0 - see Gamestudio manual
        /// </summary>
        public Boolean CLIP0
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.CLIP0) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags | (int)eFLAG.CLIP0;
                    else
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags & ~(int)eFLAG.CLIP0;
                }
            }
        }


        /// <summary>
        /// CLIP1 - see Gamestudio manual
        /// </summary>
        public Boolean CLIP1
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.CLIP1) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags | (int)eFLAG.CLIP1;
                    else
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags & ~(int)eFLAG.CLIP1;
                }
            }
        }


        /// <summary>
        /// shadow - see Gamestudio manual
        /// </summary>
        public Boolean SHADOW
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.SHADOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.SHADOW;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.SHADOW;
                }
            }
        }


        /// <summary>
        /// cast - see Gamestudio manual
        /// </summary>
        public Boolean CAST
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.CAST) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.CAST;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.CAST;
                }
            }
        }


        /// <summary>
        /// translucent - see Gamestudio manual
        /// </summary>
        public Boolean TRANSLUCENT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.TRANSLUCENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.TRANSLUCENT;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.TRANSLUCENT;
                }
            }
        }


        /// <summary>
        /// bright - see Gamestudio manual
        /// </summary>
        public Boolean BRIGHT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.BRIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.BRIGHT;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.BRIGHT;
                }
            }
        }


        /// <summary>
        /// light - see Gamestudio manual
        /// </summary>
        public Boolean LIGHT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.LIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.LIGHT;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.LIGHT;
                }
            }
        }


        /// <summary>
        /// spotlight - see Gamestudio manual
        /// </summary>
        public Boolean SPOTLIGHT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.SPOTLIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.SPOTLIGHT;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.SPOTLIGHT;
                }
            }
        }


        /// <summary>
        /// unlit - see Gamestudio manual
        /// </summary>
        public Boolean UNLIT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.UNLIT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.UNLIT;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.UNLIT;
                }
            }
        }


        /// <summary>
        /// overlay - see Gamestudio manual
        /// </summary>
        public Boolean OVERLAY
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.OVERLAY) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.OVERLAY;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.OVERLAY;
                }
            }
        }


        /// <summary>
        /// decal - see Gamestudio manual
        /// </summary>
        public Boolean DECAL
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.DECAL) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.DECAL;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.DECAL;
                }
            }
        }


        /// <summary>
        /// nofog - see Gamestudio manual
        /// </summary>
        public Boolean NOFOG
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.NOFOG) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.NOFOG;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.NOFOG;
                }
            }
        }


        /// <summary>
        /// nofilter - see Gamestudio manual
        /// </summary>
        public Boolean NOFILTER
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.NOFILTER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.NOFILTER;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.NOFILTER;
                }
            }
        }


        /// <summary>
        /// invisible - see Gamestudio manual
        /// </summary>
        public Boolean INVISIBLE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.INVISIBLE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.INVISIBLE;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.INVISIBLE;
                }
            }
        }


        /// <summary>
        /// znear - see Gamestudio manual
        /// </summary>
        public Boolean ZNEAR
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags & (int)FLAG.ZNEAR) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags | (int)FLAG.ZNEAR;
                    else
                        (*ENTITYPointer).flags = (*ENTITYPointer).flags & ~(int)FLAG.ZNEAR;
                }
            }
        }


        /// <summary>
        /// animate - see Gamestudio manual
        /// </summary>
        public Boolean ANIMATE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).eflags & (int)eFLAG.ANIMATE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags | (int)eFLAG.ANIMATE;
                    else
                        (*ENTITYPointer).eflags = (*ENTITYPointer).eflags & ~(int)eFLAG.ANIMATE;
                }
            }
        }


        /// <summary>
        /// dynamic - see Gamestudio manual
        /// </summary>
        public Boolean DYNAMIC
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).emask & (int)eMask.DYNAMIC) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask | (int)eMask.DYNAMIC;
                    else
                        (*ENTITYPointer).emask = (*ENTITYPointer).emask & ~(int)eMask.DYNAMIC;
                }
            }
        }


        /// <summary>
        /// nosend - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND;
                }
            }
        }


        /// <summary>
        /// nosend_alpha - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_ALPHA
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_ALPHA) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_ALPHA;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_ALPHA;
                }
            }
        }


        /// <summary>
        /// nosend_ambient
        /// </summary>
        public Boolean NOSEND_AMBIENT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_AMBIENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_AMBIENT;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_AMBIENT;
                }
            }
        }


        /// <summary>
        /// nosend_angles - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_ANGLES
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_ANGLES) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_ANGLES;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_ANGLES;
                }
            }
        }


        /// <summary>
        /// nosend_attach - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_ATTACH
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_ATTACH) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_ATTACH;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_ATTACH;
                }
            }
        }


        /// <summary>
        /// nosend_color - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_COLOR
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_COLOR) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_COLOR;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_COLOR;
                }
            }
        }


        /// <summary>
        /// nosend_flags - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_FLAGS
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_FLAGS) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_FLAGS;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_FLAGS;
                }
            }
        }


        /// <summary>
        /// nosend_frame - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_FRAME
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_FRAME) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_FRAME;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_FRAME;
                }
            }
        }


        /// <summary>
        /// nosend_lights - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_LIGHT
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_LIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_LIGHT;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_LIGHT;
                }
            }
        }


        /// <summary>
        /// nosend_origin - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_ORIGIN
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_ORIGIN) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_ORIGIN;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_ORIGIN;
                }
            }
        }


        /// <summary>
        /// nosend_scale
        /// </summary>
        public Boolean NOSEND_SCALE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_SCALE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_SCALE;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_SCALE;
                }
            }
        }


        /// <summary>
        /// nosend_skin - see Gamestudio manual
        /// </summary>
        public Boolean NOSEND_SKIN
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_SKIN) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_SKIN;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_SKIN;
                }
            }
        }


        /// <summary>
        /// nosend_sound
        /// </summary>
        public Boolean NOSEND_SOUND
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_SOUND) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_SOUND;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_SOUND;
                }
            }
        }


        /// <summary>
        /// nosend_sound
        /// </summary>
        public Boolean NOSEND_Z
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).smask & (int)NOSENDFLAGS.NOSEND_Z) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask | (int)NOSENDFLAGS.NOSEND_Z;
                    else
                        (*ENTITYPointer).smask = (*ENTITYPointer).smask & ~(int)NOSENDFLAGS.NOSEND_Z;
                }
            }
        }

        /// <summary>
        /// speed_u - see Gamestudio manual
        /// </summary>
        private Var speed_u_ = null;
        public Var speed_u
        {
            get
            {
                unsafe
                {
                    if (speed_u_ == null)
                    {
                        speed_u_ = (Var)(IntPtr)(&((*ENTITYPointer).u));
                    }
                }
                return speed_u_;
            }
        }

        /// <summary>
        /// speed_v - see Gamestudio manual
        /// </summary>
        private Var speed_v_ = null;
        public Var speed_v
        {
            get
            {
                unsafe
                {
                    if (speed_v_ == null)
                    {
                        speed_v_ = (Var)(IntPtr)(&((*ENTITYPointer).v));
                    }
                }
                return speed_v_;
            }
        }

        /// <summary>
        /// group - see Gamestudio manual
        /// </summary>
        private Var group_ = null;
        public Var group
        {
            get
            {
                unsafe
                {
                    if (group_ == null)
                    {
                        group_ = (Var)(IntPtr)(&((*ENTITYPointer).group));
                    }
                }
                return group_;
            }
        }


        /// <summary>
        /// show - see Gamestudio manual
        /// </summary>
        public Boolean SHOW
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags2 & (int)FLAGS2.SHOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 | (int)FLAGS2.SHOW;
                    else
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 & ~(int)FLAGS2.SHOW;
                }
            }
        }


        /// <summary>
        /// SKY - see Gamestudio manual
        /// </summary>
        public Boolean SKY
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags2 & (int)FLAGS2.SKY) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 | (int)FLAGS2.SKY;
                    else
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 & ~(int)FLAGS2.SKY;
                }
            }
        }


        /// <summary>
        /// cube - see Gamestudio manual
        /// </summary>
        public Boolean CUBE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags2 & (int)FLAGS2.CUBE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 | (int)FLAGS2.CUBE;
                    else
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 & ~(int)FLAGS2.CUBE;
                }
            }
        }


        /// <summary>
        /// cylinder - see Gamestudio manual
        /// </summary>
        public Boolean CYLINDER
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags2 & (int)FLAGS2.CYLINDER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 | (int)FLAGS2.CYLINDER;
                    else
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 & ~(int)FLAGS2.CYLINDER;
                }
            }
        }


        /// <summary>
        /// scene - see Gamestudio manual
        /// </summary>
        public Boolean SCENE
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags2 & (int)FLAGS2.SCENE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 | (int)FLAGS2.SCENE;
                    else
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 & ~(int)FLAGS2.SCENE;
                }
            }
        }


        /// <summary>
        /// dome - see Gamestudio manual
        /// </summary>
        public Boolean DOME
        {
            get
            {
                unsafe
                {
                    if (((*ENTITYPointer).flags2 & (int)FLAGS2.DOME) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 | (int)FLAGS2.DOME;
                    else
                        (*ENTITYPointer).flags2 = (*ENTITYPointer).flags2 & ~(int)FLAGS2.DOME;
                }
            }
        }




        /* -------- a7.ENTITY -------- */
        /* a7.ENTITY to a7.NativeENTITY* (address in void*) */
        public static explicit operator IntPtr(ENTITY d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativeENTITY*)d.ENTITYPointer;
                }
            }
        }
        /* a7.NativeENTITY* (address in void*) to a7.ENTITY */
        public static explicit operator ENTITY(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new ENTITY(d);
            }
        }
    }
}