﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class TEXT
    {
        private unsafe NativeTEXT* TEXTPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public TEXT(IntPtr param0)
        {
            unsafe
            {
                TEXTPointer = (NativeTEXT*)param0;
            }
        }

        /// <summary>
        /// pstring - see Gamestudio manual
        /// </summary>
        public String[] pstring
        {
            get
            {
                unsafe
                {
                    int size = (*TEXTPointer).length;
                    NativeSTRING** pString = (*TEXTPointer).pstring;

                    String[] returnString = new String[size];

                    for (int i = 0; i < size; i++)
                    {
                        returnString[i] = Marshal.PtrToStringAnsi((IntPtr)(**(pString + i)).chars);
                    }

                    return returnString;
                }
            }
            set
            {
                unsafe
                {
                    int size = value.Length;

                    NativeSTRING** pString = (*TEXTPointer).pstring;
                    NativeSTRING* xxxx;

                    for (int i = 0; i < size; i++)
                    {
                        xxxx = *(pString + i);
                        (*xxxx).chars = (char*)(void*)Marshal.StringToHGlobalAnsi(value[i]);
                    }
                }
            }
        }


        /// <summary>
        /// strings - see Gamestudio manual
        /// </summary>
        private Var strings_ = null;
        public Var strings
        {
            get
            {
                unsafe
                {
                    if (strings_ == null)
                    {
                        strings_ = (Var)(IntPtr)(&((*TEXTPointer).strings));
                    }
                }
                return strings_;
            }
        }


        /// <summary>
        /// layer - see Gamestudio manual
        /// </summary>
        private Var layer_ = null;
        public Var layer
        {
            get
            {
                unsafe
                {
                    if (layer_ == null)
                    {
                        layer_ = (Var)(IntPtr)(&((*TEXTPointer).layer));
                    }
                }
                return layer_;
            }
        }


        /// <summary>
        /// pos_x - see Gamestudio manual
        /// </summary>
        private Var pos_x_ = null;
        public Var pos_x
        {
            get
            {
                unsafe
                {
                    if (pos_x_ == null)
                    {
                        pos_x_ = (Var)(IntPtr)(&((*TEXTPointer).pos_x));
                    }
                }
                return pos_x_;
            }
        }


        /// <summary>
        /// pos_y - see Gamestudio manual
        /// </summary>
        private Var pos_y_ = null;
        public Var pos_y
        {
            get
            {
                unsafe
                {
                    if (pos_y_ == null)
                    {
                        pos_y_ = (Var)(IntPtr)(&((*TEXTPointer).pos_y));
                    }
                }
                return pos_y_;
            }
        }


        /// <summary>
        /// scale_x - see Gamestudio manual
        /// </summary>
        private Var scale_x_ = null;
        public Var scale_x
        {
            get
            {
                unsafe
                {
                    if (scale_x_ == null)
                    {
                        scale_x_ = (Var)(IntPtr)(&((*TEXTPointer).scale_x));
                    }
                }
                return scale_x_;
            }
        }


        /// <summary>
        /// scale_y - see Gamestudio manual
        /// </summary>
        private Var scale_y_ = null;
        public Var scale_y
        {
            get
            {
                unsafe
                {
                    if (scale_y_ == null)
                    {
                        scale_y_ = (Var)(IntPtr)(&((*TEXTPointer).scale_y));
                    }
                }
                return scale_y_;
            }
        }


        /// <summary>
        /// size_x - see Gamestudio manual
        /// </summary>
        private Var size_x_ = null;
        public Var size_x
        {
            get
            {
                unsafe
                {
                    if (size_x_ == null)
                    {
                        size_x_ = (Var)(IntPtr)(&((*TEXTPointer).size_x));
                    }
                }
                return size_x_;
            }
        }


        /// <summary>
        /// size_y - see Gamestudio manual
        /// </summary>
        private Var size_y_ = null;
        public Var size_y
        {
            get
            {
                unsafe
                {
                    if (size_y_ == null)
                    {
                        size_y_ = (Var)(IntPtr)(&((*TEXTPointer).size_y));
                    }
                }
                return size_y_;
            }
        }


        /// <summary>
        /// offset_y - see Gamestudio manual
        /// </summary>
        private Var offset_y_ = null;
        public Var offset_y
        {
            get
            {
                unsafe
                {
                    if (offset_y_ == null)
                    {
                        offset_y_ = (Var)(IntPtr)(&((*TEXTPointer).offset_y));
                    }
                }
                return offset_y_;
            }
        }


        /// <summary>
        /// alpha - see Gamestudio manual
        /// </summary>
        private Var alpha_ = null;
        public Var alpha
        {
            get
            {
                unsafe
                {
                    if (alpha_ == null)
                    {
                        alpha_ = (Var)(IntPtr)(&((*TEXTPointer).alpha));
                    }
                }
                return alpha_;
            }
        }


        /// <summary>
        /// font - see Gamestudio manual
        /// </summary>
        public FONT font
        {
            get
            {
                unsafe
                {
                    return (FONT)(IntPtr)(*TEXTPointer).font;
                }
            }
            set
            {
                unsafe
                {
                    (*TEXTPointer).font = (NativeFONT*)(IntPtr)value;
                }
            }
        }


        /// <summary>
        /// red - see Gamestudio manual
        /// </summary>
        private Var red_ = null;
        public Var red
        {
            get
            {
                unsafe
                {
                    if (red_ == null)
                    {
                        red_ = (Var)(IntPtr)(&((*TEXTPointer).red));
                    }
                }
                return red_;
            }
        }

        /// <summary>
        /// green - see Gamestudio manual
        /// </summary>
        private Var green_ = null;
        public Var green
        {
            get
            {
                unsafe
                {
                    if (green_ == null)
                    {
                        green_ = (Var)(IntPtr)(&((*TEXTPointer).green));
                    }
                }
                return green_;
            }
        }

        /// <summary>
        /// blue - see Gamestudio manual
        /// </summary>
        private Var blue_ = null;
        public Var blue
        {
            get
            {
                unsafe
                {
                    if (blue_ == null)
                    {
                        blue_ = (Var)(IntPtr)(&((*TEXTPointer).blue));
                    }
                }
                return blue_;
            }
        }

        /// <summary>
        /// greenBlueRed - not listed in the Gamestudio manual - allows to access scale_x, scale_y and scale_z as a Vector
        /// </summary>
        private Color greenBlueRed_ = null;
        public Color greenBlueRed
        {
            get
            {
                unsafe
                {
                    if (greenBlueRed_ == null)
                    {
                        greenBlueRed_ = new Color((Var)0, (Var)0, (Var)0);
                        greenBlueRed_.red = (Var)(IntPtr)(&((*TEXTPointer).red));
                        greenBlueRed_.green = (Var)(IntPtr)(&((*TEXTPointer).green));
                        greenBlueRed_.blue = (Var)(IntPtr)(&((*TEXTPointer).blue));
                    }
                }
                return greenBlueRed_;
            }
        }

        /// <summary>
        /// TEXT flags
        /// </summary>
        public enum FLAG : int
        {
            SHOW = 1 << 14,
            TRANSLUCENT = 1 << 10,
            FILTER = 1 << 20,
            CENTER_X = 1 << 8,
            CENTER_Y = 1 << 9,
            ARIGHT = 1 << 7,
            WWRAP = 1 << 6,
            CONDENSED = 1 << 13,
            SHADOW = 1 << 18,
            OUTLINE = 1 << 15,
            LIGHT = 1 << 19
        }

        /// <summary>
        /// flags - see Gamestudio manual
        /// </summary>
        public int flags
        {
            get
            {
                unsafe
                {
                    return (*TEXTPointer).flags;
                }
            }
            set
            {
                unsafe
                {
                    (*TEXTPointer).flags = value;
                }
            }
        }

        /// <summary>
        /// show - see Gamestudio manaul
        /// </summary>
        public Boolean SHOW
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.SHOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.SHOW;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.SHOW;

                }
            }
        }

        /// <summary>
        /// LIGHT - see Gamestudio manual
        /// </summary>
        public Boolean LIGHT
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.LIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.LIGHT;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.LIGHT;

                }
            }
        }

        /// <summary>
        /// translucent - see Gamestudio manual
        /// </summary>
        public Boolean TRANSLUCENT
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.TRANSLUCENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.TRANSLUCENT;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.TRANSLUCENT;

                }
            }
        }

        /// <summary>
        /// filter - see Gamestudio manual
        /// </summary>
        public Boolean FILTER
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.FILTER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.FILTER;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.FILTER;

                }
            }
        }

        /// <summary>
        /// center_x - see Gamestudio manual
        /// </summary>
        public Boolean CENTER_X
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.CENTER_X) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.CENTER_X;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.CENTER_X;

                }
            }
        }

        /// <summary>
        /// center_y - see Gamestudio manual
        /// </summary>
        public Boolean CENTER_Y
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.CENTER_Y) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.CENTER_Y;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.CENTER_Y;

                }
            }
        }

        /// <summary>
        /// aright - see Gamestudio manual
        /// </summary>
        public Boolean ARIGHT
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.ARIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.ARIGHT;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.ARIGHT;

                }
            }
        }

        /// <summary>
        /// wwrap - see Gamestudio manual
        /// </summary>
        public Boolean WWRAP
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.WWRAP) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.WWRAP;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.WWRAP;

                }
            }
        }

        /// <summary>
        /// condensed - see Gamestudio manual
        /// </summary>
        public Boolean CONDENSED
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.CONDENSED) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.CONDENSED;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.CONDENSED;

                }
            }
        }

        /// <summary>
        /// shadow - see Gamestudio manual
        /// </summary>
        public Boolean SHADOW
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.SHADOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.SHADOW;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.SHADOW;

                }
            }
        }

        /// <summary>
        /// outline - see Gamestudio manual
        /// </summary>
        public Boolean OUTLINE
        {
            get
            {
                unsafe
                {
                    if (((*TEXTPointer).flags & (int)FLAG.OUTLINE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*TEXTPointer).flags = (*TEXTPointer).flags | (int)FLAG.OUTLINE;
                    else
                        (*TEXTPointer).flags = (*TEXTPointer).flags & ~(int)FLAG.OUTLINE;

                }
            }
        }

        /// <summary>
        /// target_map - see Gamestudio manual
        /// </summary>
        public BMAP target_map
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*TEXTPointer).target_map;
                }
            }
            set
            {
                unsafe
                {
                    (*TEXTPointer).target_map = (NativeBMAP*)(IntPtr)value;
                }
            }
        }


        /// <summary>
        /// skill_x - see Gamestudio manual
        /// </summary>
        private Var skill_x_ = null;
        public Var skill_x
        {
            get
            {
                unsafe
                {
                    if (skill_x_ == null)
                    {
                        skill_x_ = (Var)(IntPtr)(&((*TEXTPointer).skill_x));
                    }
                }
                return skill_x_;
            }
        }


        /// <summary>
        /// skill_y - see Gamestudio manual
        /// </summary>
        private Var skill_y_ = null;
        public Var skill_y
        {
            get
            {
                unsafe
                {
                    if (skill_y_ == null)
                    {
                        skill_y_ = (Var)(IntPtr)(&((*TEXTPointer).skill_y));
                    }
                }
                return skill_y_;
            }
        }



        /* -------- TEXT -------- */
        /* TEXT to NativeTEXT* (address in void*) */
        public static explicit operator IntPtr(TEXT d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativeTEXT*)d.TEXTPointer;
                }
            }
        }
        /* NativeTEXT* (address in void*) to TEXT */
        public static explicit operator TEXT(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new TEXT(d);
            }
        }

    }
}