﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class Vector
    {
        /// <summary>
        /// the struct behind the Vector - DON'T CHANGE!
        /// </summary>
        internal NativeVector HiddenVector = new NativeVector();
        private IntPtr VectorIntPtr = IntPtr.Zero;

        protected Var X;
        protected Var Y;
        protected Var Z;

        public Var x
        {
            get
            {
                return X;
            }
            set
            {
                this.X = value;
            }
        }

        public Var y
        {
            get
            {
                return Y;
            }
            set
            {
                this.Y = value;
            }
        }

        public Var z
        {
            get
            {
                return Z;
            }
            set
            {
                this.Z = value;
            }
        }

        /// <summary>
        /// returns if this object uses a pointer - else it only stores a value
        /// </summary>
        public Boolean ISPointer()
        {
            if (VectorIntPtr == IntPtr.Zero)
                return false;
            else
                return true;
        }

        /// <summary>
        /// set the pointer of the current object - use carefully
        /// </summary>
        public void setPointer(IntPtr VecPtr)
        {
            VectorIntPtr = VecPtr;
        }

        /// <summary>
        /// set the pointer of the current object - use carefully
        /// </summary>
        public IntPtr getPointer()
        {
            return VectorIntPtr;
        }

        /// <summary>
        /// standard constructor
        /// </summary>
        public Vector(Var x, Var y, Var z)
        {
            this.X = x;
            this.Y = y;
            this.Z = z;
        }

        /// <summary>
        /// set the value of the current object's vars to the value of the object's vars given as parameter (only value no pointer or object is copied)
        /// </summary>
        public void getValueFrom(Vector tempVec)
        {
            X.getValueFrom(tempVec.X);
            Y.getValueFrom(tempVec.Y);
            Z.getValueFrom(tempVec.Z);
        }

        public static explicit operator Vector(IntPtr d)
        {
            Vector tempVec = new Vector((Var)0, (Var)0, (Var)0);
            unsafe
            {
                tempVec.X = (Var)(IntPtr)(&(*((NativeVector*)d)).x);
                tempVec.Y = (Var)(IntPtr)(&(*((NativeVector*)d)).y);
                tempVec.Z = (Var)(IntPtr)(&(*((NativeVector*)d)).z);
            }
            tempVec.VectorIntPtr = d;
            return tempVec;
        }

        public static explicit operator IntPtr(Vector d)   //return a pointer to the structure stored in unmanaged memory
        {
            if (d.VectorIntPtr == IntPtr.Zero)
            {
                d.VectorIntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(NativeVector)));
                d.HiddenVector.x = (NativeVar)d.X;
                d.HiddenVector.y = (NativeVar)d.Y;
                d.HiddenVector.z = (NativeVar)d.Z;
                Marshal.StructureToPtr(d.HiddenVector, d.VectorIntPtr, false);
            }
            return d.VectorIntPtr;
        }
    }           
}