﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using FXED.Classes.Helper;
using FXED.Classes.PropertySupport;
using System.Reflection;
using System.IO;
using AcknexWrapper;

namespace FXED.Classes.Objects
{
    [Serializable]
    public abstract class DynamicParent : IDynamicClass
    {
        private PropertyList public_properties = new PropertyList();

        public PropertyList PublicProperties
        {
            get { return public_properties; }
            set { public_properties = value; }
        }
    }

    [Serializable]
    public class Effect : DynamicParent
    {
        private bool use_fx_file = false;
        private string fx_code = string.Empty;

        // information

        private string author = string.Empty;
        private string description = string.Empty;
        private string shader_model = "2.0";
        private Bitmap thumbnail = null;
        
        // material settings

        private System.Drawing.Color ambient = System.Drawing.Color.FromArgb(200, 200, 200);
        private System.Drawing.Color diffuse = System.Drawing.Color.FromArgb(200, 200, 200);
        private System.Drawing.Color specular = System.Drawing.Color.FromArgb(0, 0, 0);
        private System.Drawing.Color emissive = System.Drawing.Color.FromArgb(0, 0, 0);
        private int power = 0;
        private int alpha = 100;
        private int albedo = 100;
        private int cycle = 480;
        private int lod = 0;

        // skins
        private Bitmap mtlskin1 = null;
        private string mtlskin1_format = string.Empty;

        private Bitmap mtlskin2 = null;
        private string mtlskin2_format = string.Empty;

        private Bitmap mtlskin3 = null;
        private string mtlskin3_format = string.Empty;

        private Bitmap mtlskin4 = null;
        private string mtlskin4_format = string.Empty;

        // flags

        private List<string> flags = new List<string>();
        private bool tangent = false;
        private bool translucent = false;
        private bool pass_solid = false;
        private bool auto_reload = false;
        private bool _override = false;
        private bool enable_render = false;
        private bool enable_tree = false;
        private bool enable_view = false;

        // shader components

        private List<Pass> passes = new List<Pass>();
        private List<ParentComponent> variables = new List<ParentComponent>();
        private List<ParentComponent> arrays = new List<ParentComponent>();
        private List<ParentComponent> vectors = new List<ParentComponent>();
        private List<ParentComponent> colors = new List<ParentComponent>();
        private List<ParentComponent> matrices = new List<ParentComponent>();
        private List<ParentComponent> textures = new List<ParentComponent>();
        private List<ParentComponent> samplers = new List<ParentComponent>();
        private List<ParentComponent> functions = new List<ParentComponent>();

        private List<string> includes = new List<string>();
        private List<Define> defines = new List<Define>();
        private List<Struct> structs = new List<Struct>();
        private SkillsSet skills = new SkillsSet();

        // form settings

        private Size preview_size = new Size(350, 350);
        private Point preview_location = new Point(656, 195);

        private Size skins_size = new Size(430, 130);
        private Point skins_location = new Point(0, 0);
        private bool skins_visible = false;

        private Size skills_size = new Size(230, 280);
        private Point skills_location = new Point(0, 0);
        private bool skills_visible = false;

        // preview settings

        PreviewSettings preview_settings = new PreviewSettings();

        // methods

        public Effect()
        {
            PublicProperties.Add(new DynamicProperty("AmbientColor", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("DiffuseColor", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("SpecularColor", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("EmissiveColor", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("Power", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("Alpha", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("Albedo", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("Cycle", "Material Settings"));
            PublicProperties.Add(new DynamicProperty("LOD", "Material Settings"));

            PublicProperties.Add(new DynamicProperty("Tangent", "Flags"));
            PublicProperties.Add(new DynamicProperty("Translucent", "Flags"));
            PublicProperties.Add(new DynamicProperty("PassSolid", "Flags"));
            PublicProperties.Add(new DynamicProperty("AutoReload", "Flags"));
            PublicProperties.Add(new DynamicProperty("Override", "Flags"));
            PublicProperties.Add(new DynamicProperty("EnableRender", "Flags"));
            PublicProperties.Add(new DynamicProperty("EnableTree", "Flags"));
            PublicProperties.Add(new DynamicProperty("EnableView", "Flags"));

            this.author = System.Environment.UserName;
        }

        public void CreateDefaultEffect(string type)
        {
            if (type == "entity") // entity
            {
                Matrix matworldviewproj = new Matrix("matWorldViewProj", true);
                matworldviewproj.SetComment("The product of the world, view, and projection matrix.");
                matrices.Add(matworldviewproj);

                Pass new_pass = new Pass("AmbientPass");
                new_pass.SetComment("Renders the object in a single color.");

                new_pass.VertexShader.Inputs.Add(new Parameter("in", "float4", "inPos", "POSITION"));
                new_pass.VertexShader.Outputs.Add(new Parameter("out", "float4", "outPos", "POSITION"));
                new_pass.VertexShader.SetCode("outPos = mul(inPos, matWorldViewProj);");

                new_pass.PixelShader.Outputs.Add(new Parameter("out", "float4", "outColor", "COLOR"));
                new_pass.PixelShader.SetCode("outColor = float4(1,0,0,1);");

                passes.Add(new_pass);
            }
            else // pp
            {
                Texture target_map = new Texture("TargetMap", true);
                target_map.SetComment("The render target of the current view.");
                textures.Add(target_map);
                Sampler texture_sampler = new Sampler("screen_sampler");
                texture_sampler.Texture = "TargetMap";
                samplers.Add(texture_sampler);

                Pass new_pass = new Pass("MonochromePass");
                new_pass.VertexShader = null;

                new_pass.PixelShader.Inputs.Add(new Parameter("in","float2", "inTex", "TEXCOORD"));
                new_pass.PixelShader.Outputs.Add(new Parameter("out", "float4", "outColor", "COLOR"));

                StringBuilder code = new StringBuilder();
                code.Append("float4 Color = tex2D(screen_sampler, inTex);\r\n");
                code.Append("Color.rgb = (Color.r+Color.g+Color.b)/3.0f;\r\n");
                code.Append("outColor = float4(2*Color.xyz,Color.w);");
                new_pass.PixelShader.SetCode(code.ToString());

                passes.Add(new_pass);
            }
        }

        public string GenerateEffect()
        {
            if (use_fx_file) { return fx_code; }

            StringBuilder temp = new StringBuilder();

            // write includes

            if (includes.Count != 0)
            {
                temp.Append("// included scripts\r\n");
                foreach (string include in includes)
                {
                    temp.Append("#include <" + include + ">\r\n");
                }
                temp.Append("\r\n");
            }

            // write defines

            if (defines.Count != 0)
            {
                temp.Append("// defines\r\n");
                foreach (Define define in defines)
                {
                    temp.Append("#define " + define.Name + ((define.Value != string.Empty) ? " " + define.Value : string.Empty) + "\r\n");
                }
                temp.Append("\r\n");
            }

            // write vars/vectors/etc

            if (variables.Count != 0)
            {
                temp.Append("// variables\r\n");
                foreach (ParentComponent variable in variables)
                {
                    temp.Append(variable.GenerateCode() + "\r\n");
                }
                temp.Append("\r\n");
            }

            if (Arrays.Count != 0)
            {
                 temp.Append("// arrays\r\n");
                foreach (ParentComponent array in Arrays)
                {
                     temp.Append(array.GenerateCode() + "\r\n");
                }
                 temp.Append("\r\n");
            }

            if (vectors.Count != 0)
            {
                 temp.Append("// vectors\r\n");
                foreach (ParentComponent vector in vectors)
                {
                     temp.Append(vector.GenerateCode() + "\r\n");
                }
                 temp.Append("\r\n");
            }

            if (colors.Count != 0)
            {
                 temp.Append("// colors\r\n");
                foreach (ParentComponent color in colors)
                {
                     temp.Append(color.GenerateCode() + "\r\n");
                }
                 temp.Append("\r\n");
            }

            if (matrices.Count != 0)
            {
                 temp.Append("// matrices\r\n");
                foreach (ParentComponent matrix in matrices)
                {
                     temp.Append(matrix.GenerateCode() + "\r\n");
                }
                 temp.Append("\r\n");
            }

            if (textures.Count != 0)
            {
                 temp.Append("// textures\r\n");
                foreach (ParentComponent texture in textures)
                {
                     temp.Append(texture.GenerateCode() + "\r\n");
                }
                 temp.Append("\r\n");
            }

            if (samplers.Count != 0)
            {
                temp.Append("// samplers\r\n");
                foreach (ParentComponent sampler in samplers)
                {
                     temp.Append(sampler.GenerateCode() + "\r\n");
                }
                 temp.Append("\r\n");
            }

            // write structs

            if (structs.Count != 0)
            {
                temp.Append("// custom structs\r\n");
                foreach (Struct str in structs)
                {
                    temp.Append(str.GenerateCode() + "\r\n");
                }
                temp.Append("\r\n");
            }

            if (functions.Count != 0)
            {
                temp.Append("// custom functions\r\n");
                foreach (ParentComponent function in functions)
                {
                     temp.Append(function.GenerateCode() + "\r\n");
                }
                 temp.Append("\r\n");
            }

            // generate vertex/pixel shaders

            temp.Append("// shader functions\r\n");

            foreach (Pass pass in passes)
            {
                if (pass.VertexShader != null)
                {
                     temp.Append(pass.VertexShader.GenerateCode(pass.Name));
                }

                if (pass.PixelShader != null)
                {
                     temp.Append(pass.PixelShader.GenerateCode(pass.Name));
                }
            }

            // write technique(s)

            temp.Append("// techniques\r\n");

            List<string> technique_names = new List<string>();
            List<int> pass_orders = new List<int>();
            foreach (Pass pass in passes) 
            {
                if (!technique_names.Contains(pass.TechniqueName)) { technique_names.Add(pass.TechniqueName); }
            }


            foreach (string technique in technique_names)
            {
                if (technique == "(default)")
                {
                    temp.Append("technique ShaderTechnique\r\n{\r\n");
                }
                else
                {
                    temp.Append("technique "+technique+"\r\n{\r\n");
                }

                foreach (Pass pass in passes.FindAll(delegate(Pass p){return p.TechniqueName == technique;}))
                { 
                    if (!pass_orders.Contains(pass.Order)) { pass_orders.Add(pass.Order); } 
                }
                pass_orders.Sort();

                foreach (int i in pass_orders)
                {
                    foreach (Pass p in passes.FindAll(delegate(Pass pass) { return pass.TechniqueName == technique && pass.Order == i; }))
                    {
                        temp.Append(p.GenerateCode());
                        temp.Append("\r\n");
                    }
                }
                temp.Append("}\r\n\r\n");
                pass_orders.Clear();
            }

            // finally write a fallback

            temp.Append("// fallback\r\n");
            temp.Append("technique fallback { pass one { } }");
            return temp.ToString();
        }

        public string GenerateMaterial()
        {
            StringBuilder result = new StringBuilder();

            string name = Classes.Variables.current_name.Remove(Classes.Variables.current_name.IndexOf('.'),Classes.Variables.current_name.Length - Classes.Variables.current_name.IndexOf('.')).Replace(" ","");

            result.Append("MATERIAL* mtl" + name + " =\r\n{\r\n");

            result.Append("effect = \"" + name + ".fx\";\r\n");

            if (ambient != System.Drawing.Color.FromArgb(200, 200, 200))
            {
                result.Append("ambient_red = " + ambient.R + ";\r\n"); result.Append("ambient_green = " + ambient.G + ";\r\n");
                result.Append("ambient_blue = " + ambient.B + ";\r\n");
            }

            if (diffuse != System.Drawing.Color.FromArgb(200, 200, 200))
            {
                result.Append("diffuse_red = " + diffuse.R + ";\r\n"); result.Append("diffuse_green = " + diffuse.G + ";\r\n");
                result.Append("diffuse_blue = " + diffuse.B + ";\r\n");
            }

            if (specular != System.Drawing.Color.FromArgb(0, 0, 0))
            {
                result.Append("specular_red = " + specular.R + ";\r\n"); result.Append("specular_green = " + specular.G + ";\r\n");
                result.Append("specular_blue = " + specular.B + ";\r\n");
            }

            if (emissive != System.Drawing.Color.FromArgb(0, 0, 0))
            {
                result.Append("emissive_red = " + emissive.R + ";\r\n"); result.Append("emissive_green = " + emissive.G + ";\r\n");
                result.Append("emissive_blue = " + emissive.B + ";\r\n");
            }

            if (power != 0) { result.Append("power = " + power + ";\r\n"); }
            if (alpha != 100) { result.Append("alpha = " + alpha + ";\r\n"); }
            if (albedo != 100) { result.Append("albedo = " + albedo + ";\r\n"); }
            if (cycle != 480) { result.Append("cycle = " + cycle + ";\r\n"); }

            if (flags.Count != 0)
            {
                result.Append("flags = ");
                for (int i = 0; i < flags.Count; i++)
                {
                    result.Append(flags[i]);
                    if ((i + 1) < flags.Count) { result.Append(","); }
                }
                result.Append(";\r\n");
            }

            result.Append("}\r\n\r\n");

            result.Append("action act" + name + "()\r\n{\r\n");

            result.Append("my.material = mtl" + name + ";\r\n");
            if (lod != 0) { result.Append("mtl" + name + ".lod = "+lod+";\r\n"); }

            result.Append(skills.GenerateCode(name));

            result.Append("}\r\n");

            return result.ToString();
        }

        // properties

        public bool UseFxFile
        {
            get { return use_fx_file; }
            set { use_fx_file = value; }
        }

        public string FxCode
        {
            get { return fx_code; }
            set { fx_code = value; }
        }

        public string Author
        {
            get { return author; }
            set { author = value; }
        }

        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        public string ShaderModel
        {
            get { return shader_model; }
            set { shader_model = value; }
        }

        public Bitmap Thumbnail
        {
            get { return thumbnail; }
            set { thumbnail = value; }
        }

        [DescriptionAttribute("The ambient colour of the material.")]
        public System.Drawing.Color AmbientColor
        {
            get { return ambient; }
            set 
            { 
                ambient = value; 
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetAmbientColor);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The diffuse colour of the material.")]
        public System.Drawing.Color DiffuseColor
        {
            get { return diffuse; }
            set 
            { 
                diffuse = value; 
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetDiffuseColor);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The specular colour of the material.")]
        public System.Drawing.Color SpecularColor
        {
            get { return specular; }
            set 
            { 
                specular = value; 
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetSpecularColor);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The emissive colour of the material.")]
        public System.Drawing.Color EmissiveColor
        {
            get { return emissive; }
            set 
            { 
                emissive = value; 
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetEmissiveColor);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The sharpness of specular highlights.")]
        public int Power
        {
            get { return power; }
            set 
            { 
                power = Classes.Functions.Clamp(value, 0, 10);
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetMaterialPower);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The opacity of the material.")]
        public int Alpha
        {
            get { return alpha; }
            set
            {
                alpha = Classes.Functions.Clamp(value, 0, 100);
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetMaterialAlpha);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The strength that sun light is reflected.")]
        public int Albedo
        {
            get { return albedo; }
            set 
            {
                albedo = Classes.Functions.Clamp(value, -100, 200);
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetMaterialAlbedo);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("Determines the time value in ticks for the 'vecCycle' vector.")]
        public int Cycle
        {
            get { return cycle; }
            set 
            {
                cycle = Classes.Functions.Clamp(value, 1, 999999);
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetMaterialCycle);
                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The LOD offset of the material.")]
        public int LOD
        {
            get { return lod; }
            set 
            { 
                lod = Classes.Functions.Clamp(value, 0, 3);
                Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.SetMaterialLOD);
                Classes.Functions.ProjectChanged(); 
            }
        }

        [DescriptionAttribute("Determines if tangents are calculated for the 3rd texture coordinate set.")]
        public bool Tangent
        {
            get { return tangent; }
            set 
            {
                tangent = value;
                if (tangent) { flags.Add("TANGENT"); } else { flags.Remove("TANGENT"); }
                //Classes.Variables.preview.SetMaterialFlag("TANGENT");
                Classes.Functions.ProjectChanged();
            }
        }

        public bool Translucent
        {
            get { return translucent; }
            set
            {
                translucent = value;
                if (translucent) { flags.Add("TRANSLUCENT"); } else { flags.Remove("TRANSLUCENT"); }
                Classes.Variables.preview.SetMaterialFlag("TRANSLUCENT");
                Classes.Functions.ProjectChanged();
            }
        }

        public bool PassSolid
        {
            get { return pass_solid; }
            set
            {
                pass_solid = value;
                if (pass_solid) { flags.Add("PASS_SOLID"); } else { flags.Remove("PASS_SOLID"); }
                Classes.Variables.preview.SetMaterialFlag("PASS_SOLID");
                Classes.Functions.ProjectChanged();
            }
        }

        public bool AutoReload
        {
            get { return auto_reload; }
            set
            {
                auto_reload = value;
                if (auto_reload) { flags.Add("AUTORELOAD"); } else { flags.Remove("AUTORELOAD"); }
                Classes.Functions.ProjectChanged();
            }
        }

        public bool Override
        {
            get { return _override; }
            set
            {
                _override = value;
                if (_override) { flags.Add("OVERRIDE"); } else { flags.Remove("OVERRIDE"); }
                Classes.Functions.ProjectChanged();
            }
        }

        public bool EnableRender
        {
            get { return enable_render; }
            set
            {
                enable_render = value;
                if (enable_render) { flags.Add("ENABLE_RENDER"); } else { flags.Remove("ENABLE_RENDER"); }
                Classes.Functions.ProjectChanged();
            }
        }

        public bool EnableTree
        {
            get { return enable_tree; }
            set
            {
                enable_tree = value;
                if (enable_tree) { flags.Add("ENABLE_TREE"); } else { flags.Remove("ENABLE_TREE"); }
                Classes.Functions.ProjectChanged();
            }
        }

        public bool EnableView
        {
            get { return enable_view; }
            set
            {
                enable_view = value;
                if (enable_view) { flags.Add("ENABLE_VIEW"); } else { flags.Remove("ENABLE_VIEW"); }
                Classes.Functions.ProjectChanged();
            }
        }

        public Bitmap MtlSkin1
        {
            get { return mtlskin1; }
            set { mtlskin1 = value; Classes.Functions.ProjectChanged(); }
        }

        public string MtlSkin1Format
        {
            get { return mtlskin1_format; }
            set { mtlskin1_format = value; Classes.Functions.ProjectChanged(); }
        }

        public Bitmap MtlSkin2
        {
            get { return mtlskin2; }
            set { mtlskin2 = value; Classes.Functions.ProjectChanged(); }
        }

        public string MtlSkin2Format
        {
            get { return mtlskin2_format; }
            set { mtlskin2_format = value; Classes.Functions.ProjectChanged(); }
        }

        public Bitmap MtlSkin3
        {
            get { return mtlskin3; }
            set { mtlskin3 = value; Classes.Functions.ProjectChanged(); }
        }

        public string MtlSkin3Format
        {
            get { return mtlskin3_format; }
            set { mtlskin3_format = value; Classes.Functions.ProjectChanged(); }
        }

        public Bitmap MtlSkin4
        {
            get { return mtlskin4; }
            set { mtlskin4 = value; Classes.Functions.ProjectChanged(); }
        }

        public string MtlSkin4Format
        {
            get { return mtlskin4_format; }
            set { mtlskin4_format = value; Classes.Functions.ProjectChanged(); }
        }

        public List<Pass> Passes
        {
            get { return passes; }
            set { passes = value; }
        }

        public List<ParentComponent> Variables
        {
            get { return variables; }
            set { variables = value; }
        }

        public List<ParentComponent> Arrays
        {
            get { return arrays; }
            set { arrays = value; }
        }

        public List<ParentComponent> Vectors
        {
            get { return vectors; }
            set { vectors = value; }
        }

        public List<ParentComponent> Colors
        {
            get { return colors; }
            set { colors = value; }
        }

        public List<ParentComponent> Matrices
        {
            get { return matrices; }
            set { matrices = value; }
        }

        public List<ParentComponent> Textures
        {
            get { return textures; }
            set { textures = value; }
        }

        public List<ParentComponent> Samplers
        {
            get { return samplers; }
            set { samplers = value; }
        }

        public List<ParentComponent> Functions
        {
            get { return functions; }
            set { functions = value; }
        }

        public List<string> Includes
        {
            get { return includes; }
            set { includes = value; }
        }

        public List<Define> Defines
        {
            get { return defines; }
            set { defines = value; }
        }

        public List<Struct> Structs
        {
            get { return structs; }
            set { structs = value; }
        }

        public SkillsSet Skills
        {
            get { return skills; }
            set { skills = value; }
        }

        public Size PreviewSize
        {
            get { return preview_size; }
            set { preview_size = value; Classes.Functions.ProjectChanged(); }
        }

        public Point PreviewLocation
        {
            get { return preview_location; }
            set { preview_location = value; Classes.Functions.ProjectChanged(); }
        }

        public Size SkinsSize
        {
            get { return skins_size; }
            set { skins_size = value; Classes.Functions.ProjectChanged(); }
        }

        public Point SkinsLocation
        {
            get { return skins_location; }
            set { skins_location = value; Classes.Functions.ProjectChanged(); }
        }

        public bool SkinsVisible
        {
            get { return skins_visible; }
            set { skins_visible = value; Classes.Functions.ProjectChanged(); }
        }

        public Size SkillsSize
        {
            get { return skills_size; }
            set { skills_size = value; Classes.Functions.ProjectChanged(); }
        }

        public Point SkillsLocation
        {
            get { return skills_location; }
            set { skills_location = value; Classes.Functions.ProjectChanged(); }
        }

        public bool SkillsVisible
        {
            get { return skills_visible; }
            set { skills_visible = value; Classes.Functions.ProjectChanged(); }
        }

        public PreviewSettings Settings
        {
            get{return preview_settings;}
            set{preview_settings = value;}
        }

    }

    [Serializable]
    public class PreviewSettings
    {
        private string model_file = "Data\\sphere.mdl";
        private byte[] embedded_model = null;

        private bool use_skybox = false;
        private string sky_image = string.Empty;
        private Bitmap embedded_sky = null;
        private System.Drawing.Color sky_color = System.Drawing.Color.Black;

        private Bitmap entskin1 = null;
        private string entskin1_format = string.Empty;

        private Bitmap entskin2 = null;
        private string entskin2_format = string.Empty;

        private Bitmap entskin3 = null;
        private string entskin3_format = string.Empty;

        private Bitmap entskin4 = null;
        private string entskin4_format = string.Empty;

        private int camera_x = -100, camera_y = 0, camera_z = 0;
        private int camera_pan = 0, camera_tilt = 0, camera_roll = 0;

        private int entity_x = 0, entity_y = 0, entity_z = 0;
        private int entity_pan = 0, entity_tilt = 0, entity_roll = 0;

        private int sun_brightness = 100;
        private int sun_azimuth = 0;
        private int sun_elevation = 30;
        private System.Drawing.Color sun_color = System.Drawing.Color.FromArgb(128, 128, 128);

        private List<Light> lights = new List<Light>();

        //methods

        public void UpdatePositions()
        {
            camera_x = EngVar.camera.x.IntValue;
            camera_y = EngVar.camera.y.IntValue;
            camera_z = EngVar.camera.z.IntValue;
            camera_pan = EngVar.camera.pan.IntValue;
            camera_tilt = EngVar.camera.tilt.IntValue;
            camera_roll = EngVar.camera.roll.IntValue;

            entity_pan = Classes.Variables.preview.Entity.pan.IntValue;
            entity_tilt = Classes.Variables.preview.Entity.tilt.IntValue;
            entity_roll = Classes.Variables.preview.Entity.roll.IntValue;
        }

        public void EmbedSky()
        {
            if (!use_skybox) { return; }

            if (Properties.Settings.Default.EmbedSky)
            {
                if (Path.GetExtension(sky_image) == ".tga" || Path.GetExtension(sky_image) == ".dds")
                {
                    embedded_sky = DevIL.DevIL.LoadBitmap(sky_image);
                }
                else
                {
                    embedded_sky = new Bitmap(sky_image);                 
                }
            }
            else
            {
                embedded_sky = null;
            }

        }

        public void EmbedModel()
        {
            if (Properties.Settings.Default.EmbedModel)
            {
                FileStream fs = new FileStream(model_file, FileMode.Open, FileAccess.Read);
                BinaryReader reader = new BinaryReader(fs);
                embedded_model = reader.ReadBytes((int)fs.Length);
                reader.Close();
                fs.Close();
            }
            else
            {
                embedded_model = null;
            }
        }

        // properties

        public string ModelFile
        {
            get { return model_file; }
            set { model_file = value; Classes.Functions.ProjectChanged(); }
        }

        public byte[] EmbeddedModel
        {
            get { return embedded_model; }
            set { embedded_model = value; }
        }

        public bool UseSkyBox
        {
            get { return use_skybox; }
            set { use_skybox = value; Classes.Functions.ProjectChanged(); }
        }

        public string SkyImage
        {
            get { return sky_image; }
            set { sky_image = value; Classes.Functions.ProjectChanged(); }
        }

        public Bitmap EmbeddedSky
        {
            get { return embedded_sky; }
            set { embedded_sky = value; }
        }

        public System.Drawing.Color SkyColor
        {
            get { return sky_color; }
            set { sky_color = value; }
        }

        public Bitmap EntSkin1
        {
            get { return entskin1; }
            set { entskin1 = value; Classes.Functions.ProjectChanged(); }
        }

        public string EntSkin1Format
        {
            get { return entskin1_format; }
            set { entskin1_format = value; }
        }

        public Bitmap EntSkin2
        {
            get { return entskin2; }
            set { entskin2 = value; Classes.Functions.ProjectChanged(); }
        }

        public string EntSkin2Format
        {
            get { return entskin2_format; }
            set { entskin2_format = value; }
        }

        public Bitmap EntSkin3
        {
            get { return entskin3; }
            set { entskin3 = value; Classes.Functions.ProjectChanged(); }
        }

        public string EntSkin3Format
        {
            get { return entskin3_format; }
            set { entskin3_format = value; }
        }

        public Bitmap EntSkin4
        {
            get { return entskin4; }
            set { entskin4 = value; Classes.Functions.ProjectChanged(); }
        }

        public string EntSkin4Format
        {
            get { return entskin4_format; }
            set { entskin4_format = value; }
        }

        public int CameraX
        {
            get { return camera_x; }
            set { camera_x = value; }
        }

        public int CameraY
        {
            get { return camera_y; }
            set { camera_y = value; }
        }

        public int CameraZ
        {
            get { return camera_z; }
            set { camera_z = value; }
        }

        public int CameraPan
        {
            get { return camera_pan; }
            set { camera_pan = value; }
        }

        public int CameraTilt
        {
            get { return camera_tilt; }
            set { camera_tilt = value; }
        }

        public int CameraRoll
        {
            get { return camera_roll; }
            set { camera_roll = value; }
        }

        public int EntityX
        {
            get { return entity_x; }
            set { entity_x = value; }
        }

        public int EntityY
        {
            get { return entity_y; }
            set { entity_y = value; }
        }

        public int EntityZ
        {
            get { return entity_z; }
            set { entity_z = value; }
        }

        public int EntityPan
        {
            get { return entity_pan; }
            set { entity_pan = value; }
        }

        public int EntityTilt
        {
            get { return entity_tilt; }
            set { entity_tilt = value; }
        }

        public int EntityRoll
        {
            get { return entity_roll; }
            set { entity_roll = value; }
        }

        public int SunBrightness
        {
            get { return sun_brightness; }
            set { sun_brightness = value; }
        }

        public int SunAzimuth
        {
            get { return sun_azimuth; }
            set { sun_azimuth = value; }
        }

        public int SunElevation
        {
            get { return sun_elevation; }
            set { sun_elevation = value; }
        }

        public System.Drawing.Color SunColor
        {
            get { return sun_color; }
            set { sun_color = value; }
        }

        public List<Light> Lights
        {
            get { return lights; }
            set { lights = value; }
        }
    }

    [Serializable]
    public class Skill
    {
        private float value = 0.0f, min = 0.0f, max = 100.0f;
        private string display_name = string.Empty;
        private string description = string.Empty;
        private int skill_index = 0, skill_number = 0;

        [Browsable(false)]
        public int SkillIndex
        {
            get { return skill_index; }
            set { skill_index = value; }
        }

        [Browsable(false)]
        public int SkillNumber
        {
            get { return skill_number; }
            set { skill_number = value; }
        }

        public Skill(int index, int number)
        {
            this.skill_index = index;
            this.skill_number = number;
            this.display_name = "Skill"+number;
        }

        [Description("The minimum value for the skill")]
        public float Min
        {
            get { return min; }
            set { min = Classes.Functions.Clamp(value,Single.MinValue,this.value); }
        }

        [Description("The maximum value for the skill")]
        public float Max
        {
            get { return max; }
            set { max = Classes.Functions.Clamp(value, this.value, Single.MaxValue); }
        }

        [Description("The value of the skill.")]
        [EditorAttribute(typeof(Classes.Helper.SkillValueEditor), typeof(UITypeEditor))]
        public float Value
        {
            get { return value; }
            set 
            { 
                this.value = Classes.Functions.Clamp(value,min,max); 
                Classes.Variables.preview.SetSkill(this.skill_number, value);
                Classes.Variables.skills.SkillsGrid.Refresh();
                Classes.Functions.ProjectChanged();
            }
        }

        [Description("A useful name to identify the skill.")]
        public string Caption
        {
            get { return display_name; }
            set 
            {
                if (Classes.Variables.my_effect.Skills.Skills.FindAll(delegate(Classes.Objects.Skill s) { return s.Caption == value; }).Count == 0)
                {
                    display_name = value;
                    Classes.Functions.SetDisplayName(Classes.Variables.my_effect.Skills, "Skill" + skill_number, value);
                    Classes.Variables.skills.SkillsGrid.Refresh();
                }
            }
        }

        [Description("A description of what the skill does.")]
        public string Description
        {
            get { return description; }
            set
            {
                description = value;
                Classes.Functions.SetDescription(Classes.Variables.my_effect.Skills, "Skill" + skill_number, value);
                Classes.Variables.skills.SkillsGrid.Refresh();
            }
        }
    }

    [Serializable]
    public class SkillsSet
    {
        private List<Skill> skills = new List<Skill>(28);

        [Browsable(false)]
        public List<Skill> Skills
        {
            get { return skills; }
            set { skills = value; }
        }

        public SkillsSet()
        {
            for (int i = 0; i < 28; i++)
            {
                if (i < 20)
                {
                    skills.Add(new Skill(i, i+1));
                }
                else
                {
                    skills.Add(new Skill(i, i+21));
                }
            }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill1.x in shaders.")]
        [DisplayName("Skill1")]
        public Skill Skill1
        {
            get { return skills[0]; }
            set { skills[0] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill1.y in shaders.")]
        [DisplayName("Skill2")]
        public Skill Skill2
        {
            get { return skills[1]; }
            set { skills[1] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill1.z in shaders.")]
        [DisplayName("Skill3")]
        public Skill Skill3
        {
            get { return skills[2]; }
            set { skills[2] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill1.w in shaders.")]
        [DisplayName("Skill4")]
        public Skill Skill4
        {
            get { return skills[3]; }
            set { skills[3] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill5.x in shaders.")]
        [DisplayName("Skill5")]
        public Skill Skill5
        {
            get { return skills[4]; }
            set { skills[4] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill5.y in shaders.")]
        [DisplayName("Skill6")]
        public Skill Skill6
        {
            get { return skills[5]; }
            set { skills[5] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill5.z in shaders.")]
        [DisplayName("Skill7")]
        public Skill Skill7
        {
            get { return skills[6]; }
            set { skills[6] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill5.w in shaders.")]
        [DisplayName("Skill8")]
        public Skill Skill8
        {
            get { return skills[7]; }
            set { skills[7] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill9.x in shaders.")]
        [DisplayName("Skill9")]
        public Skill Skill9
        {
            get { return skills[8]; }
            set { skills[8] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill9.y in shaders.")]
        [DisplayName("Skill10")]
        public Skill Skill10
        {
            get { return skills[9]; }
            set { skills[9] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill9.z in shaders.")]
        [DisplayName("Skill11")]
        public Skill Skill11
        {
            get { return skills[10]; }
            set { skills[10] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill9.w in shaders.")]
        [DisplayName("Skill12")]
        public Skill Skill12
        {
            get { return skills[11]; }
            set { skills[11] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill13.x in shaders.")]
        [DisplayName("Skill13")]
        public Skill Skill13
        {
            get { return skills[12]; }
            set { skills[12] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill13.y in shaders.")]
        [DisplayName("Skill14")]
        public Skill Skill14
        {
            get { return skills[13]; }
            set { skills[13] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill13.z in shaders.")]
        [DisplayName("Skill15")]
        public Skill Skill15
        {
            get { return skills[14]; }
            set { skills[14] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill13.w in shaders.")]
        [DisplayName("Skill16")]
        public Skill Skill16
        {
            get { return skills[15]; }
            set { skills[15] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill17.x in shaders.")]
        [DisplayName("Skill17")]
        public Skill Skill17
        {
            get { return skills[16]; }
            set { skills[16] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill17.y in shaders.")]
        [DisplayName("Skill18")]
        public Skill Skill18
        {
            get { return skills[17]; }
            set { skills[17] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill17.z in shaders.")]
        [DisplayName("Skill19")]
        public Skill Skill19
        {
            get { return skills[18]; }
            set { skills[18] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill17.w in shaders.")]
        [DisplayName("Skill20")]
        public Skill Skill20
        {
            get { return skills[19]; }
            set { skills[19] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill41.x in shaders.")]
        [DisplayName("Skill41")]
        public Skill Skill41
        {
            get { return skills[20]; }
            set { skills[20] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill41.y in shaders.")]
        [DisplayName("Skill42")]
        public Skill Skill42
        {
            get { return skills[21]; }
            set { skills[21] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill41.z in shaders.")]
        [DisplayName("Skill43")]
        public Skill Skill43
        {
            get { return skills[22]; }
            set { skills[22] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill41.w in shaders.")]
        [DisplayName("Skill44")]
        public Skill Skill44
        {
            get { return skills[23]; }
            set { skills[23] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill45.x in shaders.")]
        [DisplayName("Skill45")]
        public Skill Skill45
        {
            get { return skills[24]; }
            set { skills[24] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill45.y in shaders.")]
        [DisplayName("Skill46")]
        public Skill Skill46
        {
            get { return skills[25]; }
            set { skills[25] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill45.z in shaders.")]
        [DisplayName("Skill47")]
        public Skill Skill47
        {
            get { return skills[26]; }
            set { skills[26] = value; }
        }

        [TypeConverter(typeof(Classes.Helper.SkillExpandableConverter))]
        [Description("A general purpose variable.\r\nvecSkill45.w in shaders.")]
        [DisplayName("Skill48")]
        public Skill Skill48
        {
            get { return skills[27]; }
            set { skills[27] = value; }
        }

        public string GenerateCode(string name)
        {
            StringBuilder result = new StringBuilder();

            for (int i = 1; i <= skills.Count; i++)
            {
                if (skills[i - 1].Value == 0.0) { continue; }

                if (i <= 20)
                {
                    result.Append(name + ".skill" + i + " = floatv(" + skills[i - 1].Value + ");");
                }
                else
                {
                    result.Append("my.skill" + (i + 20) + " = floatv(" + skills[i - 1].Value + ");");
                }

                if (skills[i - 1].Caption != "Skill" + skills[i - 1].SkillNumber)
                {
                    result.Append(" // " + skills[i - 1].Caption);
                }
                result.Append("\r\n");
            }

            return result.ToString();
        }
    }

    [Serializable]
    public class Shader
    {
        private string type = string.Empty;
        private string model = string.Empty;
        private string code = string.Empty;
        private List<Parameter> inputs = new List<Parameter>();
        private List<Parameter> outputs = new List<Parameter>();

        bool visible = true;
        Size size = new Size(360, 260);
        Point location = new Point(0, 0);

        public Shader() { }

        public Shader(string type, string model, string code)
        {
            this.type = type;
            this.model = model;
            this.code = code;

            if(this.type == "Vertex")
            {
                this.location = new Point(140,77);
            }
            else
            {
                if (Classes.Variables.shader_type == "entity")
                {
                    this.location = new Point(117, 391);
                }
                else
                {

                }
            }
        }

        public string GenerateCode(string pass_name)
        {
            if (code == string.Empty) { return string.Empty; }
            
            string temp = string.Empty;
            string key = (type == "Vertex") ? "VS" : "PS";
            int counter = 1;

            temp += "void " + pass_name + "_" + key + "(";
            counter = 1;

            List<Parameter> parameters = new List<Parameter>();
            parameters.AddRange(inputs);
            parameters.AddRange(outputs);

            foreach (Parameter param in parameters)
            {
                temp += param.GenerateCode();
                if (counter < parameters.Count) { temp += ","; }
                counter++;
            }
            temp += ")\r\n{\r\n";
            temp += code;
            temp += "\r\n}\r\n\r\n";

            return temp;
        }

        public string Type
        {
            get { return type; }
            set { type = value; }
        }

        public string Model
        {
            get { return model; }
            set { model = value; Classes.Functions.ProjectChanged(); }
        }

        public string Code
        {
            get { return code; }
            set { code = value; Classes.Functions.ProjectChanged(); }
        }

        public List<Parameter> Inputs
        {
            get { return inputs; }
            set { inputs = value; }
        }

        public List<Parameter> Outputs
        {
            get { return outputs; }
            set { outputs = value; }
        }

        public bool Visible
        {
            get { return visible; }
            set { visible = value; Classes.Functions.ProjectChanged(); }
        }

        public Size Size
        {
            get { return size; }
            set { size = value; Classes.Functions.ProjectChanged(); }
        }

        public Point Location
        {
            get { return location; }
            set { location = value; Classes.Functions.ProjectChanged(); }
        }

        public void SetCode(string code) { this.code = code; }
    }

    [Serializable]
    public class Uniform
    {
        private readonly string name = string.Empty;
        private string type = string.Empty;
        private string value = string.Empty;
        
        public Uniform(string type, string name)
        {
            this.type = type;
            this.name = name;
            this.value = Classes.Functions.GetDefaultValue(type);
            if (type == "half" || type == "float") { this.value += "f"; }
        }

        [Browsable(false)]
        public string Type
        {
            get { return type; }
            set { type = value; this.value = Classes.Functions.GetDefaultValue(type); }
        }

        [Browsable(false)]
        public string Name
        {
            get { return name; }
        }

        public string Value
        {
            get { return this.value; }
            set
            {
                if (value != string.Empty)
                {
                    if (Classes.Functions.CheckCastIsValid(type, value))
                    {
                        this.value = value;
                    }
                    else
                    {
                        MessageBox.Show("The datatype of the uniform variable does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }
    }

    [Serializable]
    public class RenderStates
    {
        bool visible = false;
        Size size = new Size(350, 300);
        Point location = new Point(0, 0);

        string zenable = "True";
        string fillmode = "Solid";
        string shademode = "Gouraud";
        string zwriteenable = "True";
        string alphatestenable = "False";
        string lastpixel = "True";
        string srcblend = "One";
        string destblend = "Zero";
        string cullmode = "CCW";
        string zfunc = "LessEqual";
        int alpharef = 0;
        string alphafunc = "Always";
        string ditherenable = "False";
        string alphablendenable = "False";
        string fogenable = "False";
        string specularenable = "False";
        System.Drawing.Color fogcolor = System.Drawing.Color.Black;
        string fogtablemode = "None";
        float fogstart = 0.0f;
        float fogend = 1.0f;
        float fogdensity = 1.0f;
        string rangefogenable = "False";
        string stencilenable = "False";
        string stencilfail = "Keep";
        string stencilzfail = "Keep";
        string stencilpass = "Keep";
        string stencilfunc = "Always";
        int stencilref = 0;
        string stencilmask = "0xFFFFFFFF";
        string stencilwritemask = "0xFFFFFFFF";
        System.Drawing.Color texturefactor = System.Drawing.Color.White;
        string clipping = "True";
        string lighting = "True";
        System.Drawing.Color ambient = System.Drawing.Color.Black;
        string fogvertexmode = "None";
        string colorvertex = "True";
        string localviewer = "True";
        string normalizenormals = "False";
        string diffusematerialsource = "Color1";
        string specularmaterialsource = "Color2";
        string ambientmaterialsource = "Material";
        string emissivematerialsource = "Material";
        private string vertexblend = "Disable";
        private int clipplaneenable = 0;
        private float pointsize = 0.0f;
        private float pointsizemin = 1.0f;
        private string pointspriteenable = "False";
        private string pointscaleenable = "False";
        private float pointscale_a = 1.0f;
        private float pointscale_b = 1.0f;
        private float pointscale_c = 1.0f;
        private string multisampleantialias = "True";
        private string multisamplemask = "0xFFFFFFFF";
        private string patchedgestyle = "Discrete";
        private float pointsizemax = 64.0f;
        private string indexedvertexblendenable = "False";
        private string colorwriteenable = "0x0000000F";
        private float tweenfactor = 0.0f;
        private string blendop = "Add";
        private string positiondegree = "Cubic";
        private string normaldegree = "Linear";
        private string scissortestenable = "False";
        private int slopescaledepthbias = 0;
        private string antialiasedlineenable = "False";
        private float mintessellationlevel = 1.0f;
        private float maxtessellationlevel = 1.0f;
        private float adaptivetess_x = 0.0f;
        private float adaptivetess_y = 0.0f;
        private float adaptivetess_z = 0.0f;
        private float adaptivetess_w = 0.0f;
        private string enableadaptivetessellation = "False";
        private string twosidedstencilmode = "False";
        string ccw_stencilfail = "Keep";
        string ccw_stencilzfail = "Keep";
        string ccw_stencilpass = "Keep";
        string ccw_stencilfunc = "Always";
        private string colorwriteenable1 = "0x0000000F";
        private string colorwriteenable2 = "0x0000000F";
        private string colorwriteenable3 = "0x0000000F";
        private System.Drawing.Color blendfactor = System.Drawing.Color.White;
        private float srgbwriteenable = 0.0f;
        private float depthbias = 0.0f;
        private string seperatealphablendenable = "False";
        string srcblendalpha = "One";
        string destblendalpha = "Zero";
        private string blendopalpha = "Add";

        public string GenerateCode()
        {
            RenderStates default_states = new RenderStates();
            string temp = string.Empty;

            PropertyInfo[] properties = this.GetType().GetProperties();
            foreach (PropertyInfo p in properties)
            {
                if (p.Name != "Visible" && p.Name != "Size" && p.Name != "Location")
                {
                    if (default_states.GetType().GetProperty(p.Name).GetValue(default_states, null).ToString() != p.GetValue(this, null).ToString())
                    {
                        if (p.Name == "FogColor" || p.Name == "TextureFactor" || p.Name == "Ambient" || p.Name == "BlendFactor")
                        {
                            temp += p.Name.ToUpper() + " = 0x" +((System.Drawing.Color)p.GetValue(this, null)).ToArgb().ToString("X")+";\r\n";
                            continue;
                        }

                        if (p.Name == "StencilMask" || p.Name == "StencilWriteMask" || p.Name == "MultiSampleMask" || p.Name == "ColorWriteEnable" || p.Name == "ColorWriteEnable1" || p.Name == "ColorWriteEnable2" || p.Name == "ColorWriteEnable3")
                        {
                            temp += p.Name.ToUpper() + " = " + p.GetValue(this, null).ToString() + ";\r\n";
                            continue;
                        }

                        temp += p.Name.ToUpper() + " = " + p.GetValue(this, null).ToString().ToUpper() + ";\r\n";
                    }
                }
            }

            return temp;
        }

        [Browsable(false)]
        public bool Visible
        {
            get { return visible; }
            set { visible = value; Classes.Functions.ProjectChanged(); }
        }

        [Browsable(false)]
        public Size Size
        {
            get { return size; }
            set { size = value; Classes.Functions.ProjectChanged(); }
        }

        [Browsable(false)]
        public Point Location
        {
            get { return location; }
            set { location = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The depth buffering used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string ZEnable
        {
            get { return zenable; }
            set { zenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The fill mode of the geometry.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string FillMode
        {
            get { return fillmode; }
            set { fillmode = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The shading applied to the geometry.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string ShadeMode
        {
            get { return shademode; }
            set { shademode = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if the depth buffer can be written to.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string ZWriteEnable
        {
            get { return zwriteenable; }
            set { zwriteenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if an per-pixel alpha test is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string AlphaTestEnable
        {
            get { return alphatestenable; }
            set { alphatestenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if the last pixel in a line is drawn.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string LastPixel
        {
            get { return lastpixel; }
            set { lastpixel = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The source blending mode used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string SrcBlend
        {
            get { return srcblend; }
            set { srcblend = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The destination blending mode used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string DestBlend
        {
            get { return destblend; }
            set { destblend = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if and how back-facing triangles are culled.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string CullMode
        {
            get { return cullmode; }
            set { cullmode = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The comparison mode to accept or reject pixels based on the depth value.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string ZFunc
        {
            get { return zfunc; }
            set { zfunc = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The alpha value used when alpha testing is enabled.")]
        public int AlphaRef
        {
            get { return alpharef; }
            set { alpharef = Functions.Clamp(value, 0, 255); }
        }

        [DescriptionAttribute("The comparison mode to accept or reject pixels based on the alpha value.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string AlphaFunc
        {
            get { return alphafunc; }
            set { alphafunc = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if dithering is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string DitherEnable
        {
            get { return ditherenable; }
            set { ditherenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if alpha-blended transparency is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string AlphaBlendEnable
        {
            get { return alphablendenable; }
            set { alphablendenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if fog blending is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string FogEnable
        {
            get { return fogenable; }
            set { fogenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if specular highlights are used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string SpecularEnable
        {
            get { return specularenable; }
            set { specularenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The color used for fog.")]
        public System.Drawing.Color FogColor
        {
            get { return fogcolor; }
            set { fogcolor = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The formula used to calculate pixel fog.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string FogTableMode
        {
            get { return fogtablemode; }
            set { fogtablemode = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The depth at which pixel or vertex fog effects begin.")]
        public float FogStart
        {
            get { return fogstart; }
            set { fogstart = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The depth at which pixel or vertex fog effects end.")]
        public float FogEnd
        {
            get { return fogend; }
            set { fogend = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The Fog density used in the exponential fog modes.")]
        public float FogDensity
        {
            get { return fogdensity; }
            set { fogdensity = Functions.Clamp(value, 0.0f, 1.0f); }
        }

        [DescriptionAttribute("Determines if range-based vertex fog is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string RangeFogEnable
        {
            get { return rangefogenable; }
            set { rangefogenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if stenciling is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string StencilEnable
        {
            get { return stencilenable; }
            set { stencilenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The stencil operation used when the stencil test fails.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string StencilFail
        {
            get { return stencilfail; }
            set { stencilfail = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The stencil operation used when both the stencil test and depth test fail.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string StencilZFail
        {
            get { return stencilzfail; }
            set { stencilzfail = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The stencil operation used when both the stencil test and depth test pass.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string StencilPass
        {
            get { return stencilpass; }
            set { stencilpass = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The comparison mode to accept or reject pixels based on the stencil value.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string StencilFunc
        {
            get { return stencilfunc; }
            set { stencilfunc = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("A reference value used for the stencil test.")]
        public int StencilRef
        {
            get { return stencilref; }
            set { stencilref = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The mask applied to the reference value and stencil buffer entries.")]
        public string StencilMask
        {
            get { return stencilmask; }
            set { stencilmask = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The mask applied to values written into the stencil buffer.")]
        public string StencilWriteMask
        {
            get { return stencilwritemask; }
            set { stencilwritemask = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The color used for multiple-texture blending.")]
        public System.Drawing.Color TextureFactor
        {
            get { return texturefactor; }
            set { texturefactor = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if Direct3D primitive clipping is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string Clipping
        {
            get { return clipping; }
            set { clipping = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if Direct3D lighting is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string Lighting
        {
            get { return lighting; }
            set { lighting = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The ambient light color.")]
        public System.Drawing.Color Ambient
        {
            get { return ambient; }
            set { ambient = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The formula used to calculate vertex fog.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string FogVertexMode
        {
            get { return fogvertexmode; }
            set { fogvertexmode = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if per-vertex coloring is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string ColorVertex
        {
            get { return colorvertex; }
            set { colorvertex = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if specular highlights are camera-relative or orthagonal.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string LocalViewer
        {
            get { return localviewer; }
            set { localviewer = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if vertex normals are automatically normalized.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string NormalizeNormals
        {
            get { return normalizenormals; }
            set { normalizenormals = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The diffuse color source for lighting calculations.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string DiffuseMaterialSource
        {
            get { return diffusematerialsource; }
            set { diffusematerialsource = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The specular color source for lighting calculations.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string SpecularMaterialSource
        {
            get { return specularmaterialsource; }
            set { specularmaterialsource = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The ambient color source for lighting calculations.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string AmbientMaterialSource
        {
            get { return ambientmaterialsource; }
            set { ambientmaterialsource = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The emissive color source for lighting calculations.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string EmissiveMaterialSource
        {
            get { return emissivematerialsource; }
            set { emissivematerialsource = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The number of matrices used to perform geometry blending, if any.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string VertexBlend
        {
            get { return vertexblend; }
            set { vertexblend = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Enables or disables user-defined clipping planes.")]
        public int ClipPlaneEnable
        {
            get { return clipplaneenable; }
            set { clipplaneenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The value that specifies the size to use for point size computation.")]
        public float PointSize
        {
            get { return pointsize; }
            set { pointsize = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The value that specifies the minimum size of point primitives.")]
        public float PointSizeMin
        {
            get { return pointsizemin; }
            set { pointsizemin = Functions.Clamp(value, 0.0f, float.MaxValue); }
        }

        [DescriptionAttribute("Determines if full textures are mapped on each point.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string PointSpriteEnable
        {
            get { return pointspriteenable; }
            set { pointspriteenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if point sizes are treated as camera space or screen space values.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string PointScaleEnable
        {
            get { return pointscaleenable; }
            set { pointscaleenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The value that controls distance-based size attenuation for point primitives.")]
        public float PointScale_A
        {
            get { return pointscale_a; }
            set { pointscale_a = Functions.Clamp(value, 0.0f, float.MaxValue); }
        }

        [DescriptionAttribute("The value that controls distance-based size attenuation for point primitives.")]
        public float PointScale_B
        {
            get { return pointscale_b; }
            set { pointscale_b = Functions.Clamp(value, 0.0f, float.MaxValue); }
        }

        [DescriptionAttribute("The value that controls distance-based size attenuation for point primitives.")]
        public float PointScale_C
        {
            get { return pointscale_c; }
            set { pointscale_c = Functions.Clamp(value, 0.0f, float.MaxValue); }
        }

        [DescriptionAttribute("Determines how individual samples are computed when using a multisample render-target buffer.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string MultiSampleAntiAlias
        {
            get { return multisampleantialias; }
            set { multisampleantialias = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Controls modification of one of the samples in a multisample render target.")]
        public string MultiSampleMask
        {
            get { return multisamplemask; }
            set { multisamplemask = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Sets whether patch edges will use float style tessellation.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string PatchEdgeStyle
        {
            get { return patchedgestyle; }
            set { patchedgestyle = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The value that specifies the maximum size of point primitives.")]
        public float PointSizeMax
        {
            get { return pointsizemax; }
            set { pointsizemax = Functions.Clamp(value, pointsizemin, float.MaxValue); }
        }

        [DescriptionAttribute("Determines if indexed vertex blending is enabled.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string IndexedVertexBlendEnable
        {
            get { return indexedvertexblendenable; }
            set { indexedvertexblendenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("A value that enables a per-channel write for the render-target color buffer.")]
        public string ColorWriteEnable
        {
            get { return colorwriteenable; }
            set { colorwriteenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("A value that controls the tween factor.")]
        public float TweenFactor
        {
            get { return tweenfactor; }
            set { tweenfactor = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The arithmetic operation applied when alpha blending is enabled.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string BlendOP
        {
            get { return blendop; }
            set { blendop = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The N-patch position interpolation degree.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string PositionDegree
        {
            get { return positiondegree; }
            set { positiondegree = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The N-patch normal interpolation degree.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string NormalDegree
        {
            get { return normaldegree; }
            set { normaldegree = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if scissor testing is performed.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string ScissorTestEnable
        {
            get { return scissortestenable; }
            set { scissortestenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determine how much bias can be applied to co-planar primitives to reduce z-fighting.")]
        public int SlopeScaleDepthBias
        {
            get { return slopescaledepthbias; }
            set { slopescaledepthbias = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Determines if line antialiasing is enabled.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string AntiAliasedLineEnable
        {
            get { return antialiasedlineenable; }
            set { antialiasedlineenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The minimum tessellation level used.")]
        public float MinTessellationLevel
        {
            get { return mintessellationlevel; }
            set { mintessellationlevel = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The maximum tessellation level used.")]
        public float MaxTessellationLevel
        {
            get { return maxtessellationlevel; }
            set { maxtessellationlevel = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The amount to adaptively tessellate, in the x direction.")]
        public float AdaptiveTess_X
        {
            get { return adaptivetess_x; }
            set { adaptivetess_x = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The amount to adaptively tessellate, in the y direction.")]
        public float AdaptiveTess_Y
        {
            get { return adaptivetess_y; }
            set { adaptivetess_y = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The amount to adaptively tessellate, in the z direction.")]
        public float AdaptiveTess_Z
        {
            get { return adaptivetess_z; }
            set { adaptivetess_z = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The amount to adaptively tessellate, in the w direction.")]
        public float AdaptiveTess_W
        {
            get { return adaptivetess_w; }
            set { adaptivetess_w = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Detemines if adaptive tessellation is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string EnableAdaptiveTessellation
        {
            get { return enableadaptivetessellation; }
            set { enableadaptivetessellation = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Detemines if two-sided stenciling is used.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string TwoSidedStencilMode
        {
            get { return twosidedstencilmode; }
            set { twosidedstencilmode = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The stencil operation used when the CCW stencil test fails.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string CCW_StencilFail
        {
            get { return ccw_stencilfail; }
            set { ccw_stencilfail = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The stencil operation used when both the CCW stencil test and depth test fail.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string CCW_StencilZFail
        {
            get { return ccw_stencilzfail; }
            set { ccw_stencilzfail = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The stencil operation used when both the CCW stencil test and depth test pass.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string CCW_StencilPass
        {
            get { return ccw_stencilpass; }
            set { ccw_stencilpass = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The comparison mode to accept or reject pixels based on the CCW stencil value.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string CCW_StencilFunc
        {
            get { return ccw_stencilfunc; }
            set { ccw_stencilfunc = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Additional ColorWriteEnable values for the device.")]
        public string ColorWriteEnable1
        {
            get { return colorwriteenable1; }
            set { colorwriteenable1 = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Additional ColorWriteEnable values for the device.")]
        public string ColorWriteEnable2
        {
            get { return colorwriteenable2; }
            set { colorwriteenable2 = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Additional ColorWriteEnable values for the device.")]
        public string ColorWriteEnable3
        {
            get { return colorwriteenable3; }
            set { colorwriteenable3 = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The color used for a constant blend-factor during alpha blending.")]
        public System.Drawing.Color BlendFactor
        {
            get { return blendfactor; }
            set { blendfactor = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Enables render-target writes to be gamma corrected to sRGB.")]
        public float SRGBWriteEnable
        {
            get { return srgbwriteenable; }
            set { srgbwriteenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("A value used for comparison of depth values.")]
        public float DepthBias
        {
            get { return depthbias; }
            set { depthbias = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("Enables a separate blend mode for the alpha channel.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string SeperateAlphaBlendEnable
        {
            get { return seperatealphablendenable; }
            set { seperatealphablendenable = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The source blending mode used for the alpha channel.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string SrcBlendAlpha
        {
            get { return srcblendalpha; }
            set { srcblendalpha = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The destination blending mode used for the alpha channel.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string DestBlendAlpha
        {
            get { return destblendalpha; }
            set { destblendalpha = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The arithmetic operation applied when seperate alpha blending is enabled.")]
        [TypeConverter(typeof(Helper.RenderStates))]
        public string BlendOPAlpha
        {
            get { return blendopalpha; }
            set { blendopalpha = value; Classes.Functions.ProjectChanged(); }
        }

    }

    [Serializable]
    public class Parameter
    {
        string mode = string.Empty;
        string data_type = "float4";
        string name = string.Empty;
        string semantic = string.Empty;
        string index = "0";
        bool uniform = false;

        public Parameter()
        {
            List<Parameter> aggregated_params = new List<Parameter>();
            aggregated_params.AddRange(Classes.Variables.current_shader.Inputs);
            aggregated_params.AddRange(Classes.Variables.current_shader.Outputs);

            for (int i = 1; i < Classes.Variables.copy_check_count; i++)
            {
                if (aggregated_params.Find(delegate(Parameter param) { return param.Name == "Parameter" + i; }) == null)
                {
                    name = "Parameter" + i; break;
                }
            }

            if (Classes.Variables.current_shader.Type == "Vertex")
            {
                semantic = "POSITION";
            }
            else
            {
                semantic = "COLOR";
            }
        }

        public Parameter(string mode, string type, string name, string semantic)
        {
            this.mode = mode;
            this.data_type = type;
            this.name = name;
            this.semantic = semantic;
        }

        public string GenerateCode()
        {
            if (Classes.Variables.my_effect.Structs.Find(delegate(Struct str) { return str.Name == data_type; }) != null)
            {
                return mode + " " + data_type + " " + name;
            }
            else
            {
                if (uniform)
                {
                    return "uniform " + data_type + " " + name;
                }
                else
                {
                    return mode + " " + data_type + " " + name + " : " + semantic + index;
                }
            }
        }

        [Browsable(false)]
        public string Mode
        {
            get { return mode; }
            set { mode = value; }
        }

        [Description("The data type of the shader parameter.")]
        [TypeConverter(typeof(Classes.Helper.DataTypes))]
        public string DataType
        {
            get { return data_type; }
            set 
            { 
                data_type = value;
                if (Classes.Variables.my_effect.Structs.Find(delegate(Struct str){return str.Name == value;}) != null)
                {
                    Classes.Functions.SetBrowsable(this, "Semantic", false);
                    Classes.Functions.SetBrowsable(this, "Index", false);
                    Classes.Functions.SetBrowsable(this, "Uniform", false);
                    uniform = false;
                }
                else
                {
                    Classes.Functions.SetBrowsable(this, "Semantic", true);
                    Classes.Functions.SetBrowsable(this, "Index", true);
                    Classes.Functions.SetBrowsable(this, "Uniform", true);
                }
            }
        }

        [Description("The name of the shader parameter.")]
        public string Name
        {
            get { return name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "parameter"))
                {
                    bool exists = false;

                    while (true)
                    {
                        if (Classes.Variables.current_shader.Inputs.Find(delegate(Parameter param) { return param.Name == value; }) != null)
                        {
                            exists = true; break;
                        }

                        if (Classes.Variables.current_shader.Outputs.Find(delegate(Parameter param) { return param.Name == value; }) != null)
                        {
                            exists = true; break;
                        }

                        break;
                    }

                    if (exists)
                    {
                        MessageBox.Show("A parameter with this name already exists.\r\nPlease rename the current parameter.", "Duplicate parameter name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                    else
                    {
                        name = value;
                        Classes.Functions.ProjectChanged();
                    }
                }
            }
        }

        [Description("The intended usage of the shader parameter.")]
        [TypeConverter(typeof(Classes.Helper.ParameterSemantics))]
        [Browsable(true)]
        public string Semantic
        {
            get { return semantic; }
            set { semantic = value; }
        }

        [Description("The channel of the parameter semantic.")]
        [TypeConverter(typeof(Classes.Helper.ParameterIndices))]
        [Browsable(true)]
        public string Index
        {
            get { return index; }
            set { index = value; }
        }

        [Browsable(true)]
        [Description("Sets the current parameter as a uniform variable.")]
        public bool Uniform
        {
            get { return uniform; }
            set
            {
                uniform = value;
                if (uniform)
                {
                    Classes.Functions.SetBrowsable(this, "Semantic", false);
                    Classes.Functions.SetBrowsable(this, "Index", false);
                }
                else
                {
                    Classes.Functions.SetBrowsable(this, "Semantic", true);
                    Classes.Functions.SetBrowsable(this, "Index", true);
                }
            }
        }

    }

    [Serializable]
    public class Define
    {
        private string name = string.Empty;
        private string value = string.Empty;

        public Define()
        {
            name = Functions.GetNewName("Define");
        }

        [Description("The name of the define.")]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        [Description("The value to replace all instances of the defined name.")]
        public string Value
        {
            get { return value; }
            set { this.value = value; }
        }
    }

    [Serializable]
    public class StructMember
    {
        private string data_type = "float4";
        private string semantic = "POSITION";
        private string index = "0";
        private string name = string.Empty;

        public StructMember()
        {
            for (int i = 1; i < Classes.Variables.copy_check_count; i++)
            {
                if (Classes.Variables.current_struct.Members.Find(delegate(Classes.Objects.StructMember mem) { return mem.Name == "Member" + i; }) == null)
                {
                    name = "Member" + i; break;
                }
            }
        }

        public string GenerateCode()
        {
            if (Classes.Variables.my_effect.Structs.Find(delegate(Struct str) { return str.Name == data_type; }) != null)
            {
                return data_type + " " + name+";";
            }
            else
            {
                return data_type + " " + name + " : " + semantic + index + ";";
            }
        }

        [TypeConverter(typeof(Classes.Helper.DataTypes))]
        public string DataType
        {
            get { return data_type; }
            set 
            { 
                data_type = value;
                if (Classes.Variables.my_effect.Structs.Find(delegate(Struct str) { return str.Name == value; }) != null)
                {
                    Classes.Functions.SetBrowsable(this, "Semantic", false);
                    Classes.Functions.SetBrowsable(this, "Index", false);
                }
                else
                {
                    Classes.Functions.SetBrowsable(this, "Semantic", true);
                    Classes.Functions.SetBrowsable(this, "Index", true);
                }
            }
        }

        [Description("The name of the struct member.")]
        public string Name
        {
            get { return name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "struct member"))
                {
                    if (Classes.Variables.current_struct.Members.Find(delegate(Classes.Objects.StructMember mem) { return mem.Name == value; }) != null)
                    {
                        MessageBox.Show("A member with this name already exists in the current struct.\r\nPlease rename the current member.", "Duplicate member name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                    else
                    {
                        name = value;
                        Classes.Functions.ProjectChanged();
                    }
                }
            }
        }

        [Description("The intended usage of the struct member.")]
        [TypeConverter(typeof(Classes.Helper.StructSemantics))]
        [Browsable(true)]
        public string Semantic
        {
            get { return semantic; }
            set { semantic = value; }
        }

        [Description("The channel of the member semantic.")]
        [TypeConverter(typeof(Classes.Helper.ParameterIndices))]
        [Browsable(true)]
        public string Index
        {
            get { return index; }
            set { index = value; }
        }
    }

    [Serializable]
    public class Struct
    {
        private string name = string.Empty;
        private List<StructMember> members = new List<StructMember>();

        public Struct()
        {
            this.name = Classes.Functions.GetNewName("Struct");
        }

        public string GenerateCode()
        {
            StringBuilder result = new StringBuilder();
            result.Append("struct " + name + "\r\n{\r\n");

            foreach (StructMember member in members)
            {
                result.Append(member.GenerateCode() + "\r\n");
            }
            result.Append("};");
            return result.ToString();

        }

        [DescriptionAttribute("The name of the struct.")]
        public string Name
        {
            get { return name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "struct"))
                {
                    string old_name = name;
                    name = value;
                    Classes.Functions.UpdateReferences("Struct", old_name, value);
                    Classes.Functions.ProjectChanged();
                }
            }
        }

        [DescriptionAttribute("The members of the struct.")]
        [EditorAttribute(typeof(Classes.Helper.StructMemberCollectionEditor), typeof(UITypeEditor))]
        public List<StructMember> Members
        {
            get { return members; }
            set { members = value; }
        }
    }

    [Serializable]
    public class Light : DynamicParent
    {
        private string name = string.Empty;
        private int range = 200;
        private System.Drawing.Color color = System.Drawing.Color.Yellow;
        private int x = 0, y = 0, z = 0;

        public Light()
        {
            PublicProperties.Add(new DynamicProperty("Range", "Light Settings"));
            PublicProperties.Add(new DynamicProperty("Color", "Light Settings"));
            this.name = Classes.Functions.GetNewName("Light");
        }

        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        [Description("The range of the dynamic light.")]
        public int Range
        {
            get { return range; }
            set 
            {   
                range = value; 
                Classes.Variables.preview.SetLightRange(name,range);
                Classes.Functions.ProjectChanged();
            }
        }

        [Description("The color of the dynamic light.")]
        public System.Drawing.Color Color
        {
            get { return color; }
            set 
            { 
                color = value; 
                Classes.Variables.preview.SetLightColor(name, color.R, color.G, color.B);
                Classes.Functions.ProjectChanged();
            }
        }

        public void UpdatePosition(int x, int y, int z)
        {
            this.x = x;
            this.y = y;
            this.z = z;
        }

        public int X
        {
            get { return x; }
            set { x = value; }
        }

        public int Y
        {
            get { return y; }
            set { y = value; }
        }

        public int Z
        {
            get { return z; }
            set { z = value; }
        }
    }

    public class LibraryItem
    {
        private string name = string.Empty;
        private string author = string.Empty;
        private string description = string.Empty;
        private string shader_model = string.Empty;
        
        public LibraryItem (string name,string author, string description, string shader_model)
	    {
            this.name = name;
            this.author = author;
            this.description = description;
            this.shader_model = shader_model;
	    }

        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        public string Author
        {
            get { return author; }
            set { author = value; }
        }

        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        public string ShaderModel
        {
            get { return shader_model; }
            set { shader_model= value; }
        }

    }

    // Components

    public enum BasicDataTypes { @bool, @int, half, @float, @double };

    [Serializable]
    public abstract class ParentComponent : DynamicParent
    {
        private string name = string.Empty;
        private string comment = string.Empty;
        private bool predefined = false;
        
        public ParentComponent() 
        {
            PublicProperties.Add(new DynamicProperty("Name", "General Settings"));
            PublicProperties.Add(new DynamicProperty("Comment", "General Settings"));
        }

        public ParentComponent(string name) : this()
        {
            this.name = name;
        }

        public ParentComponent(string name, bool predefined) : this(name)
        {
            this.predefined = predefined;
        }

        public virtual string GenerateCode() { return string.Empty; }

        public void SetName(string name) { this.name = name; }

        public void SetComment(string comment) { this.comment = comment; }

        public virtual string Name
        {
            get { return name; }
            set
            {
                name = value; 
            }
        }

        [DescriptionAttribute("A helper comment for the component.")]
        public string Comment
        {
            get { return comment; }
            set { comment = value; Classes.Variables.main_form.SetComponentTooltip(name, value); }
        }

        public bool PreDefined
        {
            get { return predefined; }
            set { predefined = value; }
        }
    }

    [Serializable]
    public class Pass : ParentComponent
    {
        private string technique_name = "(default)";
        private int order = 0;

        private Shader vertex_shader = new Shader("Vertex", "2.0", string.Empty);
        private List<Uniform> vertex_uniforms = new List<Uniform>();
        private ReadOnlyCollection<Uniform> readonly_vertex_uniforms;
        private string vertex_link = "(none)";
        private string vertex_link_model = string.Empty;

        private Shader pixel_shader = new Shader("Pixel", "2.0", string.Empty);
        private List<Uniform> pixel_uniforms = new List<Uniform>();
        private ReadOnlyCollection<Uniform> readonly_pixel_uniforms;
        private string pixel_link = "(none)";
        private string pixel_link_model = string.Empty;

        private RenderStates states = new RenderStates();

        //methods

        public Pass(string name) : base(name, false)
        {
           order = Classes.Variables.my_effect.Passes.Count + 1;
           PublicProperties.Add(new DynamicProperty("TechniqueName", "Pass Settings"));
           PublicProperties.Add(new DynamicProperty("Order", "Pass Settings"));

            if (Classes.Variables.shader_type == "entity")
            {
                PublicProperties.Add(new DynamicProperty("VertexLink", "Vertex Shader"));
            }
            PublicProperties.Add(new DynamicProperty("PixelLink", "Pixel Shader"));

            readonly_vertex_uniforms = new ReadOnlyCollection<Uniform>(vertex_uniforms);
            readonly_pixel_uniforms = new ReadOnlyCollection<Uniform>(pixel_uniforms);
        }

        public override string GenerateCode()
        {
            StringBuilder temp = new StringBuilder();

            temp.Append("pass " + base.Name + "\r\n{\r\n");
            temp.Append(states.GenerateCode());

            if (vertex_link == "(none)")
            {
                if (vertex_shader == null || vertex_shader.Code == string.Empty)
                {
                    temp.Append("VertexShader = null;\r\n");
                }
                else
                {
                    temp.Append("VertexShader = compile " + Classes.Functions.GetShaderModel(vertex_shader.Type, vertex_shader.Model) + " " + base.Name + "_VS(" + GenerateUniforms(this, "Vertex") + ");\r\n");
                }
            }
            else
            {
                Pass link = Classes.Variables.my_effect.Passes.Find(delegate(Pass p) { return p.Name == vertex_link; });
                if (link.VertexShader == null || link.VertexShader.Code == string.Empty)
                {
                    temp.Append("VertexShader = null;\r\n");
                }
                else
                {
                    temp.Append("VertexShader = compile " + Classes.Functions.GetShaderModel(link.VertexShader.Type, vertex_link_model) + " " + link.Name + "_VS(" + GenerateUniforms(link, "Vertex") + ");\r\n");
                }
            }

            if (pixel_link == "(none)")
            {
                if (pixel_shader.Code == string.Empty)
                {
                    temp.Append("PixelShader = null;\r\n");
                }
                else
                {
                    temp.Append("PixelShader = compile " + Classes.Functions.GetShaderModel(pixel_shader.Type, pixel_shader.Model) + " " + base.Name + "_PS(" + GenerateUniforms(this, "Pixel") + ");\r\n");
                }
            }
            else
            {
                Pass link = Classes.Variables.my_effect.Passes.Find(delegate(Pass p) { return p.Name == pixel_link; });
                if (link.PixelShader.Code == string.Empty)
                {
                    temp.Append("PixelShader = null;\r\n");
                }
                else
                {
                    temp.Append("PixelShader = compile " + Classes.Functions.GetShaderModel(link.PixelShader.Type, pixel_link_model) + " " + link.Name + "_PS(" + GenerateUniforms(link, "Pixel") + ");\r\n");
                }
            }


            temp.Append("}");

            return temp.ToString();
        }

        public void UpdateVertexUniforms(Pass update)
        {
            List<Parameter> aggregated_params = new List<Parameter>();
            List<string> param_names = new List<string>();
            Uniform temp_uniform = null;

            if (update == this)
            {
                #region Update From Self

                if (vertex_shader != null)
                {
                    aggregated_params.AddRange(vertex_shader.Inputs);
                    aggregated_params.AddRange(vertex_shader.Outputs);

                    foreach (Parameter param in aggregated_params)
                    {
                        if (param.Uniform)
                        {
                            param_names.Add(param.Name);
                            temp_uniform = vertex_uniforms.Find(delegate(Uniform u) { return u.Name == param.Name; });
                            if (temp_uniform != null)
                            {
                                if (temp_uniform.Type != param.DataType.ToString())
                                {
                                    temp_uniform.Type = param.DataType.ToString();
                                }
                            }
                            else
                            {
                                vertex_uniforms.Add(new Uniform(param.DataType.ToString(), param.Name));
                            }
                        }
                    }

                    vertex_uniforms.RemoveAll(delegate(Uniform u) { return !param_names.Contains(u.Name); });
                }

                foreach (Pass p in Classes.Variables.my_effect.Passes.FindAll(delegate(Pass p) { return p.VertexLink == Name; }))
                {
                    p.UpdateVertexUniforms(this);
                }

                #endregion
            }
            else
            {
                #region Update From Other

                if (update.VertexShader != null)
                {
                    aggregated_params.AddRange(update.VertexShader.Inputs);
                    aggregated_params.AddRange(update.VertexShader.Outputs);

                    foreach (Parameter param in aggregated_params)
                    {
                        if (param.Uniform)
                        {
                            param_names.Add(param.Name);
                            temp_uniform = vertex_uniforms.Find(delegate(Uniform u) { return u.Name == param.Name; });
                            if (temp_uniform != null)
                            {
                                if (temp_uniform.Type != param.DataType.ToString())
                                {
                                    temp_uniform.Type = param.DataType.ToString();
                                }
                            }
                            else
                            {
                                vertex_uniforms.Add(new Uniform(param.DataType.ToString(), param.Name));
                            }
                        }
                    }

                    vertex_uniforms.RemoveAll(delegate(Uniform u) { return !param_names.Contains(u.Name); });
                }

                #endregion
            }

            PublicProperties.Remove("VertexUniforms");
            if (vertex_uniforms.Count != 0) { PublicProperties.Add(new DynamicProperty("VertexUniforms", "Vertex Shader")); }

            Classes.Variables.main_form.pgr_main.Refresh();

        }

        public void UpdatePixelUniforms(Pass update)
        {
            List<Parameter> aggregated_params = new List<Parameter>();
            List<string> param_names = new List<string>();
            Uniform temp_uniform = null;

            if (update == this)
            {
                #region Update From Self

                aggregated_params.AddRange(pixel_shader.Inputs);
                aggregated_params.AddRange(pixel_shader.Outputs);

                foreach (Parameter param in aggregated_params)
                {
                    if (param.Uniform)
                    {
                        param_names.Add(param.Name);
                        temp_uniform = pixel_uniforms.Find(delegate(Uniform u) { return u.Name == param.Name; });
                        if (temp_uniform != null)
                        {
                            if (temp_uniform.Type != param.DataType.ToString())
                            {
                                temp_uniform.Type = param.DataType.ToString();
                            }
                        }
                        else
                        {
                            pixel_uniforms.Add(new Uniform(param.DataType.ToString(), param.Name));
                        }
                    }
                }

                pixel_uniforms.RemoveAll(delegate(Uniform u) { return !param_names.Contains(u.Name); });

                foreach (Pass p in Classes.Variables.my_effect.Passes.FindAll(delegate(Pass p) { return p.PixelLink == Name; }))
                {
                    p.UpdatePixelUniforms(this);
                }

                #endregion
            }
            else
            {
                #region Update From Other

                aggregated_params.AddRange(update.PixelShader.Inputs);
                aggregated_params.AddRange(update.PixelShader.Outputs);

                foreach (Parameter param in aggregated_params)
                {
                    if (param.Uniform)
                    {
                        param_names.Add(param.Name);
                        temp_uniform = pixel_uniforms.Find(delegate(Uniform u) { return u.Name == param.Name; });
                        if (temp_uniform != null)
                        {
                            if (temp_uniform.Type != param.DataType.ToString())
                            {
                                temp_uniform.Type = param.DataType.ToString();
                            }
                        }
                        else
                        {
                            pixel_uniforms.Add(new Uniform(param.DataType.ToString(), param.Name));
                        }
                    }
                }

                pixel_uniforms.RemoveAll(delegate(Uniform u) { return !param_names.Contains(u.Name); });

                #endregion
            }

            PublicProperties.Remove("PixelUniforms");
            if (pixel_uniforms.Count != 0) { PublicProperties.Add(new DynamicProperty("PixelUniforms", "Pixel Shader")); }

            Classes.Variables.main_form.pgr_main.Refresh();

        }

        public string GenerateUniforms(Pass pass, string type)
        {
            StringBuilder result = new StringBuilder();
            List<Parameter> aggregated_params = new List<Parameter>();

            if (type == "Vertex")
            {
                aggregated_params.AddRange(pass.VertexShader.Inputs);
                aggregated_params.AddRange(pass.VertexShader.Outputs);
                aggregated_params = aggregated_params.FindAll(delegate(Parameter p) { return p.Uniform == true; });

                for (int i = 0; i < aggregated_params.Count; i++)
                {
                    result.Append(vertex_uniforms.Find(delegate(Uniform u) { return u.Name == aggregated_params[i].Name; }).Value);
                    if (i + 1 < aggregated_params.Count)
                    {
                        result.Append(",");
                    }
                }
            }
            else
            {
                aggregated_params.AddRange(pass.PixelShader.Inputs);
                aggregated_params.AddRange(pass.PixelShader.Outputs);
                aggregated_params = aggregated_params.FindAll(delegate(Parameter p) { return p.Uniform == true; });

                for (int i = 0; i < aggregated_params.Count; i++)
                {
                    result.Append(pixel_uniforms.Find(delegate(Uniform u) { return u.Name == aggregated_params[i].Name; }).Value);
                    if (i + 1 < aggregated_params.Count)
                    {
                        result.Append(",");
                    }
                }
            }

            return result.ToString();
        }

        // properties

        [TypeConverter(typeof(Classes.Helper.CurrentTechniqueNames))]
        [DescriptionAttribute("The name of the technique that contains the pass.")]
        public string TechniqueName
        {
            get { return technique_name; }
            set
            {
                if (value == "(default)")
                {
                    technique_name = value;
                    Classes.Functions.ProjectChanged();
                }
                else
                {
                    if (Classes.Functions.CheckNameIsAppropriate(value, "technique"))
                    {
                        technique_name = value;
                        Classes.Functions.ProjectChanged();
                    }
                }
            }
        }

        [DescriptionAttribute("The name of the shader pass.")]
        public override string Name
        {
            get { return base.Name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;
                    Classes.Variables.main_form.RenameComponent(old_name, value);
                    Classes.Variables.main_form.RenamePassChildren(old_name, value);
                    Classes.Functions.ProjectChanged();
                }
            }
        }

        [Description("The order of the pass appears in the shader technique.")]
        public int Order
        {
            get { return order; }
            set { order = Functions.Clamp(value, 1, int.MaxValue); Classes.Functions.ProjectChanged(); }
        }

        public Shader VertexShader
        {
            get { return vertex_shader; }
            set { vertex_shader = value; }
        }

        [DisplayName("Uniforms")]
        public ReadOnlyCollection<Uniform> VertexUniforms
        {
            get { return readonly_vertex_uniforms; }
            set { readonly_vertex_uniforms = value; }
        }

        [DisplayName("Link")]
        [DescriptionAttribute("The pass to link the vertex shader to.")]
        [TypeConverter(typeof(Classes.Helper.CurrentPasses))]
        public string VertexLink
        {
            get { return vertex_link; }
            set
            {
                vertex_link = value;
                vertex_shader.Visible = false;
                Classes.Variables.main_form.ClosePassChildren(base.Name, "Vertex");
                if (value == "(none)")
                {
                    PublicProperties.Remove("VertexLinkModel");
                    UpdateVertexUniforms(this);
                }
                else
                {
                    Pass temp = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass p) { return p.Name == value; });
                    PublicProperties.Add(new DynamicProperty("VertexLinkModel", "Vertex Shader"));
                    this.vertex_link_model = temp.VertexShader.Model;
                    UpdateVertexUniforms(temp);
                }
                Classes.Functions.ProjectChanged();
            }
        }

        [DisplayName("Model")]
        [DescriptionAttribute("The shader model to compile the linked pass.")]
        [TypeConverter(typeof(Classes.Helper.VertexModels))]
        public string VertexLinkModel
        {
            get { return vertex_link_model; }
            set { vertex_link_model = value; Classes.Functions.ProjectChanged(); }
        }

        public Shader PixelShader
        {
            get { return pixel_shader; }
            set { pixel_shader = value; }
        }

        [DisplayName("Uniforms")]
        public List<Uniform> PixelUniforms
        {
            get { return pixel_uniforms; }
            set { pixel_uniforms = value; }
        }

        [DisplayName("Link")]
        [DescriptionAttribute("The pass to link the pixel shader to.")]
        [TypeConverter(typeof(Classes.Helper.CurrentPasses))]
        public string PixelLink
        {
            get { return pixel_link; }
            set
            {
                pixel_link = value;
                pixel_shader.Visible = false;
                Classes.Variables.main_form.ClosePassChildren(base.Name, "Pixel");
                if (value == "(none)")
                {
                    PublicProperties.Remove("PixelLinkModel");
                    UpdatePixelUniforms(this);
                }
                else
                {
                    Pass temp = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass p) { return p.Name == value; });
                    PublicProperties.Add(new DynamicProperty("PixelLinkModel", "Pixel Shader"));
                    this.pixel_link_model = temp.PixelShader.Model;
                    UpdatePixelUniforms(temp);
                }
                Classes.Functions.ProjectChanged();
            }
        }

        [DisplayName("Model")]
        [DescriptionAttribute("The shader model to compile the linked pass.")]
        [TypeConverter(typeof(Classes.Helper.PixelModels))]
        public string PixelLinkModel
        {
            get { return pixel_link_model; }
            set { pixel_link_model = value; Classes.Functions.ProjectChanged(); }
        }

        public RenderStates States
        {
            get { return states; }
            set { states = value; }
        }
    }

    [Serializable]
    public class Variable : ParentComponent
    {
        private BasicDataTypes datatype = BasicDataTypes.@float;
        private string initial_value = "0.0";

        public Variable(string name, bool predefined) : base(name, predefined) 
        {
            PublicProperties.Add(new DynamicProperty("DataType", "Variable Settings"));
            PublicProperties.Add(new DynamicProperty("Value", "Variable Settings"));
        }

        public override string GenerateCode()
        {
            string temp = initial_value;

            if (datatype.ToString() == "float")
            {
                temp = String.Format("{0:0.0################}", initial_value);
                int i = 0;
                if (!Int32.TryParse(initial_value, out i)) { temp += "f"; }
            }

            if (base.PreDefined)
            {
                return "const " + datatype.ToString() + " " + base.Name + ";";
            }
            else
            {
               return datatype.ToString() + " " + base.Name + " = " + temp + ";"; 
            }
        }

        [DescriptionAttribute("The name of the shader variable.")]
        public override string Name
        {
            get { return base.Name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;
                    Classes.Variables.objects[Classes.Variables.objects.IndexOf(old_name)] = value;
                    Classes.Variables.main_form.RenameComponent(old_name, value);
                    Classes.Functions.ProjectChanged();
                    Classes.Functions.UpdateHighlighting();
                }
            }
        }

        [RefreshProperties(RefreshProperties.All)]
        [DescriptionAttribute("The data type of the variable.")]
        public BasicDataTypes DataType
        {
            get { return datatype; }
            set
            {
                datatype = value;

                int int_i;
                uint uint_i;
                float float_i;
                double double_i;

                switch (datatype.ToString())
                {
                    case "int": if (!int.TryParse(Value, out int_i)) { initial_value = "0"; } break;
                    case "uint": if (!uint.TryParse(Value, out uint_i)) { initial_value = "0"; } break;
                    case "float": if (!float.TryParse(Value, out float_i)) { initial_value = "0.0"; } break;
                    case "double": if (!double.TryParse(Value, out double_i)) { initial_value = "0.00"; } break;
                    case "bool": if (Value.ToLower() != "true" || Value.ToLower() != "false") { if (Convert.ToDouble(Value.ToLower()) == 0) { initial_value = "false"; } else { initial_value = "true"; } } break;
                }

                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The default value of the variable.")]
        public string Value
        {
            get { return initial_value; }
            set
            {
                if (value != string.Empty)
                {
                    value =  value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(datatype.ToString(), value))
                    {
                        initial_value = value;
                        Classes.Functions.ProjectChanged();
                        Classes.Functions.Recompile();
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }
    }

    [Serializable]
    public class ArrayItem
    {
        private string mode = string.Empty;
        private string datatype = string.Empty;
        private string elements = string.Empty;

        private string value = string.Empty;   
        private string x = string.Empty;
        private string y = string.Empty;
        private string z = string.Empty;
        private string w = string.Empty;

        public ArrayItem(string datatype, string mode, string elements)
        {
            this.datatype = datatype;
            this.mode = mode;
            this.elements = elements;

            if (mode == "Variable")
            {
                Functions.SetBrowsable(this, "Value", true);
                Functions.SetBrowsable(this, "X", false);
                Functions.SetBrowsable(this, "Y", false);
                Functions.SetBrowsable(this, "Z", false);
                Functions.SetBrowsable(this, "W", false);
                switch (datatype)
                {
                    case "bool": value = "false"; break;
                    case "int": value = "0"; break;
                    case "float": value = "0.0"; break;
                    case "double": value = "0.00"; break;
                }
            }
            else
            {
                switch (datatype)
                {
                    case "bool": x = "false"; y = "false"; z = "false"; w = "false"; break;
                    case "int": x = "0"; y = "0"; z = "0"; w = "0"; break;
                    case "float": x = "0.0"; y = "0.0"; z = "0.0"; w = "0.0"; break;
                    case "double": x = "0.00"; y = "0.00"; z = "0.00"; w = "0.00"; break;
                }

                Functions.SetBrowsable(this, "Value", false);
                switch (elements)
                {
                    case "2":
                        Functions.SetBrowsable(this, "X", true);
                        Functions.SetBrowsable(this, "Y", true);
                        Functions.SetBrowsable(this, "Z", false);
                        Functions.SetBrowsable(this, "W", false);
                        break;
                    case "3":
                        Functions.SetBrowsable(this, "X", true);
                        Functions.SetBrowsable(this, "Y", true);
                        Functions.SetBrowsable(this, "Z", true);
                        Functions.SetBrowsable(this, "W", false);
                        break;
                    case "4":
                        Functions.SetBrowsable(this, "X", true);
                        Functions.SetBrowsable(this, "Y", true);
                        Functions.SetBrowsable(this, "Z", true);
                        Functions.SetBrowsable(this, "W", true);
                        break;
                }
            }
        }

        [Browsable(false)]
        public string Type
        {
            get { return datatype; }
            set { datatype = value; }
        }

        [Browsable(false)]
        public string Elements
        {
            get { return elements; }
            set { elements = value; }
        }

        [Browsable(false)]
        [Description("The value of the item in the defined array.")]
        public string Value
        {
            get { return value; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(datatype, value))
                    {
                        this.value = value;
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype of the array does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        [Browsable(false)]
        [Description("The x component of the vector entry.")]
        public string X
        {
            get { return x; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(datatype, value))
                    {
                        this.x = value;
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype of the array does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        [Browsable(false)]
        [Description("The y component of the vector entry.")]
        public string Y
        {
            get { return y; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(datatype, value))
                    {
                        this.y = value;
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype of the array does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        [Browsable(false)]
        [Description("The z component of the vector entry.")]
        public string Z
        {
            get { return z; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(datatype, value))
                    {
                        this.z = value;
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype of the array does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        [Browsable(false)]
        [Description("The w component of the vector entry.")]
        public string W
        {
            get { return w; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(datatype, value))
                    {
                        this.w = value;
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype of the array does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        public override string ToString()
        {
            if (this.mode == "Variable")
            {
                return value;
            }
            else
            {
                switch (elements)
                {
                    case "2": return "{" + x + "," + y + "}";
                    case "3": return "{" + x + "," + y + "," + z + "}";
                    case "4": return "{" + x + "," + y + "," + z + "," + w + "}";
                }
                return string.Empty;
            }
        }
    }

    [Serializable]
    public class Array : ParentComponent
    {
        private string mode = string.Empty;
        public enum ArrayType { empty, defined }
        private ArrayType type = ArrayType.empty;
        private BasicDataTypes datatype = BasicDataTypes.@float;
        private int dimension = 5;
        private string elements = "4";
        private List<ArrayItem> values = new List<ArrayItem>();

        public Array(string name, string mode) : base(name) 
        {
            this.mode = mode;

            PublicProperties.Add(new DynamicProperty("Type", "Array Settings"));
            PublicProperties.Add(new DynamicProperty("DataType", "Array Settings"));
            PublicProperties.Add(new DynamicProperty("Dimension", "Array Settings"));

            if (mode == "Vector")
            {
                PublicProperties.Add(new DynamicProperty("Elements", "Array Settings"));
            }
        }

        public override string GenerateCode()
        {
            string temp = string.Empty;
            if (type == ArrayType.empty)
            {
                if (mode == "Variable")
                {
                    return datatype.ToString() + " " + base.Name + "[" + dimension + "];";
                }
                else
                {
                    return datatype.ToString() + elements + " " + base.Name + "[" + dimension + "];";
                }
            }
            else
            {
                if (mode == "Variable")
                {
                    temp += datatype.ToString() + " " + base.Name + "["+values.Count+"] = {";
                    for (int i = 0; i < values.Count; i++)
                    {
                        temp += values[i].Value;
                        if (i + 1 < values.Count) { temp += ","; }
                    }
                    temp += "};";
                    return temp;
                }
                else
                {
                    temp += datatype.ToString() + elements+" " + base.Name + "["+values.Count+"] = {";
                    for (int i = 0; i < values.Count; i++)
                    {
                        switch (elements)
                        {
                            case "2": temp += datatype.ToString() + elements + "(" + values[i].X + "," + values[i].Y + ")"; break;
                            case "3": temp += datatype.ToString() + elements + "(" + values[i].X + "," + values[i].Y + "," + values[i].Z + ")"; break;
                            case "4": temp += datatype.ToString() + elements + "(" + values[i].X + "," + values[i].Y + "," + values[i].Z + "," + values[i].W + ")"; break;
                        }
                        if (i + 1 < values.Count) { temp += ","; }
                    }
                    temp += "};";
                    return temp;
                }
            }

        }

        [DescriptionAttribute("The name of the shader array.")]
        public override string Name
        {
            get { return base.Name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;
                    Classes.Variables.objects[Classes.Variables.objects.IndexOf(old_name)] = value;
                    Classes.Variables.main_form.RenameComponent(old_name, value);
                    Classes.Functions.ProjectChanged();
                    Classes.Functions.UpdateHighlighting();
                }
            }
        }

        public string Mode
        {
            get { return mode; }
            set { mode = value; Classes.Functions.ProjectChanged(); }
        }

        [RefreshProperties(RefreshProperties.All)]
        [DescriptionAttribute("Determines if the array is empty or has values defined.")]
        public ArrayType Type
        {
            get{return type;}
            set
            {
                type = value;

                PublicProperties.Remove("Dimension");
                PublicProperties.Remove("Values");
                PublicProperties.Remove("Elements");

                if (type == ArrayType.empty)
                {
                    PublicProperties.Add(new DynamicProperty("Dimension", "Array Settings"));
                }
                else
                {
                    PublicProperties.Add(new DynamicProperty("Values", "Array Settings"));
                }

                if (mode == "Vector")
                {
                    PublicProperties.Add(new DynamicProperty("Elements", "Array Settings"));
                }

                Classes.Functions.ProjectChanged();
            }
        }

        [DescriptionAttribute("The data type of the array.")]
        public BasicDataTypes DataType
        {
            get { return datatype; }
            set 
            {
                if (values.Count != 0 && type == ArrayType.defined)
                {
                    #region Check variable arrays

                    if (mode == "Variable")
                    {
                        int mismatch_count = 0;
                        for (int i = 0; i < values.Count; i++)
                        {
                            if (!Classes.Functions.CheckCastIsValid(value.ToString(), values[i].Value)) { mismatch_count++; }
                        }

                        if (mismatch_count != 0)
                        {
                            switch (MessageBox.Show("This array contains items that do not support the selected datatype.\r\nThey will be replaced with default values.\r\n\r\nWould you like to continue?", "Datatype mis-match", MessageBoxButtons.YesNo, MessageBoxIcon.Warning))
                            {
                                case DialogResult.No: return;
                                case DialogResult.Yes:
                                    for (int i = 0; i < values.Count; i++)
                                    {
                                        values[i].Type = value.ToString();
                                        switch (value.ToString())
                                        {
                                            case "bool": values[i].Value = "false"; break;
                                            case "int": values[i].Value = "0"; break;
                                            case "float": values[i].Value = "0.0"; break;
                                            case "double": values[i].Value = "0.00"; break;
                                        }
                                    }
                                    datatype = value; break;
                            }
                        }
                        else
                        {
                            datatype = value;
                        }
                    }

                    #endregion

                    #region Check vector arrays

                    else
                    {
                        // check array
                        int mismatch_count = 0;
                        for (int i = 0; i < values.Count; i++)
                        {
                            if (!Classes.Functions.CheckCastIsValid(value.ToString(), values[i].X)) { mismatch_count++; }
                        }

                        if (mismatch_count != 0)
                        {
                            switch (MessageBox.Show("This array contains items that do not support the selected datatype.\r\nThey will be replaced with default values.\r\n\r\nWould you like to continue?", "Datatype mis-match", MessageBoxButtons.YesNo, MessageBoxIcon.Warning))
                            {
                                case DialogResult.No: return;
                                case DialogResult.Yes:
                                    for (int i = 0; i < values.Count; i++)
                                    {
                                        values[i].Type = value.ToString();
                                        switch (value.ToString())
                                        {
                                            case "bool": 
                                                values[i].X = "false";
                                                values[i].Y = "false";
                                                values[i].Z = "false";
                                                values[i].W = "false"; 
                                                break;
                                            case "int": 
                                                values[i].X = "0";
                                                values[i].Y = "0";
                                                values[i].Z = "0";
                                                values[i].W = "0";
                                                break;
                                            case "float": 
                                                values[i].X = "0.0";
                                                values[i].Y = "0.0";
                                                values[i].Z = "0.0";
                                                values[i].W = "0.0";
                                                break;
                                            case "double": 
                                                values[i].X = "0.00";
                                                values[i].Y = "0.00";
                                                values[i].Z = "0.00";
                                                values[i].W = "0.00"; 
                                                break;
                                        }
                                    }
                                    datatype = value; break;
                            }
                        }
                        else
                        {
                            datatype = value;
                        }
                    }

                    #endregion
                }
                else
                {
                    datatype = value;
                }
                Classes.Functions.ProjectChanged();
            }
        }
            
        [Description("The size of the array.")]
        public int Dimension
        {
            get { return dimension; }
            set 
            { 
                dimension = Classes.Functions.Clamp(value,2,int.MaxValue);
                Classes.Functions.ProjectChanged();
            }
        }

        [Description("The number of elements in the vector.")]
        [TypeConverter(typeof(Classes.Helper.VectorElements))]
        public string Elements
        {
            get { return elements; }
            set 
            { 
                elements = value;
                if (type == ArrayType.defined && values.Count != 0)
                {
                    switch (elements)
                    {
                        case "2":
                            Functions.SetBrowsable(values[0], "X", true);
                            Functions.SetBrowsable(values[0], "Y", true);
                            Functions.SetBrowsable(values[0], "Z", false);
                            Functions.SetBrowsable(values[0], "W", false);
                            break;

                        case "3":
                            Functions.SetBrowsable(values[0], "X", true);
                            Functions.SetBrowsable(values[0], "Y", true);
                            Functions.SetBrowsable(values[0], "Z", true);
                            Functions.SetBrowsable(values[0], "W", false);
                            break;

                        case "4":
                            Functions.SetBrowsable(values[0], "X", true);
                            Functions.SetBrowsable(values[0], "Y", true);
                            Functions.SetBrowsable(values[0], "Z", true);
                            Functions.SetBrowsable(values[0], "W", true);
                            break;
                    }
                }
                foreach (ArrayItem item in values)
                {
                    item.Elements = value;
                }
                Classes.Functions.ProjectChanged();
            }
        }

        [Description("The values of the array.")]
        [EditorAttribute(typeof(Classes.Helper.ArrayValueCollectionEditor), typeof(UITypeEditor))]
        public List<ArrayItem> Values
        {
            get { return values; }
            set { values = value; }
        }
    }

    [Serializable]
    public class Vector : ParentComponent
    {
        private BasicDataTypes type = BasicDataTypes.@float;
        private string elements = "4";
        private string x = "0.0";
        private string y = "0.0";
        private string z = "0.0";
        private string w = "0.0";

        public Vector(string name, bool predefined) : base(name, predefined)
        {
            PublicProperties.Add(new DynamicProperty("DataType", "Vector Settings"));
            PublicProperties.Add(new DynamicProperty("Elements", "Vector Settings"));
            PublicProperties.Add(new DynamicProperty("X", "Vector Settings"));
            PublicProperties.Add(new DynamicProperty("Y", "Vector Settings"));
            PublicProperties.Add(new DynamicProperty("Z", "Vector Settings"));
            PublicProperties.Add(new DynamicProperty("W", "Vector Settings"));
        }

        public override string GenerateCode()
        {
            if(PreDefined)
            {
                return type.ToString() + elements + " " + Name + ";";
            }

            string temp_x = x;
            string temp_y = y;
            string temp_z = z;
            string temp_w = w;

            int i = 0;

            if (type.ToString() == "float")
            {
                temp_x = String.Format("{0:0.0################}", x);          
                if (!Int32.TryParse(x, out i)) { temp_x += "f"; }

                temp_y = String.Format("{0:0.0################}", y);
                if (!Int32.TryParse(y, out i)) { temp_y += "f"; }

                temp_z = String.Format("{0:0.0################}", z);
                if (!Int32.TryParse(z, out i)) { temp_z += "f"; }

                temp_w = String.Format("{0:0.0################}", w);
                if (!Int32.TryParse(w, out i)) { temp_w += "f"; }
            }

            switch (elements)
            {
                case "2": return type.ToString() + "2 " + Name + " = {" + temp_x + "," + temp_y + "};";
                case "3": return type.ToString() + "3 " + Name + " = {" + temp_x + "," + temp_y + "," + temp_z + "};";
                case "4": return type.ToString() + "4 " + Name + " = {" + temp_x + "," + temp_y + "," + temp_z + "," + temp_w + "};";
            }

            return null;
        }

        [DescriptionAttribute("The name of the shader vector.")]
        public override string Name
        {
            get { return base.Name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;
                    Classes.Variables.objects[Classes.Variables.objects.IndexOf(old_name)] = value;
                    Classes.Variables.main_form.RenameComponent(old_name, value);
                    Classes.Functions.ProjectChanged();
                    Classes.Functions.UpdateHighlighting();
                }
            }
        }

        [RefreshProperties(RefreshProperties.All)]
        [DescriptionAttribute("The data type of the vector.")]
        public BasicDataTypes DataType
        {
            get { return type; }
            set
            {
                type = value;
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), X)) { x = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), Y)) { y = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), Z)) { z = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), W)) { w = Classes.Functions.GetDefaultValue(type.ToString()); }
                Classes.Functions.ProjectChanged();
            }
        }

        [RefreshProperties(RefreshProperties.All)]
        [DescriptionAttribute("The number of elements used in the vector.")]
        [TypeConverter(typeof(Classes.Helper.VectorElements))]
        public string Elements
        {
            get { return elements; }
            set
            {
                elements = value;

                PublicProperties.Remove("X");
                PublicProperties.Remove("Y");
                PublicProperties.Remove("Z");
                PublicProperties.Remove("W");
                switch (elements)
                {
                    case "2":
                        PublicProperties.Add(new DynamicProperty("X", "Vector Settings"));
                        PublicProperties.Add(new DynamicProperty("Y", "Vector Settings"));
                        break;
                    case "3":
                        PublicProperties.Add(new DynamicProperty("X", "Vector Settings"));
                        PublicProperties.Add(new DynamicProperty("Y", "Vector Settings"));
                        PublicProperties.Add(new DynamicProperty("Z", "Vector Settings"));
                        break;
                    case "4":
                        PublicProperties.Add(new DynamicProperty("X", "Vector Settings"));
                        PublicProperties.Add(new DynamicProperty("Y", "Vector Settings"));
                        PublicProperties.Add(new DynamicProperty("Z", "Vector Settings"));
                        PublicProperties.Add(new DynamicProperty("W", "Vector Settings"));
                        break;
                }
                Classes.Functions.ProjectChanged();
            }

        }

        [DescriptionAttribute("The x element of the vector.")]
        public string X
        {
            get { return x; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(type.ToString(), value))
                    {
                        x = value;
                        Classes.Functions.ProjectChanged();
                        Classes.Functions.Recompile();
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                    
                }
            }
        }

        [DescriptionAttribute("The y element of the vector.")]
        public string Y
        {
            get { return y; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(type.ToString(), value))
                    {
                        y = value;
                        Classes.Functions.ProjectChanged();
                        Classes.Functions.Recompile();
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        [DescriptionAttribute("The z element of the vector.")]
        public string Z
        {
            get { return z; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(type.ToString(), value))
                    {
                        z = value;
                        Classes.Functions.ProjectChanged();
                        Classes.Functions.Recompile();
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        [DescriptionAttribute("The w element of the vector.")]
        public string W
        {
            get { return w; }
            set
            {
                if (value != string.Empty)
                {
                    value = value.Replace(",", "");

                    if (Classes.Functions.CheckCastIsValid(type.ToString(), value))
                    {
                        w = value;
                        Classes.Functions.ProjectChanged();
                        Classes.Functions.Recompile();
                    }
                    else
                    {
                        MessageBox.Show("The selected datatype does not support the current value.", "Datatype mis-match", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }
    }

    [Serializable]
    public class Matrix : ParentComponent
    {
        private BasicDataTypes type = BasicDataTypes.@float;

        private string[] grid_values = new string[16];
        private string values = string.Empty;
        private string holder = string.Empty;
        private string rows = "4", columns = "4";
        private string modifier = "None";

        public Matrix(string name, bool predefined): base(name, predefined)
        {
            for (int i = 0; i < grid_values.Length; i++) { grid_values[i] = "0.0"; }
            PublicProperties.Add(new DynamicProperty("DataType", "Matrix Settings"));
            PublicProperties.Add(new DynamicProperty("Holder", "Matrix Settings"));
            PublicProperties.Add(new DynamicProperty("Rows", "Matrix Settings"));
            PublicProperties.Add(new DynamicProperty("Columns", "Matrix Settings"));
            PublicProperties.Add(new DynamicProperty("Modifier", "Matrix Settings"));
        }

        public override string GenerateCode()
        {
            if (PreDefined)
            {
                if (Name == "matTangent")
                {
                    return "float3x3 matTangent;";
                }
                else
                {
                    return "const float4x4 " + Name + ";";
                }
            }

            string[] row_1 = new string[4] { grid_values[0], grid_values[1], grid_values[2], grid_values[3] };
            string[] row_2 = new string[4] { grid_values[4], grid_values[5], grid_values[6], grid_values[7] };
            string[] row_3 = new string[4] { grid_values[8], grid_values[9], grid_values[10], grid_values[11] };
            string[] row_4 = new string[4] { grid_values[12], grid_values[13], grid_values[14], grid_values[15] };

            int out_i = 0;
            int i_rows = Convert.ToInt32(this.rows);
            int i_columns = Convert.ToInt32(this.columns);
            string[] current_row = null;
            StringBuilder result = new StringBuilder((modifier == "None" ? string.Empty : modifier+" ") + type.ToString() + rows + "x" + columns + " " + Name + " =\r\n{\r\n");

            for (int i = 0; i < i_rows; i++)
            {
                switch (i)
                {
                    case 0: current_row = row_1; break;
                    case 1: current_row = row_2; break;
                    case 2: current_row = row_3; break;
                    case 3: current_row = row_4; break;
                }

                for (int j = 0; j < i_columns; j++)
                {
                    result.Append(current_row[j]);
                    if ((type.ToString() == "float" || type.ToString() == "half") && !Int32.TryParse(current_row[j], out out_i)) { result.Append("f"); }
                    result.Append(",");
                }
                result.Append("\r\n");
            }
            result.Remove(result.Length - 3, 1);
            result.Append("};");

            return result.ToString();
        }

        [DescriptionAttribute("The name of the shader matrix.")]
        public override string Name
        {
            get { return base.Name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;
                    Classes.Variables.objects[Classes.Variables.objects.IndexOf(old_name)] = value;
                    Classes.Variables.main_form.RenameComponent(old_name, value);
                    Classes.Functions.ProjectChanged();
                    Classes.Functions.UpdateHighlighting();
                }
            }
        }

        public BasicDataTypes DataType
        {
            get { return type; }
            set
            {
                type = value;

                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[0])) { grid_values[0] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[1])) { grid_values[1] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[2])) { grid_values[2] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[3])) { grid_values[3] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[4])) { grid_values[4] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[5])) { grid_values[5] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[6])) { grid_values[6] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[7])) { grid_values[7] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[8])) { grid_values[8] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[9])) { grid_values[9] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[10])) { grid_values[10] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[11])) { grid_values[11] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[12])) { grid_values[12] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[13])) { grid_values[13] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[14])) { grid_values[14] = Classes.Functions.GetDefaultValue(type.ToString()); }
                if (!Classes.Functions.CheckCastIsValid(type.ToString(), grid_values[15])) { grid_values[15] = Classes.Functions.GetDefaultValue(type.ToString()); }
                Classes.Functions.ProjectChanged();
            }
        }

        [Browsable(true)]
        [DisplayName("Values")]
        [EditorAttribute(typeof(Classes.Helper.MatrixEditor), typeof(UITypeEditor))]
        public string Holder
        {
            get { return holder; }
            set { holder = string.Empty;}
        }

        [DescriptionAttribute("The number of rows in the matrix.")]
        [TypeConverter(typeof(Classes.Helper.VectorElements))]
        public string Rows
        {
            get { return rows; }
            set { rows = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The number of columns in the matrix.")]
        [TypeConverter(typeof(Classes.Helper.VectorElements))]
        public string Columns
        {
            get { return columns; }
            set { columns = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The type modifier attached to the matrix.")]
        [TypeConverter(typeof(Classes.Helper.MatrixModifiers))]
        public string Modifier
        {
            get { return modifier; }
            set { modifier = value; Classes.Functions.ProjectChanged(); }
        }

        public string Values
        {
            get { return values; }
            set { values = value; }
        }

        public string[] GridValues
        {
            get { return grid_values; }
            set { grid_values = value; Classes.Functions.ProjectChanged(); }
        }
    }

    [Serializable]
    public class Color : ParentComponent
    {
        private System.Drawing.Color value = System.Drawing.Color.Black;
        private string mode = "RGBA";

        public Color(string name) : base(name) 
        {
            PublicProperties.Add(new DynamicProperty("Mode", "Color Settings"));
            PublicProperties.Add(new DynamicProperty("Value", "Color Settings"));
        }
        
        public override string GenerateCode()
        {
            string temp = string.Empty;
            double luminance = 0.0;

            switch (mode)
            {
                case "ARGB": return "float4 " + Name + " = {" + (float)value.A / 255 + "," + (float)value.R / 255 + "," + (float)value.G / 255+"," + (float)value.B / 255+"};";
                case "RGBA": return "float4 " + Name + " = {" + (float)value.R / 255 + "," + (float)value.G / 255 + "," + (float)value.B / 255 + "," + (float)value.A / 255 + "};";
                case "RGB": return "float3 " + Name + " = {" + (float)value.R / 255 + "," + (float)value.G / 255+"," + (float)value.B / 255+"};";

                case "AYUV":
                    luminance = 0.299 * (((float)value.R / 255)) + 0.587 * ((float)value.G / 255) + 0.114 * ((float)value.B / 255);
                    return "float4 " + Name + " = {" + (float)value.A / 255 + "," + (float)luminance + "," + (float)((value.B / 255) - luminance) + "," + (float)((value.R / 255) - luminance) + "};";

                case "YUV":
                    luminance = 0.299 * (((float)value.R / 255)) + 0.587 * ((float)value.G / 255) + 0.114 * ((float)value.B / 255);
                    return "float4" + Name + " = {" + (float)luminance + "," + (float)((value.B / 255) - luminance) + "," + (float)((value.R / 255) - luminance) + "};";
            }

            return temp;
        }

        [DescriptionAttribute("Name of the shader color.")]
        public override string Name
        {
            get { return base.Name; }
            set 
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;                  
                    Classes.Variables.objects[Classes.Variables.objects.IndexOf(old_name)] = value; 
                    Classes.Variables.main_form.RenameComponent(old_name,value);
                    Classes.Functions.ProjectChanged();
                    Classes.Functions.UpdateHighlighting();
                }
            }
        }

        [DescriptionAttribute("The mode that determines how the color is used.")]
        [TypeConverter(typeof(Classes.Helper.ColorModes))]
        public string Mode
        {
            get { return mode; }
            set 
            { 
                mode = value; 
                Classes.Functions.ProjectChanged();
                Classes.Functions.Recompile();
            }
        }

        [DescriptionAttribute("The value of the shader color.")]
        public System.Drawing.Color Value
        {
            get { return value; }
            set 
            { 
                this.value = value; 
                Classes.Functions.ProjectChanged();
                Classes.Functions.Recompile();
            }
        }
    }

    [Serializable]
    public class Texture : ParentComponent
    {
        public Texture(string name, bool predefined) : base(name, predefined) { }

        public override string GenerateCode()
        {
            return "texture " + Name + ";";
        }
    }

    [Serializable]
    public class Sampler : ParentComponent
    {
        public enum SamplerType {sampler, sampler1D, sampler2D, sampler3D, samplerCUBE };
        SamplerType type = SamplerType.sampler;
        string texture = "(none)";

        string addressu = "Wrap";
        string addressv = "Wrap";
        string addressw = "Wrap";
        System.Drawing.Color border_color = System.Drawing.Color.Black;
        string magfilter = "Point";
        string minfilter = "Point";
        string mipfilter = "None";
        float mipmap_lodbias = 0;
        int maxmiplevel = 0;
        uint maxanisotropy = 1;
        float srgb_texture = 0.0f;
        int element_index = 0;
        int dmap_offset = 256;

        public Sampler(string name): base(name)
        {
            PublicProperties.Add(new DynamicProperty("Type", "Sampler Settings"));
            PublicProperties.Add(new DynamicProperty("Texture", "Sampler Settings"));

            PublicProperties.Add(new DynamicProperty("AddressU", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("AddressV", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("AddressW", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("BorderColor", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("MagFilter", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("MinFilter", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("MipFilter", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("MipMapLODBias", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("MaxMipLevel", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("MaxAnisotropy", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("SRGBTexture", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("ElementIndex", "Sampler States"));
            PublicProperties.Add(new DynamicProperty("DmapOffset", "Sampler States"));
        }

        public override string GenerateCode()
        {
            if (texture == "(none)")
            {
                return type.ToString() + " " + Name + ";";
            }
            else
            {
                string temp = type.ToString() + " " + Name + " = sampler_state\r\n{\r\n";
                temp += "Texture = <" + texture + ">;\r\n";

                temp += "ADDRESSU = " + addressu.ToUpper() + ";\r\n";
                temp += "ADDRESSV = " + addressv.ToUpper() + ";\r\n";
                temp += "ADDRESSW = " + addressw.ToUpper() + ";\r\n"; 
                if (border_color != System.Drawing.Color.Black) { temp += "BORDERCOLOR = float4(" + ((float)border_color.R) / 255 + "," + ((float)border_color.G) / 255 + "," + ((float)border_color.B) / 255 + "," + ((float)border_color.A) / 255 + ");\r\n"; }
                temp += "MAGFILTER = " + magfilter.ToUpper() + ";\r\n";
                temp += "MINFILTER = " + minfilter.ToUpper() + ";\r\n";
                temp += "MIPFILTER = " + mipfilter.ToUpper() + ";\r\n";
                if (mipmap_lodbias != 0) { temp += "MIPMAPLODBIAS = " + mipmap_lodbias + ";\r\n"; }
                if (maxmiplevel != 0) { temp += "MAXMIPLEVEL = " + maxmiplevel + ";\r\n"; }
                if (maxanisotropy != 1) { temp += "MAXANISOTROPY = " + maxanisotropy + ";\r\n"; }
                if (srgb_texture != 0.0) { temp += "SRGBTEXTURE = " + srgb_texture + ";\r\n"; }
                if (element_index != 0) { temp += "ELEMENTINDEX = " + element_index + ";\r\n"; }
                if (dmap_offset != 256) { temp += "DMAPOFFSET = " + dmap_offset + ";\r\n"; }

                temp += "};";

                return temp;
            }
        }

        [DescriptionAttribute("Name of the shader sampler.")]
        public override string Name
        {
            get { return base.Name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;
                    Classes.Variables.objects[Classes.Variables.objects.IndexOf(old_name)] = value;
                    Classes.Variables.main_form.RenameComponent(old_name, value);
                    Classes.Functions.ProjectChanged();
                    Classes.Functions.UpdateHighlighting();
                }
            }
        }

        [RefreshProperties(RefreshProperties.All)]
        [DescriptionAttribute("The type used for the shader sampler.")]
        public SamplerType Type
        {
            get { return type; }
            set { type = value; Classes.Functions.ProjectChanged(); }
        }

        [DescriptionAttribute("The texture used in the sampler.")]
        [TypeConverter(typeof(Classes.Helper.CurrentTextures))]
        public string Texture
        {
            get { return texture; }
            set 
            { 
                texture = value; 
                Classes.Functions.ProjectChanged();
                Classes.Functions.Recompile();
            }
        }

        [DescriptionAttribute("The texture address mode for the u coordinate.")]
        [TypeConverter(typeof(Classes.Helper.SamplerStates))]
        public string AddressU
        {
            get { return addressu; }
            set { addressu = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The texture address mode for the v coordinate.")]
        [TypeConverter(typeof(Classes.Helper.SamplerStates))]
        public string AddressV
        {
            get { return addressv; }
            set { addressv = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The texture address mode for the w coordinate.")]
        [TypeConverter(typeof(Classes.Helper.SamplerStates))]
        public string AddressW
        {
            get { return addressw; }
            set { addressw = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The color used for texture coordinates outside the 0.0 to 1.0 range.")]
        public System.Drawing.Color BorderColor
        {
            get { return border_color; }
            set { border_color = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The filtering used when the texture is rendered larger.")]
        [TypeConverter(typeof(Classes.Helper.SamplerStates))]
        public string MagFilter
        {
            get { return magfilter; }
            set { magfilter = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The filtering used when the texture is rendered smaller.")]
        [TypeConverter(typeof(Classes.Helper.SamplerStates))]
        public string MinFilter
        {
            get { return minfilter; }
            set { minfilter = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The filtering used between 2 mipmaps of the texture.")]
        [TypeConverter(typeof(Classes.Helper.SamplerStates))]
        public string MipFilter
        {
            get { return mipfilter; }
            set { mipfilter = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The mipmap level-of-detail bias.")]
        public float MipMapLODBias
        {
            get { return mipmap_lodbias; }
            set { mipmap_lodbias = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The level-of-detail index of the largest map to use.")]
        public int MaxMipLevel
        {
            get { return maxmiplevel; }
            set { maxmiplevel = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The maximum anisotrophy used.")]
        public uint MaxAnisotropy
        {
            get { return maxanisotropy; }
            set { maxanisotropy = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The gamma correction value.")]
        public float SRGBTexture
        {
            get { return srgb_texture; }
            set { srgb_texture = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("Indicates which element index to use from a multielement texture.")]
        public int ElementIndex
        {
            get { return element_index; }
            set { element_index = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }

        [DescriptionAttribute("The vertex offset in a presampled displacement map.")]
        public int DmapOffset
        {
            get { return dmap_offset; }
            set { dmap_offset = value; Classes.Functions.ProjectChanged(); Classes.Functions.Recompile(); }
        }
    }

    [Serializable]
    public class FunctionParameter
    {
        string data_type = "float4";
        string name = string.Empty;

        public FunctionParameter()
        {

        }

        public FunctionParameter(string mode, string type, string name, string semantic)
        {
            this.data_type = type;
            this.name = name;
        }

        public string GenerateCode()
        {
            return data_type + " " + name;
        }

        [Description("The data type of the function parameter.")]
        [TypeConverter(typeof(Classes.Helper.DataTypes))]
        public string DataType
        {
            get { return data_type; }
            set { data_type = value; }
        }

        [Description("The name of the function parameter.")]
        public string Name
        {
            get { return name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "parameter"))
                {
                    if (Variables.temp_params.Find(delegate(FunctionParameter param) { return param.Name == value; }) != null)
                    {
                        MessageBox.Show("A parameter with this name already exits.\r\nPlease rename the current parameter.", "Duplicate parameter name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                    else
                    {
                        name = value;
                        Classes.Functions.ProjectChanged();
                    }
                }
            }
        }
    }

    [Serializable]
    public class Function : ParentComponent
    {
        private int current_overload = 0;

        private List<string> return_type = new List<string>{"void"};
        private List<string> signatures = new List<string>{ "(  )" };
        private List<string> code = new List<string>{string.Empty};
        private List<List<FunctionParameter>> parameters = new List<List<FunctionParameter>>{new List<FunctionParameter>()};

        bool visible = true;
        Size size = new Size(360, 260);
        Point location = new Point(0, 0);

        public Function(string name): base(name)
        {
            //PublicProperties.Add(new DynamicProperty("Type", "Function Settings"));
            //PublicProperties.Add(new DynamicProperty("Parameters", "Function Settings"));
        }

        public List<string> ReturnType
        {
            get { return return_type; }
            set { return_type = value; }
        }

        public List<string> Signatures
        {
            get { return signatures; }
            set { signatures = value; }
        }

        [DescriptionAttribute("Name of the shader function.")]
        public override string Name
        {
            get { return base.Name; }
            set
            {
                if (Classes.Functions.CheckNameIsUnique(value, "component"))
                {
                    string old_name = base.Name;
                    base.Name = value;
                    Classes.Variables.objects[Classes.Variables.objects.IndexOf(old_name)] = value;
                    Classes.Variables.main_form.RenameComponent(old_name, value);
                    Classes.Variables.main_form.RenameFunctionChildren(old_name, value);
                    Classes.Functions.ProjectChanged();
                    Classes.Functions.UpdateHighlighting();
                }
            }
        }

        public List<string> Code
        {
            get{return code;}
            set{code = value;}
        }

        public List<List<FunctionParameter>> Parameters
        {
            get { return parameters; }
            set { parameters = value; }
        }

        public override string GenerateCode()
        {
            string temp = string.Empty;
            int overload_counter = 1;
            int param_counter = 1;

            for (int i = 0; i < code.Count; i++)
            {
                temp += return_type[i] + " " + base.Name + "(";
                param_counter = 1;

                foreach (FunctionParameter param in parameters[i])
                {
                    temp += param.GenerateCode();
                    if (param_counter < parameters[i].Count) { temp += ","; }
                    param_counter++;
                }
                
                temp += ")\r\n{\r\n";
                temp += code[i] + "\r\n}";

                if (overload_counter < code.Count) { temp += "\r\n\r\n"; }
                overload_counter++;
            }

            return temp; 
        }

        public int CurrentOverload
        {
            get { return current_overload; }
            set { current_overload = value; }
        }

        public string GenerateParameterSignature(List<FunctionParameter> p)
        {
            string temp = "( ";
            int counter = 1;

            foreach (FunctionParameter param in p)
            {
                temp += param.DataType.ToString();
                if(counter < p.Count){temp+=",";}
                counter++;
            }

            return temp + " )";
        }

        public bool Visible
        {
            get { return visible; }
            set { visible = value; Classes.Functions.ProjectChanged(); }
        }

        public Size Size
        {
            get { return size; }
            set { size = value; Classes.Functions.ProjectChanged(); }
        }

        public Point Location
        {
            get { return location; }
            set { location = value; Classes.Functions.ProjectChanged(); }
        }
    }
}
