﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FXED.Forms
{
    public partial class frm_function : Form
    {
        Classes.Objects.Function my_function = null;
        public Classes.Objects.Function MyFunction
        {
            get { return my_function; }
        }

        public frm_function(Classes.Objects.Function function)
        {
            this.my_function = function;

            InitializeComponent();          
        }

        CustomControls.SyntaxEditor CurrentEditor
        {
            get { return (CustomControls.SyntaxEditor)tab_overloads.SelectedTab.Controls[0]; }
        }

        public void ReloadReturnTypes()
        {
            cmb_types.Items.Clear();
            List<string> values = Classes.Helper.DataTypes.GetValues();
            values.Add("void");
            values.Sort();
            cmb_types.Items.AddRange(values.ToArray());

            if (cmb_types.Items.Contains(my_function.ReturnType[tab_overloads.SelectedIndex]))
            {
                cmb_types.SelectedItem = my_function.ReturnType[tab_overloads.SelectedIndex];
            }
            else
            {
                cmb_types.SelectedItem = "void";
            }
        }

        private void frm_function_Load(object sender, EventArgs e)
        {
            this.Location = my_function.Location;
            this.Size = my_function.Size;
            this.Text = this.Tag.ToString();
            my_function.Visible = true;
            this.Closing += new CancelEventHandler(frm_shader_Closing);
            this.LocationChanged += new EventHandler(FormMoved);
            this.ResizeEnd += new EventHandler(FormResized);

            List<string> values = Classes.Helper.DataTypes.GetValues();
            values.Add("void");
            values.Sort();
            cmb_types.Items.AddRange(values.ToArray());

            for (int i = 0; i < my_function.Code.Count; i++)
            {
                TabPage new_tab = new TabPage(my_function.Signatures[i]);

                CustomControls.SyntaxEditor new_editor = new FXED.CustomControls.SyntaxEditor();
                new_tab.Controls.Add(new_editor);
                new_editor.Dock = DockStyle.Fill;
                new_editor.Font = Properties.Settings.Default.EditorFont;
                new_editor.ContextMenuStrip = this.cms_options;
                new_editor.Text = my_function.Code[i];
                new_editor.TextChanged += new EventHandler(UpdateCode);

                tab_overloads.TabPages.Add(new_tab);

                cmb_types.SelectedItem = my_function.ReturnType[i];
                UpdateParameters(i);
            }

            btn_remove_overload.Enabled = my_function.Signatures.Count > 1;

            UpdateObjects();
        }

        public void UpdateColors()
        {
            for (int i = 0; i < tab_overloads.TabPages.Count; i++)
            {
                ((CustomControls.SyntaxEditor)tab_overloads.TabPages[i].Controls[0]).UpdateColors();
            }
        }

        public void UpdateFonts()
        {
            for (int i = 0; i < tab_overloads.TabPages.Count; i++)
            {
                ((CustomControls.SyntaxEditor)tab_overloads.TabPages[i].Controls[0]).UpdateFont();
            }
        }

        public void UpdateObjects()
        {
            CustomControls.SyntaxEditor editor;

            for (int i = 0; i < tab_overloads.TabPages.Count; i++)
            {
                editor = (CustomControls.SyntaxEditor)tab_overloads.TabPages[i].Controls[0];
                editor.ObjectDescriptors.Clear();
                foreach (string obj in Classes.Variables.objects)
                {
                    editor.AddObject(obj);
                }
                editor.PerformHighlighting();
            }
        }

        public void UpdateParameters(int index)
        {
            CurrentEditor.ParameterDescriptors.Clear();

            foreach (Classes.Objects.FunctionParameter param in my_function.Parameters[index])
            {
                CurrentEditor.AddParameter(param.Name);
            }

            CurrentEditor.PerformHighlighting();
        }

        #region Text Operations

        private void CutText(object sender, EventArgs e)
        {
            CurrentEditor.Cut();
        }

        private void CopyText(object sender, EventArgs e)
        {
            CurrentEditor.Copy();
        }

        private void PasteText(object sender, EventArgs e)
        {
            CurrentEditor.Paste();
        }

        private void SelectAllText(object sender, EventArgs e)
        {
            CurrentEditor.SelectAll();
        }

        private void DeleteText(object sender, EventArgs e)
        {
            CurrentEditor.SelectedText = string.Empty;
        }

        private void UndoText(object sender, EventArgs e)
        {
            CurrentEditor.Undo();
        }

        private void RedoText(object sender, EventArgs e)
        {
            CurrentEditor.Redo();
        }

        private void UpdateCode(object sender, EventArgs e)
        {
            CurrentEditor.PerformHighlighting();
            my_function.Code[tab_overloads.SelectedIndex] = CurrentEditor.Text;
            Classes.Functions.ProjectChanged(); 
        }

        private void InsertFunction(object sender, EventArgs e)
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;
            if (CurrentEditor.SelectedText != string.Empty)
            {
                CurrentEditor.SelectedText = item.Tag.ToString();
            }
            else
            {
                CurrentEditor.Text = CurrentEditor.Text.Insert(CurrentEditor.SelectionStart, item.Tag.ToString());
            }
        }

        private void ShortcutPressed(object sender, KeyEventArgs e)
        {
            if (e.Control)
            {
                switch (e.KeyCode)
                {
                    case Keys.Y: CurrentEditor.Redo(); break;
                    case Keys.Z: CurrentEditor.Undo(); break;
                }
            }
        }

        private void CommentText(object sender, EventArgs e)
        {
            PrependToLine("//");
        }

        private void IndentText(object sender, EventArgs e)
        {
            PrependToLine("\t");
        }

        private void PrependToLine(string value)
        {
            int start = CurrentEditor.SelectionStart;
            int end = CurrentEditor.SelectionStart + CurrentEditor.SelectionLength - 1;
            bool zeroWidth = (CurrentEditor.SelectionLength == 0);
            int pos = FindStartOfFirstLine();
            bool selectionStartsAtBOL = start == pos;
            while (pos <= end || zeroWidth)
            {
                CurrentEditor.SelectionStart = pos;
                CurrentEditor.SelectionLength = 0;
                CurrentEditor.SelectedText = value;
                end += value.Length;
                while (pos <= end && CurrentEditor.Text[pos++] != '\n') { }
                if (zeroWidth) { break; }
            }
            if (!selectionStartsAtBOL || zeroWidth)
            {
                start += value.Length;
            }

            CurrentEditor.Select(start, end - start + 1);
        }

        private int FindStartOfFirstLine()
        {
            int pos = CurrentEditor.SelectionStart - 1;

            while (pos > 0)
            {
                if (CurrentEditor.Text[pos] == '\n') { break; }
                pos--;
            }

            if (pos != 0 || CurrentEditor.Text[pos] == '\n') { pos++; }

            return pos;
        }

        private void RemovePrefixFromLineRange(bool isComment, params string[] prefixList)
        {
            int rememberStart = CurrentEditor.SelectionStart;
            int endPos = CurrentEditor.SelectionStart + System.Math.Max(0, CurrentEditor.SelectionLength - 1);
            bool zeroWidth = (CurrentEditor.SelectionLength == 0);
            int pos = FindStartOfFirstLine();
            bool selectionStartsAtBOL = rememberStart == pos;
            string foundPrefix = "";
            bool cursorStartsBeforePrefix = false;

            while ((pos < CurrentEditor.TextLength) && (pos <= endPos))
            {
                if (isComment)
                {
                    while (pos < CurrentEditor.TextLength && CurrentEditor.Text[pos] != '\n' && Char.IsWhiteSpace(CurrentEditor.Text[pos]))
                    { pos++; }
                }

                cursorStartsBeforePrefix |= (rememberStart < pos);

                foreach (var prefix in prefixList)
                {
                    if (DeletePrefix(prefix, pos, ref endPos)) { foundPrefix = prefix; break; }
                }
                while (pos <= endPos && pos < CurrentEditor.TextLength && CurrentEditor.Text[pos++] != '\n') { }

            }

            if (!cursorStartsBeforePrefix)
            { rememberStart = System.Math.Max(0, rememberStart - foundPrefix.Length); }

            CurrentEditor.SelectionStart = rememberStart;
            if (!zeroWidth)
            { CurrentEditor.Select(rememberStart, endPos - rememberStart + 1); }
        }

        private bool DeletePrefix(string prefix, int pos, ref int endPos)
        {
            CurrentEditor.Select(pos, prefix.Length);
            if (CurrentEditor.SelectedText.Equals(prefix))
            {
                CurrentEditor.SelectedText = "";
                endPos -= prefix.Length;
                return true;
            }
            return false;
        }

        private void OutdentText(object sender, EventArgs e)
        {
            RemovePrefixFromLineRange(false, new string[] { "\t", "    " });
        }

        private void UncommentText(object sender, EventArgs e)
        {
            RemovePrefixFromLineRange(true, new string[] { "//", "//\t", "//" });
        }


        #endregion

        #region Form Functions

        private void FormMoved(object sender, EventArgs e)
        {
            my_function.Location = this.Location;
        }

        private void FormResized(object sender, EventArgs e)
        {
            my_function.Size = this.Size;
        }

        void frm_shader_Closing(object sender, CancelEventArgs e)
        {
            my_function.Visible = false;
        }

        #endregion

        private void EditParameters(object sender, EventArgs e)
        {
            Classes.Variables.current_function = my_function;

            Classes.Variables.temp_params = (List<Classes.Objects.FunctionParameter>)Classes.Functions.Clone(my_function.Parameters[my_function.CurrentOverload]);

            Classes.Helper.RuntimeCollectionEditor parameter_collection = new Classes.Helper.RuntimeCollectionEditor();
            parameter_collection.EditCollection(Classes.Variables.temp_params,new Classes.Helper.FunctionParameterCollection(Classes.Variables.temp_params), this);

            if (Classes.Variables.edit_ok)
            {
                my_function.Signatures[tab_overloads.SelectedIndex] = my_function.GenerateParameterSignature(Classes.Variables.temp_params);
                tab_overloads.SelectedTab.Text = my_function.Signatures[tab_overloads.SelectedIndex];

                my_function.Parameters[my_function.CurrentOverload].Clear();
                foreach (Classes.Objects.FunctionParameter param in Classes.Variables.temp_params)
                {
                    my_function.Parameters[my_function.CurrentOverload].Add(param);
                }
                Classes.Variables.temp_params.Clear();

                UpdateParameters(tab_overloads.SelectedIndex);
            }
        }

        private void SetReturnType(object sender, EventArgs e)
        {
            my_function.ReturnType[tab_overloads.SelectedIndex] = cmb_types.SelectedItem.ToString();
        }

        private void NewTabSelected(object sender, EventArgs e)
        {
            my_function.CurrentOverload = tab_overloads.SelectedIndex;
            cmb_types.SelectedItem = my_function.ReturnType[tab_overloads.SelectedIndex];

            btn_remove_overload.Enabled = my_function.Signatures.Count > 1;
        }

        private void AddOverload(object sender, EventArgs e)
        {
            Classes.Variables.current_function = my_function;
            Classes.Variables.temp_params = new List<FXED.Classes.Objects.FunctionParameter>();
            my_function.CurrentOverload = -1;
            // show editor

            Classes.Helper.RuntimeCollectionEditor parameter_collection = new Classes.Helper.RuntimeCollectionEditor();
            parameter_collection.EditCollection(Classes.Variables.temp_params,new Classes.Helper.FunctionParameterCollection(Classes.Variables.temp_params), this);

            if (Classes.Variables.temp_params.Count != 0)
            {
                my_function.Parameters.Add(Classes.Variables.temp_params);
                my_function.ReturnType.Add("void");
                my_function.Signatures.Add(my_function.GenerateParameterSignature(Classes.Variables.temp_params));
                my_function.Code.Add(string.Empty);

                TabPage new_tab = new TabPage(my_function.Signatures[my_function.Parameters.Count - 1]);

                CustomControls.SyntaxEditor new_editor = new FXED.CustomControls.SyntaxEditor();
                new_tab.Controls.Add(new_editor);
                new_editor.Dock = DockStyle.Fill;
                new_editor.Font = Properties.Settings.Default.EditorFont;
                new_editor.ContextMenuStrip = this.cms_options;
                new_editor.Text = my_function.Code[my_function.Parameters.Count - 1];
                new_editor.TextChanged += new EventHandler(UpdateCode);

                tab_overloads.TabPages.Add(new_tab);

                cmb_types.SelectedItem = my_function.ReturnType[my_function.Parameters.Count - 1];

                tab_overloads.SelectedTab = new_tab;

                UpdateObjects();
            }
        }

        private void RemoveOverload(object sender, EventArgs e)
        {
            my_function.Parameters.RemoveAt(tab_overloads.SelectedIndex);
            my_function.ReturnType.RemoveAt(tab_overloads.SelectedIndex);
            my_function.Code.RemoveAt(tab_overloads.SelectedIndex);
            my_function.Signatures.RemoveAt(tab_overloads.SelectedIndex);
            tab_overloads.TabPages.Remove(tab_overloads.SelectedTab);
        }   
    }
}
