﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Diagnostics;
using AcknexWrapper;

namespace FXED.Forms
{
    public enum ShaderIcons { pass, variable, array, vector, color, matrix, texture, sampler, function };

    public partial class frm_main : Form
    {
        ToolTip ttip = new ToolTip();

        public frm_main()
        {
            Classes.Variables.main_form = this;
            InitializeComponent();
            this.Closing += new CancelEventHandler(frm_main_Closing);
        }

        private void frm_main_Shown(object sender, EventArgs e)
        {
            Forms.frm_startup_options show_options = new frm_startup_options();
            show_options.ShowDialog();
        }

        #region Creating Shader Components

        private void AddNewPass(object sender, EventArgs e)
        {
            Classes.Objects.Pass new_pass = new Classes.Objects.Pass(Classes.Functions.GetNewName("Pass"));
            new_pass.VertexShader.Visible = false;
            new_pass.PixelShader.Visible = false;

            if (Classes.Variables.shader_type == "pp")
            {
                new_pass.VertexShader = null;
            }
            Classes.Variables.my_effect.Passes.Add(new_pass);

            ListViewItem new_pass_item = new ListViewItem(new_pass.Name, (int)ShaderIcons.pass);
            new_pass_item.Name = new_pass.Name;
            new_pass_item.Tag = "Pass";
            lstview_components.Items.Add(new_pass_item);

            pgr_main.SelectedObject = new_pass;
            Classes.Functions.ProjectChanged(); 
        }

        private void AddNewVariable(object sender, EventArgs e)
        {
            Classes.Objects.Variable new_variable = new Classes.Objects.Variable(Classes.Functions.GetNewName("Variable"),false);
            Classes.Variables.my_effect.Variables.Add(new_variable);

            ListViewItem new_variable_item = new ListViewItem(new_variable.Name, (int)ShaderIcons.variable);
            new_variable_item.Name = new_variable.Name;
            Classes.Variables.objects.Add(new_variable.Name);
            new_variable_item.Tag = "Variable";

            lstview_components.Items.Add(new_variable_item);

            pgr_main.SelectedObject = new_variable;
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddPredefinedVariable(object sender, EventArgs e)
        {
            ToolStripMenuItem item = ((ToolStripMenuItem)sender);

            if (Classes.Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == item.Text; }) != null)
            {
                return;
            }

            Classes.Objects.Variable new_variable = new Classes.Objects.Variable(item.Text, true);

            if (new_variable.Name == "iRepeat") { new_variable.DataType = Classes.Objects.BasicDataTypes.@int; }
            Classes.Variables.my_effect.Variables.Add(new_variable);

            ListViewItem new_variable_item = new ListViewItem(new_variable.Name, (int)ShaderIcons.variable);
            new_variable_item.Name = new_variable.Name;
            Classes.Variables.objects.Add(new_variable.Name);
            new_variable_item.Tag = "Variable";
            lstview_components.Items.Add(new_variable_item);
            new_variable.Comment = item.Tag.ToString();

            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddNewArray(object sender, EventArgs e)
        {
            Classes.Objects.Array new_array = new Classes.Objects.Array(Classes.Functions.GetNewName("Array"),((ToolStripMenuItem)sender).Text);
            Classes.Variables.my_effect.Arrays.Add(new_array);

            ListViewItem new_array_item = new ListViewItem(new_array.Name, (int)ShaderIcons.array);
            new_array_item.Name = new_array.Name;
            Classes.Variables.objects.Add(new_array.Name);
            new_array_item.Tag = "Array";
            lstview_components.Items.Add(new_array_item);

            pgr_main.SelectedObject = new_array;
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddNewVector(object sender, EventArgs e)
        {
            Classes.Objects.Vector new_vector = new Classes.Objects.Vector(Classes.Functions.GetNewName("Vector"), false);
            Classes.Variables.my_effect.Vectors.Add(new_vector);

            ListViewItem new_vector_item = new ListViewItem(new_vector.Name, (int)ShaderIcons.vector);
            new_vector_item.Name = new_vector.Name;
            Classes.Variables.objects.Add(new_vector.Name);
            new_vector_item.Tag = "Vector";
            lstview_components.Items.Add(new_vector_item);

            pgr_main.SelectedObject = new_vector;
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddPredefinedVector(object sender, EventArgs e)
        {
            ToolStripMenuItem item = ((ToolStripMenuItem)sender);

            if (Classes.Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent vec) { return vec.Name == item.Text; }) != null)
            {
                return;
            }

            Classes.Objects.Vector new_vector = new Classes.Objects.Vector(item.Text, true);

            if (new_vector.Name == "vecFogColor") { new_vector.Elements = "3"; }
            Classes.Variables.my_effect.Vectors.Add(new_vector);

            ListViewItem new_vector_item = new ListViewItem(new_vector.Name, (int)ShaderIcons.vector);
            new_vector_item.Name = new_vector.Name;

            switch (new_vector.Name)
            {
                case "vecLightPos[8]": Classes.Variables.objects.Add("vecLightPos"); break;
                case "vecLightViewPos[8]": Classes.Variables.objects.Add("vecLightViewPos"); break;
                case "vecLightColor[8]": Classes.Variables.objects.Add("vecLightColor"); break;
                case "vecLightDir[8]": Classes.Variables.objects.Add("vecLightDir"); break;
                default: Classes.Variables.objects.Add(new_vector.Name); break;
            }

            new_vector_item.Tag = "Vector";
            lstview_components.Items.Add(new_vector_item);

            new_vector.Comment = item.Tag.ToString();
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddNewColor(object sender, EventArgs e)
        {
            Classes.Objects.Color new_color = new Classes.Objects.Color(Classes.Functions.GetNewName("Color"));
            Classes.Variables.my_effect.Colors.Add(new_color);

            ListViewItem new_color_item = new ListViewItem(new_color.Name, (int)ShaderIcons.color);
            new_color_item.Name = new_color.Name;
            Classes.Variables.objects.Add(new_color.Name);
            new_color_item.Tag = "Color";
            lstview_components.Items.Add(new_color_item);

            pgr_main.SelectedObject = new_color;
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddNewMatrix(object sender, EventArgs e)
        {
            Classes.Objects.Matrix new_matrix = new Classes.Objects.Matrix(Classes.Functions.GetNewName("Matrix"),false);
            Classes.Variables.my_effect.Matrices.Add(new_matrix);

            ListViewItem new_matrix_item = new ListViewItem(new_matrix.Name, (int)ShaderIcons.matrix);
            new_matrix_item.Name = new_matrix.Name;
            Classes.Variables.objects.Add(new_matrix.Name);
            new_matrix_item.Tag = "Matrix";
            lstview_components.Items.Add(new_matrix_item);

            pgr_main.SelectedObject = new_matrix;
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddPredefinedMatrix(object sender, EventArgs e)
        {
            ToolStripMenuItem item = ((ToolStripMenuItem)sender);

            if (Classes.Variables.my_effect.Matrices.Find(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == item.Text; }) != null)
            {
                return;
            }

            Classes.Objects.Matrix new_matrix = new Classes.Objects.Matrix(item.Text, true);
            Classes.Variables.my_effect.Matrices.Add(new_matrix);

            ListViewItem new_matrix_item = new ListViewItem(new_matrix.Name, (int)ShaderIcons.matrix);
            new_matrix_item.Name = new_matrix.Name;
            Classes.Variables.objects.Add(new_matrix.Name);
            new_matrix_item.Tag = "Matrix";
            lstview_components.Items.Add(new_matrix_item);
            new_matrix.Comment = item.Tag.ToString();
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddPredefinedTexture(object sender, EventArgs e)
        {
            ToolStripMenuItem item = ((ToolStripMenuItem)sender);

            if (Classes.Variables.my_effect.Textures.Find(delegate(Classes.Objects.ParentComponent tex) { return tex.Name == item.Text; }) != null)
            {
                return;
            }

            Classes.Objects.Texture new_texture = new Classes.Objects.Texture(item.Text, true);
            Classes.Variables.my_effect.Textures.Add(new_texture);

            ListViewItem new_texture_item = new ListViewItem(new_texture.Name, (int)ShaderIcons.texture);
            new_texture_item.Name = new_texture.Name;
            Classes.Variables.objects.Add(new_texture.Name);
            new_texture_item.Tag = "Texture";
            lstview_components.Items.Add(new_texture_item);

            new_texture.Comment = item.Tag.ToString();
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddNewSampler(object sender, EventArgs e)
        {
            Classes.Objects.Sampler new_sampler = new Classes.Objects.Sampler(Classes.Functions.GetNewName("Sampler"));
            Classes.Variables.my_effect.Samplers.Add(new_sampler);

            ListViewItem new_sampler_item = new ListViewItem(new_sampler.Name, (int)ShaderIcons.sampler);
            new_sampler_item.Name = new_sampler.Name;
            Classes.Variables.objects.Add(new_sampler.Name);
            new_sampler_item.Tag = "Sampler";
            lstview_components.Items.Add(new_sampler_item);

            pgr_main.SelectedObject = new_sampler;
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void AddNewFunction(object sender, EventArgs e)
        {
            Classes.Objects.Function new_function = new Classes.Objects.Function(Classes.Functions.GetNewName("Function"));
            Classes.Variables.my_effect.Functions.Add(new_function);

            ListViewItem new_function_item = new ListViewItem(new_function.Name, (int)ShaderIcons.function);
            new_function_item.Name = new_function_item.Text;
            Classes.Variables.objects.Add(new_function.Name);
            new_function_item.Tag = "Function";
            lstview_components.Items.Add(new_function_item);

            Forms.frm_function new_function_form = new frm_function(new_function);
            new_function_form.Tag = new_function.Name;
            new_function_form.MdiParent = this;
            new_function_form.Show();

            pgr_main.SelectedObject = new_function;
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }
     
        #endregion

        #region Shader Components Functions

        public void LoadComponents()
        {
            EnableMenuItems(true);

            if (Classes.Variables.shader_type == "entity") { tsitem_vertex_shader.Enabled = true; }
            else { tsitem_vertex_shader.Enabled = false; }

            #region Add Components

            foreach (Classes.Objects.ParentComponent variable in Classes.Variables.my_effect.Variables)
            {
                ListViewItem new_variable = new ListViewItem(variable.Name, (int)ShaderIcons.variable);
                new_variable.Name = variable.Name;
                Classes.Variables.objects.Add(variable.Name);
                new_variable.Tag = "Variable";
                new_variable.ToolTipText = variable.Comment;
                lstview_components.Items.Add(new_variable);
            }

            foreach (Classes.Objects.ParentComponent array in Classes.Variables.my_effect.Arrays)
            {
                ListViewItem new_array = new ListViewItem(array.Name, (int)ShaderIcons.array);
                new_array.Name = array.Name;
                Classes.Variables.objects.Add(array.Name);
                new_array.Tag = "Array";
                new_array.ToolTipText = array.Comment;
                lstview_components.Items.Add(new_array);
            }

            foreach (Classes.Objects.ParentComponent vector in Classes.Variables.my_effect.Vectors)
            {
                ListViewItem new_vector = new ListViewItem(vector.Name, (int)ShaderIcons.vector);
                new_vector.Name = vector.Name;

                switch (vector.Name)
                {
                    case "vecLightPos[8]": Classes.Variables.objects.Add("vecLightPos"); break;
                    case "vecLightViewPos[8]": Classes.Variables.objects.Add("vecLightViewPos"); break;
                    case "vecLightColor[8]": Classes.Variables.objects.Add("vecLightColor"); break;
                    case "vecLightDir[8]": Classes.Variables.objects.Add("vecLightDir"); break;
                    default: Classes.Variables.objects.Add(vector.Name); break;
                }

                new_vector.Tag = "Vector";
                new_vector.ToolTipText = vector.Comment;
                lstview_components.Items.Add(new_vector);
            }

            foreach (Classes.Objects.ParentComponent color in Classes.Variables.my_effect.Colors)
            {
                ListViewItem new_color = new ListViewItem(color.Name, (int)ShaderIcons.color);
                new_color.Name = color.Name;
                Classes.Variables.objects.Add(color.Name);
                new_color.Tag = "Color";
                new_color.ToolTipText = color.Comment;
                lstview_components.Items.Add(new_color);
            }

            foreach (Classes.Objects.ParentComponent matrix in Classes.Variables.my_effect.Matrices)
            {
                ListViewItem new_matrix = new ListViewItem(matrix.Name, (int)ShaderIcons.matrix);
                new_matrix.Name = matrix.Name;
                Classes.Variables.objects.Add(matrix.Name);
                new_matrix.Tag = "Matrix";
                new_matrix.ToolTipText = matrix.Comment;
                lstview_components.Items.Add(new_matrix);
            }

            foreach (Classes.Objects.ParentComponent texture in Classes.Variables.my_effect.Textures)
            {
                ListViewItem new_texture = new ListViewItem(texture.Name, (int)ShaderIcons.texture);
                new_texture.Name = texture.Name;
                Classes.Variables.objects.Add(texture.Name);
                new_texture.Tag = "Texture";
                new_texture.ToolTipText = texture.Comment;
                lstview_components.Items.Add(new_texture);
            }

            foreach (Classes.Objects.ParentComponent sampler in Classes.Variables.my_effect.Samplers)
            {
                ListViewItem new_sampler = new ListViewItem(sampler.Name, (int)ShaderIcons.sampler);
                new_sampler.Name = sampler.Name;
                Classes.Variables.objects.Add(sampler.Name);
                new_sampler.Tag = "Sampler";
                new_sampler.ToolTipText = sampler.Comment;
                lstview_components.Items.Add(new_sampler);
            }

            foreach (Classes.Objects.Function function in Classes.Variables.my_effect.Functions)
            {
                ListViewItem new_function = new ListViewItem(function.Name, (int)ShaderIcons.function);
                new_function.Name = new_function.Text;
                new_function.Tag = "Function";
                new_function.ToolTipText = function.Comment;
                lstview_components.Items.Add(new_function);
                Classes.Variables.objects.Add(function.Name);

                if (function.Visible)
                {
                    Forms.frm_function function_form = new frm_function(function);
                    function_form.Tag = function.Name;
                    function_form.MdiParent = this;
                    function_form.Show();
                    function_form.BringToFront();
                }

            }

            #endregion

            #region Add Passes

            foreach (Classes.Objects.Pass pass in Classes.Variables.my_effect.Passes)
            {
                ListViewItem new_pass = new ListViewItem(pass.Name, (int)ShaderIcons.pass);
                new_pass.Name = pass.Name;
                new_pass.Tag = "Pass";
                new_pass.ToolTipText = pass.Comment;
                lstview_components.Items.Add(new_pass);

                if (pass.VertexShader != null)
                {
                    if (pass.VertexShader.Visible)
                    {
                        Forms.frm_shader vertex_shader = new frm_shader(pass.VertexShader);
                        vertex_shader.Tag = pass.Name;
                        vertex_shader.MdiParent = this;
                        vertex_shader.Show();
                        vertex_shader.BringToFront();
                    }
                }

                if (pass.PixelShader != null)
                {
                    if (pass.PixelShader.Visible)
                    {
                        Forms.frm_shader pixel_shader = new frm_shader(pass.PixelShader);
                        pixel_shader.Tag = pass.Name;
                        pixel_shader.MdiParent = this;
                        pixel_shader.Show();
                        pixel_shader.BringToFront();
                    }
                }

                if (pass.States.Visible)
                {
                    Forms.frm_renderstates states = new frm_renderstates(pass.States);
                    states.Tag = pass.Name;
                    states.MdiParent = this;
                    states.Show();
                    states.BringToFront();
                }

            }

            #endregion

            #region Add Skins

            Classes.Variables.skins = new frm_skins();
            Classes.Variables.skins.MdiParent = this;
            Classes.Variables.skins.BringToFront();
            if (Classes.Variables.my_effect.SkinsVisible) { Classes.Variables.skins.Show(); }
            tsitem_skins.Checked = Classes.Variables.my_effect.SkinsVisible;

            #endregion

            #region Add Preview

            Classes.Variables.preview = new frm_preview();
            Classes.Variables.preview.MdiParent = this;
            Classes.Variables.preview.Show();
            Classes.Variables.preview.BringToFront();

            #endregion

            #region Add Skills

            Classes.Variables.skills = new frm_skills();
            Classes.Variables.skills.MdiParent = this;
            Classes.Variables.skills.BringToFront();
            if (Classes.Variables.my_effect.SkillsVisible) { Classes.Variables.skills.Show(); }
            tsitem_skills.Checked = Classes.Variables.my_effect.SkillsVisible;

            Classes.Variables.skills.SetSkillCaptions();

            #endregion

            #region Check Includes

            foreach (ToolStripMenuItem item in tstrip_includes.DropDownItems )
            {
                if (Classes.Variables.my_effect.Includes.Contains(item.Text))
                {
                    item.Checked = true;
                }

            }
            #endregion

            pgr_main.SelectedObject = Classes.Variables.my_effect;

            Classes.Variables.project_changed = false;
        }

        private void GetClickedComponent(object sender, MouseEventArgs e)
        {
            ListViewItem item = lstview_components.GetItemAt(e.X, e.Y);
            Classes.Objects.ParentComponent component = null;
            if (item != null)
            {
                if (e.Button == MouseButtons.Left)
                {
                    switch (item.Tag.ToString())
                    {
                        case "Pass": pgr_main.SelectedObject = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == item.Text; }); return;
                        case "Array": component = Classes.Variables.my_effect.Arrays.Find(delegate(Classes.Objects.ParentComponent arr) { return arr.Name == item.Text; }); break;
                        case "Variable": component = Classes.Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == item.Text; }); break;
                        case "Vector": component = Classes.Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent vec) { return vec.Name == item.Text; }); break;
                        case "Color": component = Classes.Variables.my_effect.Colors.Find(delegate(Classes.Objects.ParentComponent col) { return col.Name == item.Text; }); break;
                        case "Matrix": component = Classes.Variables.my_effect.Matrices.Find(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == item.Text; }); break;
                        case "Texture": component = Classes.Variables.my_effect.Textures.Find(delegate(Classes.Objects.ParentComponent tex) { return tex.Name == item.Text; }); break;
                        case "Sampler": component = Classes.Variables.my_effect.Samplers.Find(delegate(Classes.Objects.ParentComponent sam) { return sam.Name == item.Text; }); break;
                        case "Function": component = Classes.Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == item.Text; }); break;
                    }

                    if (component != null)
                    {
                        if (!component.PreDefined)
                        {
                            pgr_main.SelectedObject = component;
                        }
                        else
                        {
                            pgr_main.SelectedObject = null;
                        }
                    }
                    return;
                }

                if (item.Tag.ToString() == "Pass") 
                { 
                    cms_passes.Show(lstview_components.PointToScreen(e.Location)); 
                    return; 
                }
                else if (item.Tag.ToString() == "Function")
                {
                    cms_functions.Show(lstview_components.PointToScreen(e.Location));
                    return;
                }
               
                cms_components.Show(lstview_components.PointToScreen(e.Location));
            }
        }

        private void CheckComponentSettings(object sender, CancelEventArgs e)
        {
            Classes.Objects.ParentComponent component = null;

            switch (lstview_components.SelectedItems[0].Tag.ToString())
            {
                case "Variable": component = Classes.Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Array": component = Classes.Variables.my_effect.Arrays.Find(delegate(Classes.Objects.ParentComponent arr) { return arr.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Vector": component = Classes.Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent vec) { return vec.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Color": component = Classes.Variables.my_effect.Colors.Find(delegate(Classes.Objects.ParentComponent col) { return col.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Matrix": component = Classes.Variables.my_effect.Matrices.Find(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Texture": component = Classes.Variables.my_effect.Textures.Find(delegate(Classes.Objects.ParentComponent tex) { return tex.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Sampler": component = Classes.Variables.my_effect.Samplers.Find(delegate(Classes.Objects.ParentComponent sam) { return sam.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Function": component = Classes.Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == lstview_components.SelectedItems[0].Text; }); break;
            }

            if (component.PreDefined)
            {
                tsitem_rename_component.Enabled = false;
                tsitem_duplicate_component.Enabled = false;
            }
            else
            {
                tsitem_rename_component.Enabled = true;
                tsitem_duplicate_component.Enabled = true;
            }
        }

        private void DeleteComponent(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete the selected component?", "Confirm deletion", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                switch (lstview_components.SelectedItems[0].Tag.ToString())
                {
                    case "Pass":
                        Classes.Variables.my_effect.Passes.RemoveAll(delegate(Classes.Objects.Pass pass) { return pass.Name == lstview_components.SelectedItems[0].Text; });
                        ClosePassChildren(lstview_components.SelectedItems[0].Text,"Both");
                        Classes.Functions.UpdateReferences("Pass", lstview_components.SelectedItems[0].Text, "(none)");
                        break;

                    case "Variable": Classes.Variables.my_effect.Variables.RemoveAll(delegate(Classes.Objects.ParentComponent var) { return var.Name == lstview_components.SelectedItems[0].Text; }); break;
                    case "Array": Classes.Variables.my_effect.Arrays.RemoveAll(delegate(Classes.Objects.ParentComponent arr) { return arr.Name == lstview_components.SelectedItems[0].Text; }); break;
                    case "Vector": Classes.Variables.my_effect.Vectors.RemoveAll(delegate(Classes.Objects.ParentComponent vec) { return vec.Name == lstview_components.SelectedItems[0].Text; }); break;
                    case "Color": Classes.Variables.my_effect.Colors.RemoveAll(delegate(Classes.Objects.ParentComponent col) { return col.Name == lstview_components.SelectedItems[0].Text; }); break;
                    case "Matrix": Classes.Variables.my_effect.Matrices.RemoveAll(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == lstview_components.SelectedItems[0].Text; }); break;
                    case "Texture": 
                        Classes.Variables.my_effect.Textures.RemoveAll(delegate(Classes.Objects.ParentComponent tex) { return tex.Name == lstview_components.SelectedItems[0].Text; });
                        Classes.Functions.UpdateReferences("Texture", lstview_components.SelectedItems[0].Text, "(none)");
                        break;
                    case "Sampler": Classes.Variables.my_effect.Samplers.RemoveAll(delegate(Classes.Objects.ParentComponent sam) { return sam.Name == lstview_components.SelectedItems[0].Text; }); break;

                    case "Function":
                        Classes.Variables.my_effect.Functions.RemoveAll(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == lstview_components.SelectedItems[0].Text; });
                        CloseFunctionChildren(lstview_components.SelectedItems[0].Text);
                        break;
                }

                if (lstview_components.SelectedItems[0].Tag.ToString() != "Pass")
                {
                    switch (lstview_components.SelectedItems[0].Text)
                    {
                        case "vecLightPos[8]": Classes.Variables.objects.Remove("vecLightPos"); break;
                        case "vecLightViewPos[8]": Classes.Variables.objects.Remove("vecLightViewPos"); break;
                        case "vecLightColor[8]": Classes.Variables.objects.Remove("vecLightColor"); break;
                        case "vecLightDir[8]": Classes.Variables.objects.Remove("vecLightDir"); break;
                        default: Classes.Variables.objects.Remove(lstview_components.SelectedItems[0].Text); break;
                    }
                    
                    Classes.Functions.UpdateHighlighting();
                }

                lstview_components.SelectedItems[0].Remove();
                pgr_main.SelectedObject = null;
                Classes.Functions.ProjectChanged(); 
            }
        }

        private void RenameComponent(object sender, EventArgs e)
        {
            lstview_components.SelectedItems[0].BeginEdit();
        }

        private void DuplicateComponent(object sender, EventArgs e)
        {
            string clone_name = string.Empty;

            switch (lstview_components.SelectedItems[0].Tag.ToString())
            {
                case "Pass":
                    Classes.Objects.Pass clone_pass = (Classes.Objects.Pass)Classes.Functions.Clone(Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == lstview_components.SelectedItems[0].Text; }));
                    clone_pass.SetName(Classes.Functions.GetNewName(lstview_components.SelectedItems[0].Tag.ToString()));
                    clone_pass.VertexShader.Visible = false;
                    clone_pass.VertexShader.Location = new Point(0, 0);
                    clone_pass.PixelShader.Visible = false;
                    clone_pass.PixelShader.Location = new Point(0, 0);
                    Classes.Variables.my_effect.Passes.Add(clone_pass);

                    clone_name = clone_pass.Name;
                    pgr_main.SelectedObject = clone_pass;
                    break;

                case "Variable":
                    Classes.Objects.ParentComponent clone_variable = (Classes.Objects.ParentComponent)Classes.Functions.Clone(Classes.Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent com) { return com.Name == lstview_components.SelectedItems[0].Text; }));
                    clone_variable.SetName(Classes.Functions.GetNewName(lstview_components.SelectedItems[0].Tag.ToString()));
                    Classes.Variables.my_effect.Variables.Add(clone_variable);

                    clone_name = clone_variable.Name;
                    pgr_main.SelectedObject = clone_variable;
                    break;

                case "Array":
                    Classes.Objects.ParentComponent clone_array = (Classes.Objects.ParentComponent)Classes.Functions.Clone(Classes.Variables.my_effect.Arrays.Find(delegate(Classes.Objects.ParentComponent com) { return com.Name == lstview_components.SelectedItems[0].Text; }));
                    clone_array.SetName(Classes.Functions.GetNewName(lstview_components.SelectedItems[0].Tag.ToString()));
                    Classes.Variables.my_effect.Arrays.Add(clone_array);

                    clone_name = clone_array.Name;
                    pgr_main.SelectedObject = clone_array;
                    break;

                case "Vector":
                    Classes.Objects.ParentComponent clone_vector = (Classes.Objects.ParentComponent)Classes.Functions.Clone(Classes.Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent com) { return com.Name == lstview_components.SelectedItems[0].Text; }));
                    clone_vector.SetName(Classes.Functions.GetNewName(lstview_components.SelectedItems[0].Tag.ToString()));
                    Classes.Variables.my_effect.Vectors.Add(clone_vector);

                    clone_name = clone_vector.Name;
                    pgr_main.SelectedObject = clone_vector;
                    break;

                case "Color":
                    Classes.Objects.ParentComponent clone_color = (Classes.Objects.ParentComponent)Classes.Functions.Clone(Classes.Variables.my_effect.Colors.Find(delegate(Classes.Objects.ParentComponent com) { return com.Name == lstview_components.SelectedItems[0].Text; }));
                    clone_color.SetName(Classes.Functions.GetNewName(lstview_components.SelectedItems[0].Tag.ToString()));
                    Classes.Variables.my_effect.Colors.Add(clone_color);

                    clone_name = clone_color.Name;
                    pgr_main.SelectedObject = clone_color;
                    break;

                case "Sampler":
                    Classes.Objects.ParentComponent clone_sampler = (Classes.Objects.ParentComponent)Classes.Functions.Clone(Classes.Variables.my_effect.Samplers.Find(delegate(Classes.Objects.ParentComponent com) { return com.Name == lstview_components.SelectedItems[0].Text; }));
                    clone_sampler.SetName(Classes.Functions.GetNewName(lstview_components.SelectedItems[0].Tag.ToString()));
                    Classes.Variables.my_effect.Samplers.Add(clone_sampler);

                    clone_name = clone_sampler.Name;
                    pgr_main.SelectedObject = clone_sampler;
                    break;

                case "Function":
                    Classes.Objects.ParentComponent clone_function = (Classes.Objects.ParentComponent)Classes.Functions.Clone(Classes.Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent com) { return com.Name == lstview_components.SelectedItems[0].Text; }));
                    clone_function.SetName(Classes.Functions.GetNewName(lstview_components.SelectedItems[0].Tag.ToString()));
                    Classes.Variables.my_effect.Functions.Add(clone_function);

                    clone_name = clone_function.Name;
                    pgr_main.SelectedObject = clone_function;
                    break;

            }

            ListViewItem new_item = new ListViewItem(clone_name, lstview_components.SelectedItems[0].ImageIndex);
            new_item.Name = new_item.Text;
            
            new_item.Tag = lstview_components.SelectedItems[0].Tag.ToString();
            lstview_components.Items.Add(new_item);

            if (lstview_components.SelectedItems[0].Tag.ToString() != "Pass")
            {
                Classes.Variables.objects.Add(clone_name);
            }
            Classes.Functions.UpdateHighlighting();
            Classes.Functions.ProjectChanged(); 
        }

        private void BeforeEdit(object sender, LabelEditEventArgs e)
        {
            Classes.Objects.ParentComponent component = null;

            switch (lstview_components.Items[e.Item].Tag.ToString())
            {
                case "Variable": component = Classes.Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Array": component = Classes.Variables.my_effect.Arrays.Find(delegate(Classes.Objects.ParentComponent arr) { return arr.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Vector": component = Classes.Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent vec) { return vec.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Color": component = Classes.Variables.my_effect.Colors.Find(delegate(Classes.Objects.ParentComponent col) { return col.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Matrix": component = Classes.Variables.my_effect.Matrices.Find(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Texture": component = Classes.Variables.my_effect.Textures.Find(delegate(Classes.Objects.ParentComponent tex) { return tex.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Sampler": component = Classes.Variables.my_effect.Samplers.Find(delegate(Classes.Objects.ParentComponent sam) { return sam.Name == lstview_components.SelectedItems[0].Text; }); break;
                case "Function": component = Classes.Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == lstview_components.SelectedItems[0].Text; }); break;
            }

            if (component != null)
            {
                if (component.PreDefined)
                {
                    e.CancelEdit = true;
                }
            }
        }

        private void AfterEdit(object sender, LabelEditEventArgs e)
        {
            if (Classes.Functions.CheckNameIsUnique(e.Label, "component"))
            {
                switch (lstview_components.Items[e.Item].Tag.ToString())
                {
                    case "Pass":
                        string old_name = lstview_components.Items[e.Item].Text;
                        Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == old_name; }).Name = e.Label;
                        Classes.Functions.UpdateReferences("Pass", old_name, e.Label);
                        break;
                    case "Variable": Classes.Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                    case "Array": Classes.Variables.my_effect.Arrays.Find(delegate(Classes.Objects.ParentComponent arr) { return arr.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                    case "Vector": Classes.Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                    case "Color": Classes.Variables.my_effect.Colors.Find(delegate(Classes.Objects.ParentComponent col) { return col.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                    case "Matrix": Classes.Variables.my_effect.Matrices.Find(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                    case "Texture": Classes.Variables.my_effect.Textures.Find(delegate(Classes.Objects.ParentComponent tex) { return tex.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                    case "Sampler": Classes.Variables.my_effect.Samplers.Find(delegate(Classes.Objects.ParentComponent sam) { return sam.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                    case "Function": Classes.Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == lstview_components.Items[e.Item].Text; }).Name = e.Label; break;
                }
                lstview_components.Items[e.Item].Name = e.Label;
                Classes.Functions.UpdateHighlighting();
                pgr_main.Refresh();
            }
            else
            {
                e.CancelEdit = true;
            }
        }

        public void RenameComponent(string old_name, string new_name)
        {
            lstview_components.Items.Find(old_name, true)[0].Text = new_name;
            lstview_components.Items.Find(old_name, true)[0].Name = new_name;
        }

        public void SetComponentTooltip(string name, string tooltip)
        {
            lstview_components.Items.Find(name, true)[0].ToolTipText = tooltip;
        }

        // functions

        private void ViewFunction(object sender, EventArgs e)
        {
            Forms.frm_function function = (Forms.frm_function)FindForm(lstview_components.SelectedItems[0].Text);
            if (function != null)
            {
                function.BringToFront();
            }
            else
            {
                Classes.Objects.Function selected_function = (Classes.Objects.Function)Classes.Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == lstview_components.SelectedItems[0].Text; });
                function = new frm_function(selected_function);
                function.Tag = selected_function.Name;
                function.MdiParent = this;
                function.Show();
                function.BringToFront();
            }
        }

        public void RenameFunctionChildren(string old_name, string new_name)
        {
            foreach (Form frm in this.MdiChildren)
            {
                if (frm is frm_function)
                {
                    if (frm.Tag.ToString() == old_name)
                    {
                        frm.Tag = new_name;
                        frm.Text = new_name;
                    }
                }
            }
        }

        public void CloseFunctionChildren(string function_name)
        {
            foreach (Form frm in this.MdiChildren)
            {
                if (frm is Forms.frm_function)
                {
                    if (frm.Tag.ToString() == function_name)
                    {
                        frm.Close();
                    }
                }
            }
        }

        #endregion

        #region Pass Functions

        private void CheckPassSettings(object sender, CancelEventArgs e)
        {
            Classes.Objects.Pass current_pass = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == lstview_components.SelectedItems[0].Text; });

            if (Classes.Variables.my_effect.Passes.Count > 1)
            {
                tsitem_delete_pass.Enabled = true;
            }
            else
            {
                tsitem_delete_pass.Enabled = false;
            }

            if (Classes.Variables.shader_type == "pp") { tsitem_vertex_shader.Enabled = false; }
            else
            {
                if (current_pass.VertexLink != "(none)") { tsitem_vertex_shader.Enabled = false; } else { tsitem_vertex_shader.Enabled = true; }
            }
            if (current_pass.PixelLink != "(none)") { tsitem_pixel_shader.Enabled = false; } else { tsitem_pixel_shader.Enabled = true; }
        }

        private void ViewVertexShader(object sender, EventArgs e)
        {
            Forms.frm_shader vertex_shader = (Forms.frm_shader)FindForm("Vertex Shader (" + lstview_components.SelectedItems[0].Text + ")");
            if (vertex_shader != null)
            {
                vertex_shader.BringToFront();
            }
            else
            {
                Classes.Objects.Pass selected_pass = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == lstview_components.SelectedItems[0].Text; });
                selected_pass.VertexShader.Visible = true;
                vertex_shader = new frm_shader(selected_pass.VertexShader);
                vertex_shader.Tag = selected_pass.Name;
                vertex_shader.MdiParent = this;
                vertex_shader.Show();
                vertex_shader.BringToFront();
            }
        }

        private void ViewPixelShader(object sender, EventArgs e)
        {
            Forms.frm_shader pixel_shader = (Forms.frm_shader)FindForm("Pixel Shader (" + lstview_components.SelectedItems[0].Text + ")");
            if (pixel_shader != null)
            {
                pixel_shader.BringToFront();
            }
            else
            {
                Classes.Objects.Pass selected_pass = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == lstview_components.SelectedItems[0].Text; });
                selected_pass.PixelShader.Visible = true;
                pixel_shader = new frm_shader(selected_pass.PixelShader);
                pixel_shader.Tag = selected_pass.Name;
                pixel_shader.MdiParent = this;
                pixel_shader.Show();
                pixel_shader.BringToFront();
            }
        }

        private void ViewRenderStates(object sender, EventArgs e)
        {
            Forms.frm_renderstates states = (Forms.frm_renderstates)FindForm("Render States (" + lstview_components.SelectedItems[0].Text + ")");
            if (states != null)
            {
                states.BringToFront();
            }
            else
            {
                Classes.Objects.Pass selected_pass = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == lstview_components.SelectedItems[0].Text; });
                states = new frm_renderstates(selected_pass.States);
                states.Tag = selected_pass.Name;
                states.MdiParent = this;
                states.Show();
                states.BringToFront();
            }
        }

        public void RenamePassChildren(string old_name, string new_name)
        {
            foreach (Form frm in this.MdiChildren)
            {
                if (frm is frm_shader)
                {
                    if (frm.Tag.ToString() == old_name)
                    {
                        frm.Tag = new_name;
                        frm.Text = ((frm_shader)frm).MyShader.Type +" Shader ("+new_name+")";
                    }
                }

                if (frm is frm_renderstates)
                {
                    if (frm.Tag.ToString() == old_name)
                    {
                        frm.Tag = new_name;
                        frm.Text =  "Render States (" + new_name + ")";
                    }
                }
            }
        }

        public void ClosePassChildren(string pass_name, string type)
        {
            foreach (Form frm in this.MdiChildren)
            {
                if (frm is frm_shader)
                {
                    if (frm.Tag.ToString() == pass_name)
                    {
                        if (type == "Both") { frm.Close(); continue; }

                        if (((frm_shader)frm).MyShader.Type == type) { frm.Close(); } 
                    }
                }

                if (frm is frm_renderstates)
                {
                    if (frm.Tag.ToString() == pass_name)
                    {
                        frm.Close();
                    }
                }
            }
        }

        #endregion

        #region Helper/Validation Functions

        public Form FindForm(string name)
        {
            foreach (Form frm in MdiChildren)
            {
                if (frm.Text == name) { return frm; }
            }
            return null;
        }

        #endregion

        #region Misc Functions

        public void CompileShader(object sender, EventArgs e)
        {
            MessageBox.Show(Classes.Variables.my_effect.GenerateEffect());
            Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.ApplyShader);
        }

        private void ShowAboutDialog(object sender, EventArgs e)
        {
            Forms.frm_about about_dialog = new frm_about();
            about_dialog.ShowDialog();
        }

        private void OpenHelp(object sender, EventArgs e)
        {
            if (File.Exists("help.chm"))
            {
                Process.Start("help.chm");
            }
            else
            {
                MessageBox.Show("The help file could not be located.", "Missing help file", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ShowMaterialDefinition(object sender, EventArgs e)
        {
            Forms.frm_material_definition mat_def = new frm_material_definition();
            mat_def.ShowDialog();
        }  

        #endregion

        #region Edit Functions

        private void SetInclude(object sender, EventArgs e)
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;
            if (item.Checked)
            {
                Classes.Variables.my_effect.Includes.Add(item.Text);
            }
            else
            {
                Classes.Variables.my_effect.Includes.Remove(item.Text);
            }

        }

        private void EditDefines(object sender, EventArgs e)
        {
            Classes.Helper.RuntimeCollectionEditor inputs_collection = new Classes.Helper.RuntimeCollectionEditor();
            inputs_collection.EditCollection(Classes.Variables.my_effect.Defines,new Classes.Helper.ParentCollectionEditor(Classes.Variables.my_effect.Defines.GetType(),"Define Editor"), this);
        }

        private void EditStructs(object sender, EventArgs e)
        {
            Classes.Helper.RuntimeCollectionEditor inputs_collection = new Classes.Helper.RuntimeCollectionEditor();
            inputs_collection.EditCollection(Classes.Variables.my_effect.Structs, new Classes.Helper.ParentCollectionEditor(Classes.Variables.my_effect.Structs.GetType(), "Struct Editor"), this);

            foreach (Form frm in Classes.Variables.main_form.MdiChildren)
            {
                if (frm is Forms.frm_function)
                {
                    ((Forms.frm_function)frm).ReloadReturnTypes();
                }
            }

            #region remove non-existent structs from references

            List<string> struct_names = new List<string>();

            foreach (Classes.Objects.Struct str in Classes.Variables.my_effect.Structs) { struct_names.Add(str.Name); }

            foreach (Classes.Objects.Pass pass in Classes.Variables.my_effect.Passes)
            {
                if (pass.VertexShader != null)
                {
                    for (int i = 0; i < pass.VertexShader.Inputs.Count; i++)
                    {
                        if (!struct_names.Contains(pass.VertexShader.Inputs[i].DataType) && !Classes.Variables.reserved_words.Contains(pass.VertexShader.Inputs[i].DataType)) 
                        { pass.VertexShader.Inputs[i].DataType = "float4"; }
                    }

                    for (int i = 0; i < pass.VertexShader.Outputs.Count; i++)
                    {
                        if (!struct_names.Contains(pass.VertexShader.Outputs[i].DataType) && !Classes.Variables.reserved_words.Contains(pass.VertexShader.Outputs[i].DataType))
                        { pass.VertexShader.Outputs[i].DataType = "float4"; }
                    }
                }

                if (pass.PixelShader != null)
                {
                    for (int i = 0; i < pass.PixelShader.Inputs.Count; i++)
                    {
                        if (!struct_names.Contains(pass.PixelShader.Inputs[i].DataType) && !Classes.Variables.reserved_words.Contains(pass.PixelShader.Inputs[i].DataType))
                        { pass.PixelShader.Inputs[i].DataType = "float4"; }
                    }

                    for (int i = 0; i < pass.PixelShader.Outputs.Count; i++)
                    {
                        if (!struct_names.Contains(pass.PixelShader.Outputs[i].DataType) && !Classes.Variables.reserved_words.Contains(pass.PixelShader.Outputs[i].DataType))
                        { pass.PixelShader.Outputs[i].DataType = "float4"; }
                    }
                }
            }

            #endregion
        }

        private void ViewSkins(object sender, EventArgs e)
        {
            if (tsitem_skins.Checked)
            {
                Classes.Variables.skins.Show(); Classes.Variables.skins.BringToFront();
                Classes.Variables.my_effect.SkinsVisible = true;
            }
            else
            {
                Classes.Variables.skins.Hide();
                Classes.Variables.my_effect.SkinsVisible = false;
            }
        }

        private void ViewSkills(object sender, EventArgs e)
        {
            if (tsitem_skills.Checked)
            {
                Classes.Variables.skills.Show(); Classes.Variables.skills.BringToFront();
                Classes.Variables.my_effect.SkillsVisible = true;
            }
            else
            {
                Classes.Variables.skills.Hide();
                Classes.Variables.my_effect.SkillsVisible = false;
            }
        }

        private void ViewMaterial(object sender, EventArgs e)
        {
            pgr_main.SelectedObject = Classes.Variables.my_effect;
        }

        private void EditOptions(object sender, EventArgs e)
        {
            Forms.frm_options options = new frm_options();
            options.ShowDialog();
        }

        #endregion
 
        #region Menu Functions

        public void ResetObjects()
        {
            foreach (Form frm in this.MdiChildren) { frm.Close(); }
            lstview_components.Items.Clear();
            pgr_main.SelectedObject = null;
            Classes.Variables.objects.Clear();
            foreach (ToolStripMenuItem item in tstrip_includes.DropDownItems)
            {
                item.Checked = false;
            }
        }

        private void SerializeData(FileStream stream)
        {
            Classes.Variables.my_effect.Settings.UpdatePositions();
            Classes.Variables.my_effect.Settings.EmbedSky();
            Classes.Variables.my_effect.Settings.EmbedModel();

            BinaryFormatter bin_formatter = new BinaryFormatter();

            bin_formatter.Serialize(stream, Classes.Variables.shader_type); // type of effect
            bin_formatter.Serialize(stream, Classes.Variables.my_effect); // actual effect
        }

        private void DeserializeData(FileStream stream)
        {
            BinaryFormatter bin_formatter = new BinaryFormatter();
            
            Classes.Variables.shader_type = (string)bin_formatter.Deserialize(stream); // type of effect
            Classes.Variables.my_effect = (Classes.Objects.Effect)bin_formatter.Deserialize(stream); // actual effect
        }

        ///////////////////////////

        private void NewShader(object sender, EventArgs e)
        {
            if (Classes.Functions.IsProjectSaved())
            {
                ResetObjects();
                Classes.Variables.my_effect = new FXED.Classes.Objects.Effect();
                Classes.Variables.current_name = "New Effect";

                Classes.Functions.UpdateCurrentName();
                Classes.Variables.project_changed = false;

                Forms.frm_startup_options show_options = new frm_startup_options();
                show_options.ShowDialog();
            }
        }

        public void LoadShader(string filename)
        {
            if (Classes.Functions.IsProjectSaved())
            {
                ResetObjects();

                FileStream open_stream = new FileStream(filename, FileMode.Open, FileAccess.Read);
                DeserializeData(open_stream);
                open_stream.Close();

                tstrip_components.Enabled = true;
                tstrip_defines.Enabled = true;
                tstrip_includes.Enabled = true;
                tstrip_structs.Enabled = true;
                tstrip_export.Enabled = true;

                LoadComponents();

                Classes.Variables.current_file = filename;
                Classes.Variables.current_name = Path.GetFileName(filename);
                Classes.Variables.project_changed = false;

                Classes.Functions.UpdateCurrentName();
            }
        }

        public void LoadFXShader(string filename)
        {
            if (Classes.Functions.IsProjectSaved())
            {
                Forms.frm_fxfile_options fx_options = new frm_fxfile_options();
                fx_options.ShowDialog();

                ResetObjects();

                Classes.Variables.my_effect = new FXED.Classes.Objects.Effect();
                Classes.Variables.my_effect.UseFxFile = true;
                pgr_main.SelectedObject = Classes.Variables.my_effect;

                Forms.frm_fx_editor editor = new frm_fx_editor(filename);
                editor.MdiParent = this;
                editor.Show();

                EnableMenuItems(true);

                tstrip_components.Enabled = false;
                tstrip_defines.Enabled = false;
                tstrip_includes.Enabled = false;
                tstrip_structs.Enabled = false;
                tstrip_export.Enabled = false;

                #region Add Preview

                Classes.Variables.preview = new frm_preview();
                Classes.Variables.preview.MdiParent = this;
                Classes.Variables.preview.Show();
                Classes.Variables.preview.BringToFront();

                #endregion

                #region Add Skins

                Classes.Variables.skins = new frm_skins();
                Classes.Variables.skins.MdiParent = this;
                Classes.Variables.skins.BringToFront();
                if (Classes.Variables.my_effect.SkinsVisible) { Classes.Variables.skins.Show(); }
                tsitem_skins.Checked = Classes.Variables.my_effect.SkinsVisible;

                #endregion

                #region Add Skills

                Classes.Variables.skills = new frm_skills();
                Classes.Variables.skills.MdiParent = this;
                Classes.Variables.skills.BringToFront();
                if (Classes.Variables.my_effect.SkillsVisible) { Classes.Variables.skills.Show(); }
                tsitem_skills.Checked = Classes.Variables.my_effect.SkillsVisible;

                #endregion

                Classes.Variables.current_file = filename;
                Classes.Variables.current_name = Path.GetFileName(filename);
                Classes.Variables.project_changed = false;

                Classes.Functions.UpdateCurrentName();
            }
        }

        private void OpenShader(object sender, EventArgs e)
        {
            if (dlg_open_effect.ShowDialog() != DialogResult.Cancel)
            {
                if (Path.GetExtension(dlg_open_effect.FileName)==".lcfx")
                {
                    LoadShader(dlg_open_effect.FileName);
                }
                else if(Path.GetExtension(dlg_open_effect.FileName)==".fx")
                {
                    LoadFXShader(dlg_open_effect.FileName);
                }
                Classes.Variables.project_changed = false;
               
            }
        }

        public void SaveProject(object sender, EventArgs e)
        {
            if (!File.Exists(Classes.Variables.current_file)) 
            { 
                SaveProjectAs(null, EventArgs.Empty); 
            }
            else
            {
                FileStream save_stream = new FileStream(Classes.Variables.current_file, FileMode.Create, FileAccess.Write);

                if (Classes.Variables.my_effect.UseFxFile)
                {
                    StreamWriter fx_writer = new StreamWriter(save_stream);
                    fx_writer.Write(Classes.Variables.my_effect.FxCode);
                    fx_writer.Close();
                }
                else
                {
                    Forms.frm_save_options save_options = new frm_save_options();
                    save_options.ShowDialog();
                    SerializeData(save_stream);
                }
                save_stream.Close();

                Classes.Functions.save_return = true;
                Classes.Variables.project_changed = false;

                Classes.Variables.current_name = Path.GetFileName(Classes.Variables.current_file);
                Classes.Functions.UpdateCurrentName();
            }
        }

        private void SaveProjectAs(object sender, EventArgs e)
        {
            if (Classes.Variables.my_effect.UseFxFile)
            {
                dlg_save_effect.Filter = "Microsoft Effect Files|*.fx";
            }
            else
            {
                dlg_save_effect.Filter = "Effect Editor Files|*.lcfx";
            }

            if (dlg_save_effect.ShowDialog() != DialogResult.Cancel)
            {
                FileStream save_stream = new FileStream(dlg_save_effect.FileName, FileMode.Create, FileAccess.Write);

                if (Classes.Variables.my_effect.UseFxFile)
                {
                    StreamWriter fx_writer = new StreamWriter(save_stream);
                    fx_writer.Write(Classes.Variables.my_effect.FxCode);
                    fx_writer.Close();
                }
                else
                {
                    Forms.frm_save_options save_options = new frm_save_options();
                    save_options.ShowDialog();
                    SerializeData(save_stream);
                }

                save_stream.Close();

                Classes.Variables.current_file = dlg_save_effect.FileName;
                Classes.Variables.current_name = Path.GetFileName(dlg_save_effect.FileName);
                Classes.Variables.project_changed = false;

                Classes.Functions.UpdateCurrentName();
                Classes.Functions.save_return = true;
            }
            else
            {
                Classes.Functions.save_return = false;
            }
        }

        private void ExportToFxFile(object sender, EventArgs e)
        {
            if (dlg_export_fx.ShowDialog() != DialogResult.Cancel)
            {
                FileStream fs = new FileStream(dlg_export_fx.FileName, FileMode.Create, FileAccess.Write);
                StreamWriter writer = new StreamWriter(fs);
                writer.Write(Classes.Variables.my_effect.GenerateEffect());
                writer.Close();
                fs.Close();
            }
        }

        private void Exit(object sender, EventArgs e)
        {
            if (Classes.Functions.IsProjectSaved())
            {
                ResetObjects();
                Application.Exit();
            }
        }

        void frm_main_Closing(object sender, CancelEventArgs e)
        {
            if (Classes.Functions.IsProjectSaved())
            {
                Application.Exit();
            }
            else
            {
                e.Cancel = true;
            }
        }

        public void EnableMenuItems(bool value)
        {
            tstrip_save.Enabled = value;
            tstrip_save_as.Enabled = value;
            tstrip_export.Enabled = value;
            tstrip_edit.Enabled = value;
            tstrip_components.Enabled = value;
            tstrip_effect.Enabled = value;
        }


        #endregion
  
    }
}
