﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.Runtime.Serialization.Formatters.Binary;
using AcknexWrapper;
using DevIL;

namespace FXED.Forms
{
    public partial class frm_preview : Form
    {
        #region WinAPI constants / functions

        const int WS_VISIBLE = 0x10000000;
        const UInt32 SWP_SHOWWINDOW = 0x0040;

        [DllImport("user32.dll")]
        static extern IntPtr SetParent(IntPtr hWndChild, IntPtr hWndNewParent);

        [DllImport("user32.dll")]
        static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int X, int Y, int cx, int cy, uint uFlags);

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);

        #endregion

        #region Engine Initialization

        private void StartEngine() // open the acknex engine
        {
            EngFun.engine_open("-nc -nwnd");

            // load d3d settings

            EngVar.d3d_lightres.IntValue = Properties.Settings.Default.LightResolutionLevel;
            EngVar.d3d_anisotropy.IntValue = Properties.Settings.Default.AnistrophicLevel;
            switch (Properties.Settings.Default.AntialiasLevel)
            {
                case "None": EngVar.d3d_antialias.IntValue = 0; break;
                default: EngVar.d3d_antialias.IntValue = Convert.ToInt32(Properties.Settings.Default.AntialiasLevel); break;
            }

            EngFun.video_window(null, null, (Var)256, null);
            EngVar.screen_size.x.IntValue = this.tsc.ContentPanel.Width;
            EngVar.screen_size.y.IntValue = this.tsc.ContentPanel.Height;
            EngFun.level_load(null);
            EngVar.sky_color.red.IntValue = 0;
            EngVar.sky_color.green.IntValue = 0;
            EngVar.sky_color.blue.IntValue = 0;
            EngFun.engine_frame(); // wait one frame to allow engine variables to be accessible
            EmbedWindow(); // embeds the engine into the form
            Scheduler.StartScheduler(MainFunction); // start the scheduler
        }

        private delegate void EmbedWindowDelegate(); // delegate needed due to cross-thread calling

        public void EmbedWindow() // calls the Win32 API functions to embed the engine window
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new EmbedWindowDelegate(this.EmbedWindow));
            }
            else
            {
                try
                {
                    SetParent(EngVar.hWnd, this.tsc.ContentPanel.Handle);
                    SetWindowLong(EngVar.hWnd, -16, WS_VISIBLE);
                    SetWindowPos(EngVar.hWnd, (IntPtr)0, 0, 0, this.tsc.ContentPanel.Width, this.tsc.ContentPanel.Height, SWP_SHOWWINDOW);
                }
                catch (Exception) { EngFun.engine_close();  Application.Exit(); }
            }
        }

        #endregion

        public frm_preview()
        {
            InitializeComponent();
            this.Closing += new CancelEventHandler(frm_preview_Closing);
            this.Size = Classes.Variables.my_effect.PreviewSize;
            this.Location = Classes.Variables.my_effect.PreviewLocation;
            this.LocationChanged += new EventHandler(FormMoved);
            this.ResizeEnd += new EventHandler(FormResized);

            if (!Directory.Exists(Application.StartupPath + "\\Media\\Models"))
            {
                tsitem_media.Enabled = false;
            }
            else
            {
                DirectoryInfo model_folder = new DirectoryInfo(Application.StartupPath + "\\Media\\Models");
                foreach (FileInfo file in model_folder.GetFiles())
                {
                    if (file.Extension.ToLower() == ".mdl" || file.Extension.ToLower() == ".hmp")
                    {
                        ToolStripMenuItem new_model = new ToolStripMenuItem(file.Name);
                        new_model.Click += new EventHandler(LoadMediaModel);
                        tsitem_media.DropDownItems.Add(new_model);
                    }
                }

                if (tsitem_media.DropDownItems.Count == 0)
                {
                    tsitem_media.Enabled = false;
                }
            }
        }

        void frm_preview_Closing(object sender, CancelEventArgs e)
        {
            Scheduler.ResetScheduler();
            EngFun.engine_close();
        }

        private void FormShown(object sender, EventArgs e)
        {
            Thread EngineThread = new Thread(StartEngine);
            EngineThread.Start();
        }

        // Engine Variables / Functions

        ENTITY dummy_camera = null;
        Vector nullvector = new Vector(0,0,0);
        ENTITY entity = null;
        ENTITY sky = null;
        MATERIAL mat = null;
        MATERIAL light_mat = null;
        VIEW pp_stage = null;

        ENTITY x_arrow, y_arrow, z_arrow;
        List<ENTITY> lights = new List<ENTITY>();
        ENTITY selected_entity;
        bool allow_drag = true;

        public ENTITY Entity
        {
            get { return entity; }
        }

        public IEnumerable MainFunction()
        {
            yield return Scheduler.PROC_MODE.PROC_EARLIEST;
            // setup some basic properties

            EngVar.mouse_mode.IntValue = 4;
            EngVar.mouse_range.IntValue = 50000;
            EngVar.mouse_pointer.IntValue = 2;
            EngVar.camera.clip_near.FloatValue = 0.0001f;

            EngVar.on_del = DeleteLight;

            // create materials / views / entities

            mat = EngFun.mtl_create();
            yield return Scheduler.PROC_MODE.PROC_EARLIEST;
            light_mat = EngFun.mtl_create();
            yield return Scheduler.PROC_MODE.PROC_EARLIEST;
            pp_stage = EngFun.view_create(0);
            yield return Scheduler.PROC_MODE.PROC_EARLIEST;
            EngFun.effect_load(mat, Classes.Variables.my_effect.GenerateEffect());
            yield return Scheduler.PROC_MODE.PROC_EARLIEST;
            EngFun.effect_load(light_mat, Properties.Settings.Default.WireFrameEffect);

            #region Load model

            if (Classes.Variables.my_effect.Settings.EmbeddedModel != null)
            {
                FileStream fs = new FileStream(Path.GetTempPath() + "\\" + Path.GetFileName(Classes.Variables.my_effect.Settings.ModelFile), FileMode.Create, FileAccess.Write);
                BinaryWriter writer = new BinaryWriter(fs);
                writer.Write(Classes.Variables.my_effect.Settings.EmbeddedModel);
                writer.Close();
                fs.Close();

                entity = EngFun.ent_create(Path.GetTempPath() + "\\" + Path.GetFileName(Classes.Variables.my_effect.Settings.ModelFile), nullvector, null);
                File.Delete(Path.GetTempPath() + "\\" + Path.GetFileName(Classes.Variables.my_effect.Settings.ModelFile));
            }
            else
            {
                if (File.Exists(Classes.Variables.my_effect.Settings.ModelFile))
                {
                    entity = EngFun.ent_create(Classes.Variables.my_effect.Settings.ModelFile, nullvector, null);
                }
                else
                {
                    entity = EngFun.ent_create("Data\\sphere.mdl", nullvector, null);
                }
            }

            #endregion

            #region Load attached skins

            if (Classes.Variables.my_effect.Settings.EntSkin1 == null)
            {
                if (EngFun.ent_getskin(entity, 1) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 1), "skin1.tga");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin1.tga"), 1);
                    File.Delete("skin1.tga");
                }
            }

            if (Classes.Variables.my_effect.Settings.EntSkin2 == null)
            {
                if (EngFun.ent_getskin(entity, 2) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 2), "skin2.tga");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin2.tga"), 2);
                    File.Delete("skin2.tga");
                }
            }

            if (Classes.Variables.my_effect.Settings.EntSkin3 == null)
            {
                if (EngFun.ent_getskin(entity, 3) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 3), "skin3.tga");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin3.tga"), 3);
                    File.Delete("skin3.tga");
                }
            }

            if (Classes.Variables.my_effect.Settings.EntSkin4 == null)
            {
                if (EngFun.ent_getskin(entity, 4) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 4), "skin4.tga");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin4.tga"), 4);
                    File.Delete("skin4.tga");
                }
            }

            #endregion

            x_arrow = EngFun.ent_create("Data\\arrow_x.mdl", new Vector(0, 0, 0), Arrow);
            x_arrow.string1 = "x";
            x_arrow.ZNEAR = true;
            x_arrow.INVISIBLE = true;

            y_arrow = EngFun.ent_create("Data\\arrow_y.mdl", new Vector(0, 0, 0), Arrow);
            y_arrow.string1 = "y";
            y_arrow.ZNEAR = true;
            y_arrow.INVISIBLE = true;

            z_arrow = EngFun.ent_create("Data\\arrow_z.mdl", new Vector(0, 0, 0), Arrow);
            z_arrow.string1 = "z";
            z_arrow.ZNEAR = true;
            z_arrow.INVISIBLE = true;

            #region load material skins

            if (Classes.Variables.my_effect.MtlSkin1 != null)
            {
                if (Classes.Variables.my_effect.MtlSkin1Format == ".tga" || Classes.Variables.my_effect.MtlSkin1Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin1" + Classes.Variables.my_effect.MtlSkin1Format, Classes.Variables.my_effect.MtlSkin1);
                }
                else
                {
                    Classes.Variables.my_effect.MtlSkin1.Save("skin1" + Classes.Variables.my_effect.MtlSkin1Format);
                }
                mat.skin1 = EngFun.bmap_create("skin1" + Classes.Variables.my_effect.MtlSkin1Format);
                File.Delete("skin1" + Classes.Variables.my_effect.MtlSkin1Format);
            }

            if (Classes.Variables.my_effect.MtlSkin2 != null)
            {
                if (Classes.Variables.my_effect.MtlSkin2Format == ".tga" || Classes.Variables.my_effect.MtlSkin2Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin2" + Classes.Variables.my_effect.MtlSkin2Format, Classes.Variables.my_effect.MtlSkin2);
                }
                else
                {
                    Classes.Variables.my_effect.MtlSkin2.Save("skin2" + Classes.Variables.my_effect.MtlSkin2Format);
                }
                mat.skin2 = EngFun.bmap_create("skin2" + Classes.Variables.my_effect.MtlSkin2Format);
                File.Delete("skin2" + Classes.Variables.my_effect.MtlSkin2Format);
            }

            if (Classes.Variables.my_effect.MtlSkin3 != null)
            {
                if (Classes.Variables.my_effect.MtlSkin3Format == ".tga" || Classes.Variables.my_effect.MtlSkin3Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin3" + Classes.Variables.my_effect.MtlSkin3Format, Classes.Variables.my_effect.MtlSkin3);
                }
                else
                {
                    Classes.Variables.my_effect.MtlSkin3.Save("skin3" + Classes.Variables.my_effect.MtlSkin3Format);
                }
                mat.skin3 = EngFun.bmap_create("skin3" + Classes.Variables.my_effect.MtlSkin3Format);
                File.Delete("skin3" + Classes.Variables.my_effect.MtlSkin3Format);
            }

            if (Classes.Variables.my_effect.MtlSkin4 != null)
            {
                if (Classes.Variables.my_effect.MtlSkin4Format == ".tga" || Classes.Variables.my_effect.MtlSkin4Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin4" + Classes.Variables.my_effect.MtlSkin4Format, Classes.Variables.my_effect.MtlSkin4);
                }
                else
                {
                    Classes.Variables.my_effect.MtlSkin4.Save("skin4" + Classes.Variables.my_effect.MtlSkin4Format);
                }
                mat.skin4 = EngFun.bmap_create("skin4" + Classes.Variables.my_effect.MtlSkin4Format);
                File.Delete("skin4" + Classes.Variables.my_effect.MtlSkin4Format);
            }

            #endregion

            #region load entity skins

            if (Classes.Variables.my_effect.Settings.EntSkin1 != null)
            {
                if (Classes.Variables.my_effect.Settings.EntSkin1Format == ".tga" || Classes.Variables.my_effect.Settings.EntSkin1Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin1" + Classes.Variables.my_effect.Settings.EntSkin1Format, Classes.Variables.my_effect.Settings.EntSkin1);
                }
                else
                {
                    Classes.Variables.my_effect.Settings.EntSkin1.Save("skin1" + Classes.Variables.my_effect.Settings.EntSkin1Format);
                }
                EngFun.ent_setskin(entity, EngFun.bmap_create("skin1" + Classes.Variables.my_effect.Settings.EntSkin1Format), 1);
                File.Delete("skin1" + Classes.Variables.my_effect.Settings.EntSkin1Format);
            }

            if (Classes.Variables.my_effect.Settings.EntSkin2 != null)
            {
                if (Classes.Variables.my_effect.Settings.EntSkin2Format == ".tga" || Classes.Variables.my_effect.Settings.EntSkin2Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin2" + Classes.Variables.my_effect.Settings.EntSkin2Format, Classes.Variables.my_effect.Settings.EntSkin2);
                }
                else
                {
                    Classes.Variables.my_effect.Settings.EntSkin2.Save("skin2" + Classes.Variables.my_effect.Settings.EntSkin2Format);
                }
                EngFun.ent_setskin(entity, EngFun.bmap_create("skin2" + Classes.Variables.my_effect.Settings.EntSkin2Format), 2);
                File.Delete("skin2" + Classes.Variables.my_effect.Settings.EntSkin2Format);
            }

            if (Classes.Variables.my_effect.Settings.EntSkin3 != null)
            {
                if (Classes.Variables.my_effect.Settings.EntSkin3Format == ".tga" || Classes.Variables.my_effect.Settings.EntSkin3Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin3" + Classes.Variables.my_effect.Settings.EntSkin3Format, Classes.Variables.my_effect.Settings.EntSkin3);
                }
                else
                {
                    Classes.Variables.my_effect.Settings.EntSkin3.Save("skin3" + Classes.Variables.my_effect.Settings.EntSkin3Format);
                }
                EngFun.ent_setskin(entity, EngFun.bmap_create("skin3" + Classes.Variables.my_effect.Settings.EntSkin3Format), 3);
                File.Delete("skin3" + Classes.Variables.my_effect.Settings.EntSkin3Format);
            }

            if (Classes.Variables.my_effect.Settings.EntSkin4 != null)
            {
                if (Classes.Variables.my_effect.Settings.EntSkin4Format == ".tga" || Classes.Variables.my_effect.Settings.EntSkin4Format == ".dds")
                {
                    DevIL.DevIL.SaveBitmap("skin4" + Classes.Variables.my_effect.Settings.EntSkin4Format, Classes.Variables.my_effect.Settings.EntSkin4);
                }
                else
                {
                    Classes.Variables.my_effect.Settings.EntSkin4.Save("skin4" + Classes.Variables.my_effect.Settings.EntSkin4Format);
                }
                EngFun.ent_setskin(entity, EngFun.bmap_create("skin4" + Classes.Variables.my_effect.Settings.EntSkin4Format), 4);
                File.Delete("skin4" + Classes.Variables.my_effect.Settings.EntSkin4Format);
            }

            #endregion

            #region set skill values

            mat.skill1 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill1.Value).FloatValue;
            mat.skill2 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill2.Value).FloatValue;
            mat.skill3 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill3.Value).FloatValue;
            mat.skill4 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill4.Value).FloatValue;
            mat.skill5 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill5.Value).FloatValue;
            mat.skill6 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill6.Value).FloatValue;
            mat.skill7 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill7.Value).FloatValue;
            mat.skill8 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill8.Value).FloatValue;
            mat.skill9 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill9.Value).FloatValue;
            mat.skill10 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill10.Value).FloatValue;
            mat.skill11 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill11.Value).FloatValue;
            mat.skill12 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill12.Value).FloatValue;
            mat.skill13 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill13.Value).FloatValue;
            mat.skill14 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill14.Value).FloatValue;
            mat.skill15 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill15.Value).FloatValue;
            mat.skill16 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill16.Value).FloatValue;
            mat.skill17 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill17.Value).FloatValue;
            mat.skill18 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill18.Value).FloatValue;
            mat.skill19 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill19.Value).FloatValue;
            mat.skill20 = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill20.Value).FloatValue;
            entity.skill41.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill41.Value).FloatValue;
            entity.skill42.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill42.Value).FloatValue;
            entity.skill43.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill43.Value).FloatValue;
            entity.skill44.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill44.Value).FloatValue;
            entity.skill45.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill45.Value).FloatValue;
            entity.skill46.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill46.Value).FloatValue;
            entity.skill47.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill47.Value).FloatValue;
            entity.skill48.FloatValue = EngFun.floatv(Classes.Variables.my_effect.Skills.Skill48.Value).FloatValue;
            
            #endregion

            #region load sky settings

            if (Classes.Variables.my_effect.Settings.UseSkyBox)
            {
                if (Classes.Variables.my_effect.Settings.EmbeddedSky != null)
                {
                    if (Path.GetExtension(Classes.Variables.my_effect.Settings.SkyImage) == ".tga" || Path.GetExtension(Classes.Variables.my_effect.Settings.SkyImage) == ".dds")
                    {
                        DevIL.DevIL.SaveBitmap(Path.GetTempPath()+"\\"+Path.GetFileName(Classes.Variables.my_effect.Settings.SkyImage), Classes.Variables.my_effect.Settings.EmbeddedSky);
                    }
                    else
                    {
                        Classes.Variables.my_effect.Settings.EmbeddedSky.Save(Path.GetTempPath() + "\\" + Path.GetFileName(Classes.Variables.my_effect.Settings.SkyImage));
                    }

                    sky = EngFun.ent_createlayer(Path.GetTempPath() + "\\" + Path.GetFileName(Classes.Variables.my_effect.Settings.SkyImage), (Flags.SKY | Flags.CUBE), 0);
                    sky.SHOW = true;
                    File.Delete(Path.GetTempPath() + "\\" + Path.GetFileName(Classes.Variables.my_effect.Settings.SkyImage));
                }
                else
                {
                    if (File.Exists(Classes.Variables.my_effect.Settings.SkyImage))
                    {
                        sky = EngFun.ent_createlayer(Classes.Variables.my_effect.Settings.SkyImage, (Flags.SKY | Flags.CUBE), 0);
                        sky.SHOW = true;
                    }
                }
            }
            else
            {
                EngVar.sky_color.red.IntValue = Classes.Variables.my_effect.Settings.SkyColor.R;
                EngVar.sky_color.green.IntValue = Classes.Variables.my_effect.Settings.SkyColor.G;
                EngVar.sky_color.blue.IntValue = Classes.Variables.my_effect.Settings.SkyColor.B;
            }

            #endregion

            #region load sun settings

            EngVar.sun_light.IntValue = Classes.Variables.my_effect.Settings.SunBrightness;
            EngVar.sun_angle.pan.IntValue = Classes.Variables.my_effect.Settings.SunAzimuth;
            EngVar.sun_angle.tilt.IntValue = Classes.Variables.my_effect.Settings.SunElevation;
            EngVar.sun_color.red.IntValue = Classes.Variables.my_effect.Settings.SunColor.R;
            EngVar.sun_color.green.IntValue = Classes.Variables.my_effect.Settings.SunColor.G;
            EngVar.sun_color.blue.IntValue = Classes.Variables.my_effect.Settings.SunColor.B;

            #endregion

            #region add lights

            foreach (Classes.Objects.Light light in Classes.Variables.my_effect.Settings.Lights)
            {
                ENTITY new_light = EngFun.ent_create("Data\\light.mdl", new Vector(light.X, light.Y, light.Z), Light);
                new_light.material = light_mat;
                new_light.ZNEAR = true;
                new_light.skill1.IntValue = 0;
                new_light.string1 = light.Name;
                new_light.lightrange.IntValue = light.Range;

                EngFun.vec_set(new_light.greenBlueRed, new Vector(light.Color.B, light.Color.G, light.Color.R));
                EngFun.vec_fill(new_light.scaleXYZ, 0.15f);
                EngFun.c_setminmax(new_light);
                lights.Add(new_light);
            }

            #endregion

            #region load position settings

            dummy_camera = EngFun.ent_create(null, nullvector, null);

            dummy_camera.x.IntValue = Classes.Variables.my_effect.Settings.CameraX;
            dummy_camera.y.IntValue = Classes.Variables.my_effect.Settings.CameraY;
            dummy_camera.z.IntValue = Classes.Variables.my_effect.Settings.CameraZ;
            dummy_camera.pan.IntValue = Classes.Variables.my_effect.Settings.CameraPan;
            dummy_camera.tilt.IntValue = Classes.Variables.my_effect.Settings.CameraTilt;
            dummy_camera.roll.IntValue = Classes.Variables.my_effect.Settings.CameraRoll;

            entity.pan.IntValue = Classes.Variables.my_effect.Settings.EntityPan;
            entity.tilt.IntValue = Classes.Variables.my_effect.Settings.EntityTilt;
            entity.roll.IntValue = Classes.Variables.my_effect.Settings.EntityRoll;

            #endregion

            #region load material settings

            mat.ambient_red.IntValue = Classes.Variables.my_effect.AmbientColor.R;
            mat.ambient_green.IntValue = Classes.Variables.my_effect.AmbientColor.G;
            mat.ambient_blue.IntValue = Classes.Variables.my_effect.AmbientColor.B;

            mat.diffuse_red.IntValue = Classes.Variables.my_effect.DiffuseColor.R;
            mat.diffuse_green.IntValue = Classes.Variables.my_effect.DiffuseColor.G;
            mat.diffuse_blue.IntValue = Classes.Variables.my_effect.DiffuseColor.B;

            mat.specular_red.IntValue = Classes.Variables.my_effect.SpecularColor.R;
            mat.specular_green.IntValue = Classes.Variables.my_effect.SpecularColor.G;
            mat.specular_blue.IntValue = Classes.Variables.my_effect.SpecularColor.B;

            mat.emissive_red.IntValue = Classes.Variables.my_effect.EmissiveColor.R;
            mat.emissive_green.IntValue = Classes.Variables.my_effect.EmissiveColor.G;
            mat.emissive_blue.IntValue = Classes.Variables.my_effect.EmissiveColor.B;

            mat.power.IntValue = Classes.Variables.my_effect.Power;
            mat.alpha.IntValue = Classes.Variables.my_effect.Alpha;
            mat.albedo.IntValue = Classes.Variables.my_effect.Albedo;
            mat.cycle.IntValue = Classes.Variables.my_effect.Cycle;
            mat.lod.IntValue = Classes.Variables.my_effect.LOD;

            //mat.TANGENT = Classes.Variables.my_effect.Tangent;
            mat.TRANSLUCENT = Classes.Variables.my_effect.Translucent;
            mat.PASS_SOLID = Classes.Variables.my_effect.PassSolid;

            entity.SHOW = true;
            entity.UNTOUCHABLE = true;

            #endregion

            // determine the shader type (entity / pp)

            if (Classes.Variables.shader_type == "entity")
            {
                entity.material = mat;
            }
            else
            {
                pp_stage.material = mat;
                pp_stage.PROCESS_TARGET = true;
                EngVar.camera.stage = pp_stage;
            }

            // create basic navigation

            int orbit_speed = 15;
            int move_speed = 5;

            while (true)
            {
                EngFun.c_move(dummy_camera, new Vector(move_speed * (EngVar.key_w - EngVar.key_s) * EngVar.time_step.FloatValue, 3 * (EngVar.key_a - EngVar.key_d) * EngVar.time_step.FloatValue, 0), nullvector, 0);
                dummy_camera.pan.FloatValue -= 2 * move_speed * EngVar.mouse_right.IntValue * EngVar.mouse_force.x * EngVar.time_step.FloatValue;
                dummy_camera.tilt.FloatValue += 2 * move_speed * EngVar.mouse_right.IntValue * EngVar.mouse_force.y * EngVar.time_step.FloatValue;

                if (allow_drag)
                {
                    entity.pan.FloatValue += orbit_speed * EngVar.mouse_left.IntValue * EngVar.mouse_force.x.IntValue * EngVar.time_step.FloatValue;
                    entity.tilt.FloatValue -= orbit_speed * EngVar.mouse_left.IntValue * EngVar.mouse_force.y.IntValue * EngVar.time_step.FloatValue;
                }

                EngFun.vec_set(EngVar.camera.xyz, dummy_camera.xyz);
                EngFun.vec_set(EngVar.camera.panTiltRoll, dummy_camera.panTiltRoll);

                yield return 1;
            }
        }

        private IEnumerable ResizeEngine()
        {
            yield return 1;
            EngFun.video_window(null, new Vector(this.tsc.ContentPanel.Width, this.tsc.ContentPanel.Height, 0), 0, null);
            EngFun.video_set(this.tsc.ContentPanel.Width, this.tsc.ContentPanel.Height, 0, 0);
        }

        public IEnumerable ApplyShader()
        {
            yield return Scheduler.PROC_MODE.PROC_EARLIEST;
            EngFun.effect_load(mat, Classes.Variables.my_effect.GenerateEffect());
        }

        #region Material Functions

        public IEnumerable SetAmbientColor()
        {
            yield return 1;
            mat.ambient_red.IntValue = Classes.Variables.my_effect.AmbientColor.R;
            mat.ambient_green.IntValue = Classes.Variables.my_effect.AmbientColor.G;
            mat.ambient_blue.IntValue = Classes.Variables.my_effect.AmbientColor.B;
        }

        public IEnumerable SetDiffuseColor()
        {
            yield return 1;
            mat.diffuse_red.IntValue = Classes.Variables.my_effect.DiffuseColor.R;
            mat.diffuse_green.IntValue = Classes.Variables.my_effect.DiffuseColor.G;
            mat.diffuse_blue.IntValue = Classes.Variables.my_effect.DiffuseColor.B;
        }

        public IEnumerable SetSpecularColor()
        {
            yield return 1;
            mat.specular_red.IntValue = Classes.Variables.my_effect.SpecularColor.R;
            mat.specular_green.IntValue = Classes.Variables.my_effect.SpecularColor.G;
            mat.specular_blue.IntValue = Classes.Variables.my_effect.SpecularColor.B;
        }

        public IEnumerable SetEmissiveColor()
        {
            yield return 1;
            mat.emissive_red.IntValue = Classes.Variables.my_effect.EmissiveColor.R;
            mat.emissive_green.IntValue = Classes.Variables.my_effect.EmissiveColor.G;
            mat.emissive_blue.IntValue = Classes.Variables.my_effect.EmissiveColor.B;
        }

        public IEnumerable SetMaterialPower()
        {
            yield return 1;
            mat.power.IntValue = Classes.Variables.my_effect.Power;
        }

        public IEnumerable SetMaterialAlpha()
        {
            yield return 1;
            entity.alpha.IntValue = Classes.Variables.my_effect.Alpha;
            mat.alpha.IntValue = Classes.Variables.my_effect.Alpha;
        }

        public IEnumerable SetMaterialAlbedo()
        {
            yield return 1;
            mat.albedo.IntValue = Classes.Variables.my_effect.Albedo;
        }

        public IEnumerable SetMaterialCycle()
        {
            yield return 1;
            mat.cycle.IntValue = Classes.Variables.my_effect.Cycle;
        }

        public IEnumerable SetMaterialLOD()
        {
            yield return 1;
            mat.lod.IntValue = Classes.Variables.my_effect.LOD;
        }

        string flag = string.Empty;

        public IEnumerable SetMaterialFlag()
        {
            yield return 1;

            switch (flag)
            {
                case "TANGENT": mat.TANGENT = Classes.Variables.my_effect.Tangent; break;
                case "TRANSLUCENT": mat.TRANSLUCENT = Classes.Variables.my_effect.Translucent; break;
                case "PASS_SOLID": mat.PASS_SOLID = Classes.Variables.my_effect.PassSolid; break;
            }
        }

        public void SetMaterialFlag(string flag)
        {
            this.flag = flag;
            Scheduler.AddEventVoidThreadSafe(SetMaterialFlag);
        }

        #endregion

        #region Skin Functions

        private int skin_number = 0;
        private string skin_image = string.Empty;

        public IEnumerable ChangeMaterialSkin()
        {
            yield return 1;
            switch (skin_number)
            {
                case 1:
                    try
                    {
                        if (mat.skin1 != null) { EngFun.ptr_remove(mat.skin1); mat.skin1 = null; }
                    }
                    catch { }
                    mat.skin1 = EngFun.bmap_create(skin_image);
                    break;

                case 2:
                    try
                    {
                        if (mat.skin2 != null) { EngFun.ptr_remove(mat.skin2); mat.skin2 = null; }
                    }
                    catch { }
                    mat.skin2 = EngFun.bmap_create(skin_image);
                    break;

                case 3:
                    try
                    {
                        if (mat.skin3 != null) { EngFun.ptr_remove(mat.skin3); mat.skin3 = null; }
                    }
                    catch { }
                    mat.skin3 = EngFun.bmap_create(skin_image);
                    break;

                case 4:
                    try
                    {
                        if (mat.skin4 != null) { EngFun.ptr_remove(mat.skin4); mat.skin4 = null; }
                    }
                    catch { }
                    mat.skin4 = EngFun.bmap_create(skin_image);
                    break;
            }
        }

        public IEnumerable ChangeEntitySkin()
        {
            yield return 1;

            try
            {
                if (EngFun.ent_getskin(entity, skin_number) != null) { EngFun.ptr_remove(EngFun.ent_getskin(entity, skin_number)); }
            }
            catch { }
            EngFun.ent_setskin(entity, EngFun.bmap_create(skin_image), skin_number);
        }

        public IEnumerable RemoveMaterialSkin()
        {
            yield return 1;
            try
            {
                switch (skin_number)
                {
                    case 1: EngFun.ptr_remove(mat.skin1); mat.skin1 = null; break;
                    case 2: EngFun.ptr_remove(mat.skin2); mat.skin2 = null; break;
                    case 3: EngFun.ptr_remove(mat.skin3); mat.skin3 = null; break;
                    case 4: EngFun.ptr_remove(mat.skin4); mat.skin4 = null; break;
                }
            }
            catch { }
        }

        public IEnumerable RemoveEntitySkin()
        {
            yield return 1;
            try
            {
                if (EngFun.ent_getskin(entity, skin_number) != null)
                {
                    EngFun.ptr_remove(EngFun.ent_getskin(entity, skin_number));
                }
            }
            catch { }
        }

        public void SetMaterialSkin(string image, int number)
        {
            this.skin_image = image;
            this.skin_number = number;
            Scheduler.AddEventVoidThreadSafe(ChangeMaterialSkin);
        }

        public void SetEntitySkin(string image, int number)
        {
            this.skin_image = image;
            this.skin_number = number;
            Scheduler.AddEventVoidThreadSafe(ChangeEntitySkin);
        }

        public void ClearMaterialSkin(int number)
        {
            this.skin_number = number;
            Scheduler.AddEventVoidThreadSafe(RemoveMaterialSkin);
        }

        public void ClearEntitySkin(int number)
        {
            this.skin_number = number;
            Scheduler.AddEventVoidThreadSafe(RemoveEntitySkin);
        }

        #endregion

        #region Skill Functions

        int skill = 0;
        float skill_value = 0.0f;
        public void SetSkill(int number, float value)
        {
            skill = number;
            skill_value = value;
            Scheduler.AddEventVoidThreadSafe(SetSkill);
        }

        public IEnumerable SetSkill()
        {
            yield return 1;
            switch (skill)
            {
                case 1: mat.skill1 = EngFun.floatv(skill_value).FloatValue; break;
                case 2: mat.skill2 = EngFun.floatv(skill_value).FloatValue; break;
                case 3: mat.skill3 = EngFun.floatv(skill_value).FloatValue; break;
                case 4: mat.skill4 = EngFun.floatv(skill_value).FloatValue; break;
                case 5: mat.skill5 = EngFun.floatv(skill_value).FloatValue; break;
                case 6: mat.skill6 = EngFun.floatv(skill_value).FloatValue; break;
                case 7: mat.skill7 = EngFun.floatv(skill_value).FloatValue; break;
                case 8: mat.skill8 = EngFun.floatv(skill_value).FloatValue; break;
                case 9: mat.skill9 = EngFun.floatv(skill_value).FloatValue; break;
                case 10: mat.skill10 = EngFun.floatv(skill_value).FloatValue; break;
                case 11: mat.skill11 = EngFun.floatv(skill_value).FloatValue; break;
                case 12: mat.skill12 = EngFun.floatv(skill_value).FloatValue; break;
                case 13: mat.skill13 = EngFun.floatv(skill_value).FloatValue; break;
                case 14: mat.skill14 = EngFun.floatv(skill_value).FloatValue; break;
                case 15: mat.skill15 = EngFun.floatv(skill_value).FloatValue; break;
                case 16: mat.skill16 = EngFun.floatv(skill_value).FloatValue; break;
                case 17: mat.skill17 = EngFun.floatv(skill_value).FloatValue; break;
                case 18: mat.skill18 = EngFun.floatv(skill_value).FloatValue; break;
                case 19: mat.skill19 = EngFun.floatv(skill_value).FloatValue; break;
                case 20: mat.skill20 = EngFun.floatv(skill_value).FloatValue; break;
                case 41: entity.skill41.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
                case 42: entity.skill42.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
                case 43: entity.skill43.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
                case 44: entity.skill44.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
                case 45: entity.skill45.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
                case 46: entity.skill46.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
                case 47: entity.skill47.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
                case 48: entity.skill48.FloatValue = EngFun.floatv(skill_value).FloatValue; break;
            }
        }
        #endregion

        #region Light Functions

        string light_name = string.Empty;
        int light_range = 0;
        float drag_speed = 0.15f;
        float distance = 0.0f;
        Vector temp_vector = new Vector(0, 0, 0);
        Angle temp_angle = new Angle(0, 0, 0);
        Angle rotation_angle = new Angle(-90, 0, 0);

        private void NewLight(object sender, EventArgs e)
        {
            FXED.Classes.Objects.Light new_light = new FXED.Classes.Objects.Light();
            light_name = new_light.Name;
            light_range = new_light.Range;

            Classes.Variables.my_effect.Settings.Lights.Add(new_light);

            Classes.Variables.main_form.pgr_main.SelectedObject = new_light;
            Classes.Functions.ProjectChanged();

            Scheduler.AddEventVoidThreadSafe(AddLight);
        }

        private IEnumerable AddLight()
        {
            ENTITY ent = EngFun.ent_create("Data\\light.mdl", new Vector(0, 0, 0), Light);
            ent.material = light_mat;
            ent.ZNEAR = true;
            ent.skill1.IntValue = 0;
            ent.string1 = light_name;
            ent.lightrange.IntValue = light_range;

            EngFun.vec_set(ent.greenBlueRed, new Vector(0, 255, 255));
            EngFun.vec_fill(ent.scaleXYZ, 0.15f);
            EngFun.c_setminmax(ent);
            lights.Add(ent);

            yield return 1;
        }

        private IEnumerable DeleteLight()
        {
            if (!x_arrow.INVISIBLE)
            {
                EngFun.ent_remove(lights.Find(delegate(ENTITY e) { return e.string1 == light_name; }));
                x_arrow.INVISIBLE = true;
                y_arrow.INVISIBLE = true;
                z_arrow.INVISIBLE = true;
                DeleteLightSafe();
            }
            yield return 1;
        }

        private delegate void VoidDelegate();

        public void SelectLight()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new VoidDelegate(this.SelectLight));
            }
            else
            {
                Classes.Variables.main_form.pgr_main.SelectedObject = Classes.Variables.my_effect.Settings.Lights.Find(delegate(Classes.Objects.Light l) { return l.Name == light_name; });
            }
        }

        public void DeleteLightSafe()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new VoidDelegate(this.DeleteLightSafe));
            }
            else
            {
                Classes.Variables.my_effect.Settings.Lights.RemoveAll(delegate(Classes.Objects.Light l){return l.Name == light_name;});
                Classes.Variables.main_form.pgr_main.SelectedObject = null;
            }
        }

        public void SetLightColor(string name, int r, int g, int b)
        {
            ENTITY light = lights.Find(delegate(ENTITY e) { return e.string1 == name; });
            light.red.IntValue = r;
            light.green.IntValue = g;
            light.blue.IntValue = b;
        }

        public void SetLightRange(string name, int range)
        {
            ENTITY light = lights.Find(delegate(ENTITY e) { return e.string1 == name; });
            light.lightrange.IntValue = range;
        }

        public IEnumerable Arrow(IntPtr p)
        {
            ENTITY me = (ENTITY)p;

            me.ENABLE_TOUCH = true;
            me.ENABLE_RELEASE = true;
            me.ENABLE_CLICK = true;
            me.event_ = ArrowEvent;

            yield return 1;
        }

        public IEnumerable ArrowEvent()
        {
            if (EngVar.event_type.IntValue == (int)EngVar.EVENT_DEFINITIONS.EVENT_TOUCH)
            {
                if (EngVar.mouse_left != 1)
                {
                    EngVar.mouse_pointer.IntValue = 1;
                    EngVar.me.LIGHT = true;
                }
            }
            else if (EngVar.event_type.IntValue == (int)EngVar.EVENT_DEFINITIONS.EVENT_RELEASE)
            {
                ENTITY temp = EngVar.me;
                while (EngVar.mouse_left == 1) { yield return 1; }
                EngVar.mouse_pointer.IntValue = 2;
                temp.LIGHT = false;
            }
            else if (EngVar.event_type.IntValue == (int)EngVar.EVENT_DEFINITIONS.EVENT_CLICK)
            {
                allow_drag = false;
                switch (EngVar.me.string1)
                {
                    case "x":

                        while (EngVar.mouse_left == 1)
                        {
                            EngFun.vec_set(temp_vector, x_arrow.xyz);
                            EngFun.vec_sub(temp_vector, EngVar.camera.xyz);
                            EngFun.vec_to_angle(temp_angle, temp_vector);

                            distance = drag_speed * EngFun.sign(temp_angle.pan) * EngFun.vec_dist(EngVar.camera.xyz, x_arrow.xyz) * EngVar.mouse_force.x * EngVar.time_step.FloatValue;
                            x_arrow.x.FloatValue += distance;
                            selected_entity.x.FloatValue += distance;
                            SetGizmoPosition(selected_entity, 'x');
                            yield return 1;
                        }
                        allow_drag = true;
                        Classes.Variables.my_effect.Settings.Lights.Find(delegate(Classes.Objects.Light l) { return l.Name == light_name; }).UpdatePosition(selected_entity.x.IntValue, selected_entity.y.IntValue, selected_entity.z.IntValue);
                        break;

                    case "y":
                        while (EngVar.mouse_left == 1)
                        {
                            EngFun.vec_set(temp_vector, y_arrow.xyz);
                            EngFun.vec_sub(temp_vector, EngVar.camera.xyz);
                            EngFun.vec_to_angle(temp_angle, temp_vector);

                            EngFun.ang_rotate(temp_angle, rotation_angle);

                            distance = drag_speed * EngFun.sign(temp_angle.pan) * EngFun.vec_dist(EngVar.camera.xyz, y_arrow.xyz) * EngVar.mouse_force.x * EngVar.time_step.FloatValue;
                            y_arrow.y.FloatValue += distance;
                            selected_entity.y.FloatValue += distance;
                            SetGizmoPosition(selected_entity, 'y');
                            yield return 1;
                        }
                        allow_drag = true;
                        Classes.Variables.my_effect.Settings.Lights.Find(delegate(Classes.Objects.Light l) { return l.Name == light_name; }).UpdatePosition(selected_entity.x.IntValue, selected_entity.y.IntValue, selected_entity.z.IntValue);
                        break;

                    case "z":
                        while (EngVar.mouse_left == 1)
                        {
                            distance = drag_speed * EngFun.vec_dist(EngVar.camera.xyz, z_arrow.xyz) * EngVar.mouse_force.y * EngVar.time_step.FloatValue;
                            z_arrow.z.FloatValue += distance;
                            selected_entity.z.FloatValue += distance;
                            SetGizmoPosition(selected_entity, 'z');
                            yield return 1;
                        }
                        allow_drag = true;
                        Classes.Variables.my_effect.Settings.Lights.Find(delegate(Classes.Objects.Light l) { return l.Name == light_name; }).UpdatePosition(selected_entity.x.IntValue, selected_entity.y.IntValue, selected_entity.z.IntValue);
                        break;
                }
            }

            yield return 1;
        }

        public void SetGizmoPosition(ENTITY ent, char? exception)
        {
            if (exception == null)
            {
                EngFun.vec_set(x_arrow.xyz, new Vector(ent.x + ((ent.max_x - ent.min_x) / 2), ent.y, ent.z));
                EngFun.vec_set(y_arrow.xyz, new Vector(ent.x, ent.y + ((ent.max_y - ent.min_y) / 2), ent.z));
                EngFun.vec_set(z_arrow.xyz, new Vector(ent.x, ent.y, ent.z + ((ent.max_y - ent.min_y) / 2)));
                return;
            }

            if (exception != 'x')
            {
                EngFun.vec_set(x_arrow.xyz, new Vector(ent.x + ((ent.max_x - ent.min_x) / 2), ent.y, ent.z));
            }
            if (exception != 'y')
            {
                EngFun.vec_set(y_arrow.xyz, new Vector(ent.x, ent.y + ((ent.max_y - ent.min_y) / 2), ent.z));
            }
            if (exception != 'z')
            {
                EngFun.vec_set(z_arrow.xyz, new Vector(ent.x, ent.y, ent.z + ((ent.max_y - ent.min_y) / 2)));
            }
        }

        public IEnumerable LightEvent()
        {
            if (EngVar.event_type.IntValue == (int)EngVar.EVENT_DEFINITIONS.EVENT_CLICK)
            {
                if (EngVar.me.skill1.IntValue == 0) // switch light on
                {
                    selected_entity = EngVar.me;
                    light_name = EngVar.me.string1;
                    SelectLight();
                    SetGizmoPosition(EngVar.me, null);
                    x_arrow.INVISIBLE = false;
                    y_arrow.INVISIBLE = false;
                    z_arrow.INVISIBLE = false;

                    for (int i = 0; i < lights.Count; i++)
                    {
                        lights[i].skill1.IntValue -= lights[i].skill1.IntValue;
                    }

                    EngVar.me.skill1.IntValue = 1;
                }
                else // switch light off
                {
                    x_arrow.INVISIBLE = true;
                    y_arrow.INVISIBLE = true;
                    z_arrow.INVISIBLE = true;
                    EngVar.me.skill1.IntValue = 0;
                }
            }

            yield return 1;
        }

        public IEnumerable Light(IntPtr p)
        {
            ENTITY me = (ENTITY)p;
            me.ENABLE_CLICK = true;
            me.event_ = LightEvent;

            yield return 1;
        }

        #endregion

        #region Form Functions

        private void FormMoved(object sender, EventArgs e)
        {
            Classes.Variables.my_effect.PreviewLocation = this.Location;
        }

        private void FormResized(object sender, EventArgs e)
        {
            Classes.Variables.my_effect.PreviewSize = this.Size;
            Scheduler.AddEventVoidThreadSafe(ResizeEngine);
        }

        #endregion

        #region Toolbar Functions

        string model_file = string.Empty;
        List<string> model_cache = new List<string>();
        string zoom_mode = string.Empty;

        IEnumerable Home()
        {
            yield return 1;
            EngFun.vec_set(dummy_camera.xyz, new Vector(-100, 0, 0));
            EngFun.vec_set(dummy_camera.panTiltRoll, new Vector(0, 0, 0));
            EngFun.vec_set(entity.panTiltRoll, new Vector(0, 0, 0));
        }

        IEnumerable Zoom()
        {
            yield return 1;
            if (zoom_mode == "in")
            {
                EngFun.c_move(dummy_camera, new Vector(10, 0, 0), nullvector, 0);
            }
            else
            {
                EngFun.c_move(dummy_camera, new Vector(-10, 0, 0), nullvector, 0);
            }
        }
        
        IEnumerable ChangeModel()
        {
            yield return 1;

            BMAP skin1 = EngFun.ent_getskin(entity, 1);
            BMAP skin2 = EngFun.ent_getskin(entity, 2);
            BMAP skin3 = EngFun.ent_getskin(entity, 3);
            BMAP skin4 = EngFun.ent_getskin(entity, 4);

            EngFun.ent_morph(entity, model_file);

            if (model_cache.Contains(model_file))
            {
                EngFun.ent_reload(entity);
            }
            else
            {
                model_cache.Add(model_file);
            }
            
            if (Properties.Settings.Default.ReplaceSkins)
            {
                Classes.Variables.my_effect.Settings.EntSkin1 = null;
                Classes.Variables.my_effect.Settings.EntSkin2 = null;
                Classes.Variables.my_effect.Settings.EntSkin3 = null;
                Classes.Variables.my_effect.Settings.EntSkin4 = null;

                if (EngFun.ent_getskin(entity, 1) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 1), "skin1.png");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin1.png"), 1);
                    File.Delete("skin1.png");
                }
                else
                {
                    Classes.Variables.skins.RemoveSkin(1);
                }

                if (EngFun.ent_getskin(entity, 2) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 2), "skin2.tga");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin2.tga"), 2);
                    File.Delete("skin2.tga");
                }
                else
                {
                    Classes.Variables.skins.RemoveSkin(2);
                }

                if (EngFun.ent_getskin(entity, 3) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 3), "skin3.tga");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin3.tga"), 3);
                    File.Delete("skin3.tga");
                }
                else
                {
                    Classes.Variables.skins.RemoveSkin(3);
                }

                if (EngFun.ent_getskin(entity, 4) != null)
                {
                    EngFun.bmap_save(EngFun.ent_getskin(entity, 4), "skin4.tga");
                    Classes.Variables.skins.ReloadSkin(DevIL.DevIL.LoadBitmap("skin4.tga"), 4);
                    File.Delete("skin4.tga");
                }
                else
                {
                    Classes.Variables.skins.RemoveSkin(4);
                }
            }
            else
            {
                if (skin1 != null) { EngFun.ent_setskin(entity, skin1, 1); }
                if (skin2 != null) { EngFun.ent_setskin(entity, skin2, 2); }
                if (skin3 != null) { EngFun.ent_setskin(entity, skin3, 3); }
                if (skin4 != null) { EngFun.ent_setskin(entity, skin4, 4); }
            }
        }

        public IEnumerable SetSun()
        {
            yield return 1;
            EngVar.sun_light.IntValue = Classes.Variables.my_effect.Settings.SunBrightness;
            EngVar.sun_angle.pan.IntValue = Classes.Variables.my_effect.Settings.SunAzimuth;
            EngVar.sun_angle.tilt.IntValue = Classes.Variables.my_effect.Settings.SunElevation;
            EngVar.sun_color.red.IntValue = Classes.Variables.my_effect.Settings.SunColor.R;
            EngVar.sun_color.green.IntValue = Classes.Variables.my_effect.Settings.SunColor.G;
            EngVar.sun_color.blue.IntValue = Classes.Variables.my_effect.Settings.SunColor.B;
        }

        private void OpenModel(object sender, EventArgs e)
        {
            if (dlg_open_model.ShowDialog() != DialogResult.Cancel)
            {
                model_file = dlg_open_model.FileName;
                Classes.Variables.my_effect.Settings.ModelFile = model_file;
                Scheduler.AddEventVoidThreadSafe(ChangeModel);
            }
        }

        private void LoadMediaModel(object sender, EventArgs e)
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;

            if (!File.Exists(Application.StartupPath + "\\Media\\Models\\" + item.Text))
            {
                if (MessageBox.Show("The selected model / terrain no longer exists.\r\nWould you like to remove it from the list?", "Non-existent model/terrain", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    tsitem_media.DropDownItems.Remove(item);
                }
                return;
            }
            else
            {
                model_file = Application.StartupPath + "\\Media\\Models\\" + item.Text;
                Classes.Variables.my_effect.Settings.ModelFile = model_file;
                Scheduler.AddEventVoidThreadSafe(ChangeModel);
            }
        }

        private void ZoomView(object sender, EventArgs e)
        {
            zoom_mode = ((ToolStripButton)sender).Tag.ToString();
            Scheduler.AddEventVoidThreadSafe(Zoom);
        }

        private void CameraHome(object sender, EventArgs e)
        {
            Scheduler.AddEventVoidThreadSafe(Home);
        }

        private void ShowSunSettings(object sender, EventArgs e)
        {
            frm_sun_settings sun_settings = new frm_sun_settings();
            sun_settings.ShowDialog();
        }

        IEnumerable SetSkyColor()
        {
            yield return 1;
            if (sky != null) { sky.SHOW = false; }
            EngVar.sky_color.red.IntValue = dlg_sky_color.Color.R;
            EngVar.sky_color.green.IntValue = dlg_sky_color.Color.G;
            EngVar.sky_color.blue.IntValue = dlg_sky_color.Color.B;
        }

        IEnumerable LoadSkyCube()
        {
            yield return 1;
            if (sky != null) { EngFun.ptr_remove(sky); }
            sky = EngFun.ent_createlayer(dlg_open_sky.FileName, (Flags.SKY | Flags.CUBE), 0);
            sky.SHOW = true;
            EngFun.vec_set(EngVar.sky_color, new Vector(0, 0, 0));
        }

        private void ChangeSkyColor(object sender, EventArgs e)
        {
            if (dlg_sky_color.ShowDialog() != DialogResult.Cancel)
            {
                Scheduler.AddEventVoidThreadSafe(SetSkyColor);
                Classes.Variables.my_effect.Settings.SkyColor = dlg_sky_color.Color;
                Classes.Variables.my_effect.Settings.UseSkyBox = false;
            }
        }

        private void ChangeSkyImage(object sender, EventArgs e)
        {
            if (dlg_open_sky.ShowDialog() != DialogResult.Cancel)
            {
                Scheduler.AddEventVoidThreadSafe(LoadSkyCube);
                Classes.Variables.my_effect.Settings.SkyImage = dlg_open_sky.FileName;
                Classes.Variables.my_effect.Settings.UseSkyBox = true;
            }
        }

        #endregion

        public IEnumerable ScreenShot()
        {
            EngFun.file_for_screen(Path.GetTempPath()+"\\"+"thumb.png", 123);
            yield return 1;
        }

        public Bitmap GetThumbnail()
        {
            if (Scheduler.EngineClosed) { return new Bitmap(96, 96); }
            Scheduler.AddEventVoidThreadSafe(ScreenShot);
            Thread.Sleep(500);

            Bitmap result = new Bitmap(96, 96);

            try
            {
                using (Graphics g = Graphics.FromImage(result))
                {
                    g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                    g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                    using (FileStream fs = new FileStream(Path.GetTempPath() + "\\thumb123.png", FileMode.Open, FileAccess.Read))
                    {
                        g.DrawImage(new Bitmap(fs), 0, 0, 96, 96);
                        g.Save();
                        fs.Close();
                    }
                }
            }
            catch (Exception) { }
            return result;
        }

    }
}
