﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using DevIL; // Devil library free to use under the GNU Lesser General Public Lisence

namespace FXED.Forms
{
    public partial class frm_skins : Form
    {
        private Button current_button = null;

        public frm_skins()
        {
            InitializeComponent();
            this.Location = Classes.Variables.my_effect.SkinsLocation;
            this.FormClosing += new FormClosingEventHandler(frm_skins_Closing);
            this.LocationChanged += new EventHandler(FormMoved);

            btn_mtlskin1.BackgroundImage = Classes.Variables.my_effect.MtlSkin1;
            btn_mtlskin2.BackgroundImage = Classes.Variables.my_effect.MtlSkin2;
            btn_mtlskin3.BackgroundImage = Classes.Variables.my_effect.MtlSkin3;
            btn_mtlskin4.BackgroundImage = Classes.Variables.my_effect.MtlSkin4;

            btn_entskin1.BackgroundImage = Classes.Variables.my_effect.Settings.EntSkin1;
            btn_entskin2.BackgroundImage = Classes.Variables.my_effect.Settings.EntSkin2;
            btn_entskin3.BackgroundImage = Classes.Variables.my_effect.Settings.EntSkin3;
            btn_entskin4.BackgroundImage = Classes.Variables.my_effect.Settings.EntSkin4;
        }

        void frm_skins_Closing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason != CloseReason.MdiFormClosing)
            {
                this.Hide();
                Classes.Variables.my_effect.SkinsVisible = false;
                Classes.Variables.main_form.tsitem_skins.Checked = false;
                e.Cancel = true;
            }
        }

        private void ShowOptions(object sender, EventArgs e)
        {
            if (((Button)sender).BackgroundImage != null)
            {
                tsitem_clear.Enabled = true;
                tsitem_export.Enabled = true;
            }
            else
            {
                tsitem_clear.Enabled = false;
                tsitem_export.Enabled = false;
            }
            current_button = (Button)sender;
            cms_options.Show(Cursor.Position);
        }

        private delegate void ModifySkinDelegate(Bitmap image, int skin_number); // delegate required due to cross-thread call
        private delegate void RemoveSkinDelegate(int skin_number);

        public void ReloadSkin(Bitmap image,int number)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new ModifySkinDelegate(this.ReloadSkin), image,number);
            }
            else
            {
                switch (number)
                {
                    case 1: btn_entskin1.BackgroundImage = image; break;
                    case 2: btn_entskin2.BackgroundImage = image; break;
                    case 3: btn_entskin3.BackgroundImage = image; break;
                    case 4: btn_entskin4.BackgroundImage = image; break;
                }
            }
        }

        public void RemoveSkin(int number)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new RemoveSkinDelegate(this.RemoveSkin), number);
            }
            else
            {
                switch (number)
                {
                    case 1: btn_entskin1.BackgroundImage = null; break;
                    case 2: btn_entskin2.BackgroundImage = null; break;
                    case 3: btn_entskin3.BackgroundImage = null; break;
                    case 4: btn_entskin4.BackgroundImage = null; break;
                }
            }
        }

        private void LoadSkin(object sender, EventArgs e)
        {
            if (dlg_open_skin.ShowDialog() != DialogResult.Cancel)
            {
                Bitmap skin;
                string ext = Path.GetExtension(dlg_open_skin.FileName).ToLower();

                if (ext == ".tga" || ext == ".dds")
                {
                    skin = DevIL.DevIL.LoadBitmap(dlg_open_skin.FileName);
                }
                else
                {
                    skin = new Bitmap(dlg_open_skin.FileName);
                }

                if (tab_skintypes.SelectedIndex == 0)
                {
                    switch (current_button.Tag.ToString())
                    {
                        case "1": Classes.Variables.my_effect.MtlSkin1 = skin;
                                  Classes.Variables.my_effect.MtlSkin1Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;

                        case "2": Classes.Variables.my_effect.MtlSkin2 = skin;
                                  Classes.Variables.my_effect.MtlSkin2Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;

                        case "3": Classes.Variables.my_effect.MtlSkin3 = skin;
                            Classes.Variables.my_effect.MtlSkin3Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;

                        case "4": Classes.Variables.my_effect.MtlSkin4 = skin;
                            Classes.Variables.my_effect.MtlSkin4Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;
                    }

                    Classes.Variables.preview.SetMaterialSkin(dlg_open_skin.FileName, Convert.ToInt32(current_button.Tag));
                }
                else
                {
                    switch (current_button.Tag.ToString())
                    {
                        case "1": Classes.Variables.my_effect.Settings.EntSkin1 = skin;
                            Classes.Variables.my_effect.Settings.EntSkin1Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;

                        case "2": Classes.Variables.my_effect.Settings.EntSkin2 = skin;
                            Classes.Variables.my_effect.Settings.EntSkin2Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;

                        case "3": Classes.Variables.my_effect.Settings.EntSkin3 = skin;
                            Classes.Variables.my_effect.Settings.EntSkin3Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;

                        case "4": Classes.Variables.my_effect.Settings.EntSkin4 = skin;
                            Classes.Variables.my_effect.Settings.EntSkin4Format = Path.GetExtension(dlg_open_skin.FileName).ToLower(); break;
                    }

                    Classes.Variables.preview.SetEntitySkin(dlg_open_skin.FileName, Convert.ToInt32(current_button.Tag));
                }

                current_button.BackgroundImage = skin;
            }
        }

        private void SkinChanged(object sender, EventArgs e)
        {
            if (((Button)sender).BackgroundImage == null)
            {
                if (((Control)sender).Parent == flw_material)
                {
                    ((Button)sender).Text = "mtlSkin" + ((Button)sender).Tag.ToString();
                }
                else
                {
                    ((Button)sender).Text = "entSkin" + ((Button)sender).Tag.ToString();
                }
            }
            else
            {
                ((Button)sender).Text = string.Empty;
            }

            Classes.Functions.ProjectChanged();

        }

        private void ClearSkin(object sender, EventArgs e)
        {
            if (tab_skintypes.SelectedIndex == 0)
            {
                Classes.Variables.preview.ClearMaterialSkin(Convert.ToInt32(current_button.Tag));

                switch (current_button.Tag.ToString())
                {
                    case "1": Classes.Variables.my_effect.MtlSkin1 = null;  break;
                    case "2": Classes.Variables.my_effect.MtlSkin2 = null; break;
                    case "3": Classes.Variables.my_effect.MtlSkin3 = null; break;
                    case "4": Classes.Variables.my_effect.MtlSkin4 = null; break;
                }
            }
            else
            {
                Classes.Variables.preview.ClearEntitySkin(Convert.ToInt32(current_button.Tag));

                switch (current_button.Tag.ToString())
                {
                    case "1": Classes.Variables.my_effect.Settings.EntSkin1 = null; break;
                    case "2": Classes.Variables.my_effect.Settings.EntSkin2 = null; break;
                    case "3": Classes.Variables.my_effect.Settings.EntSkin3 = null; break;
                    case "4": Classes.Variables.my_effect.Settings.EntSkin4 = null; break;
                }
            }
            current_button.BackgroundImage = null;
        }

        private void FormMoved(object sender, EventArgs e)
        {
            Classes.Variables.my_effect.SkinsLocation = this.Location;
        }

        private void ExportSkin(object sender, EventArgs e)
        {
            if(dlg_export_skin.ShowDialog() != DialogResult.Cancel)
            {
                current_button.BackgroundImage.Save(dlg_export_skin.FileName);
            }
        }
    }
}
